Package: fastei
Title: Methods for ''A Fast Alternative for the R x C Ecological
        Inference Case''
Version: 0.0.19
Authors@R: c(person(given = "Charles", family = "Thraves", role = "aut", email = "cthraves@dii.uchile.cl", comment = c(ORCID = "0000-0001-7491-7477")),
             person(given = "Pablo", family = "Ubilla", role = "aut", comment = c(ORCID = "0009-0003-9048-1767")),
             person(given = "Daniel", family = "Hermosilla", role = c("aut", "cre"), email = "daniel.hermosilla.r@ug.uchile.cl"))
Description: Estimates the probability matrix for the R×C Ecological Inference problem using the Expectation-Maximization Algorithm with four approximation methods for the E-Step, and an exact method as well. It also provides a bootstrap function to estimate the standard deviation of the estimated probabilities. In addition, it has functions that aggregate rows optimally to have more reliable estimates in cases of having few data points. For comparing the probability estimates of two groups, a Wald test routine is implemented. The library has data from the first round of the Chilean Presidential Election 2021 and can also generate synthetic election data. Methods described in Thraves, Charles; Ubilla, Pablo; Hermosilla, Daniel (2024) ''A Fast Ecological Inference Algorithm for the R×C case'' <doi:10.2139/ssrn.4832834>.
License: MIT + file LICENSE
Encoding: UTF-8
Imports: Rcpp, jsonlite
LinkingTo: Rcpp
NeedsCompilation: yes
BugReports: https://github.com/DanielHermosilla/ecological-inference-elections/issues
URL:
        https://danielhermosilla.github.io/ecological-inference-elections/reference/fastei-package.html
Suggests: knitr, rmarkdown, ggplot2, reshape2, viridis, dplyr, qpdf,
        testthat (>= 3.0.0)
VignetteBuilder: knitr
Depends: R (>= 3.5)
LazyData: true
RoxygenNote: 7.3.3
Config/testthat/edition: 3
Packaged: 2026-05-20 14:50:32 UTC; daniel
Author: Charles Thraves [aut] (ORCID: <https://orcid.org/0000-0001-7491-7477>),
  Pablo Ubilla [aut] (ORCID: <https://orcid.org/0009-0003-9048-1767>),
  Daniel Hermosilla [aut, cre]
Maintainer: Daniel Hermosilla <daniel.hermosilla.r@ug.uchile.cl>
Repository: CRAN
Date/Publication: 2026-05-20 16:40:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-25 01:11:48 UTC; unix
Archs: fastei.so.dSYM

Package: fastei
Title: Methods for ''A Fast Alternative for the R x C Ecological
        Inference Case''
Version: 0.0.19
Authors@R: c(person(given = "Charles", family = "Thraves", role = "aut", email = "cthraves@dii.uchile.cl", comment = c(ORCID = "0000-0001-7491-7477")),
             person(given = "Pablo", family = "Ubilla", role = "aut", comment = c(ORCID = "0009-0003-9048-1767")),
             person(given = "Daniel", family = "Hermosilla", role = c("aut", "cre"), email = "daniel.hermosilla.r@ug.uchile.cl"))
Description: Estimates the probability matrix for the R×C Ecological Inference problem using the Expectation-Maximization Algorithm with four approximation methods for the E-Step, and an exact method as well. It also provides a bootstrap function to estimate the standard deviation of the estimated probabilities. In addition, it has functions that aggregate rows optimally to have more reliable estimates in cases of having few data points. For comparing the probability estimates of two groups, a Wald test routine is implemented. The library has data from the first round of the Chilean Presidential Election 2021 and can also generate synthetic election data. Methods described in Thraves, Charles; Ubilla, Pablo; Hermosilla, Daniel (2024) ''A Fast Ecological Inference Algorithm for the R×C case'' <doi:10.2139/ssrn.4832834>.
License: MIT + file LICENSE
Encoding: UTF-8
Imports: Rcpp, jsonlite
LinkingTo: Rcpp
NeedsCompilation: yes
BugReports: https://github.com/DanielHermosilla/ecological-inference-elections/issues
URL:
        https://danielhermosilla.github.io/ecological-inference-elections/reference/fastei-package.html
Suggests: knitr, rmarkdown, ggplot2, reshape2, viridis, dplyr, qpdf,
        testthat (>= 3.0.0)
VignetteBuilder: knitr
Depends: R (>= 3.5)
LazyData: true
RoxygenNote: 7.3.3
Config/testthat/edition: 3
Packaged: 2026-05-20 14:50:32 UTC; daniel
Author: Charles Thraves [aut] (ORCID: <https://orcid.org/0000-0001-7491-7477>),
  Pablo Ubilla [aut] (ORCID: <https://orcid.org/0009-0003-9048-1767>),
  Daniel Hermosilla [aut, cre]
Maintainer: Daniel Hermosilla <daniel.hermosilla.r@ug.uchile.cl>
Repository: CRAN
Date/Publication: 2026-05-20 16:40:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-22 00:14:35 UTC; unix
Archs: fastei.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(as.matrix,eim)
S3method(logLik,eim)
S3method(plot,eim)
S3method(print,eim)
S3method(summary,eim)
export(bootstrap)
export(eim)
export(get_agg_opt)
export(get_agg_proxy)
export(get_eim_chile)
export(run_em)
export(save_eim)
export(simulate_election)
export(waldtest)
import(Rcpp)
importFrom(Rcpp,evalCpp)
importFrom(stats,logLik)
importFrom(stats,pnorm)
importFrom(stats,pt)
importFrom(stats,rgamma)
importFrom(stats,rmultinom)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(utils,modifyList)
importFrom(utils,tail)
importFrom(utils,write.csv)
useDynLib(fastei, .registration = TRUE)

# *Methods for "A Fast Ecological Inference Algorithm for the RxC case".*

<!-- badges: start -->

[![R-CMD-check-Windows](https://img.shields.io/github/actions/workflow/status/DanielHermosilla/ecological-inference-elections/R-CMD-check-windows.yaml?branch=main&label=Windows)](https://github.com/DanielHermosilla/ecological-inference-elections/actions)
[![R-CMD-check-macOS](https://img.shields.io/github/actions/workflow/status/DanielHermosilla/ecological-inference-elections/R-CMD-check-macos.yaml?branch=main&label=macOS)](https://github.com/DanielHermosilla/ecological-inference-elections/actions)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/fastei)](https://CRAN.R-project.org/package=fastei)

<!-- badges: end -->

The following library includes a method (`run_em`) to solve the R×C Ecological Inference problem for the non-parametric case by using the EM algorithm with different approximation methods for the E-Step. Covariates are supported but not required. The standard deviation of the estimated probabilities can be computed using bootstrapping (bootstrap).

It also provides a function that generates synthetic election data (`simulate_election`) and a function that imports real election data (`chilean_election_2021`) from the Chilean first-round presidential election of 2021.

The setting in which the documentation presents the Ecological Inference problem is an election context where for a set of ballot-boxes we observe (i) the votes obtained by each candidate and (ii) the number of voters of each demographic group (for example, these can be defined by age ranges or sex). See [Thraves, C., Ubilla, P., Hermosilla, D. (2024): "A Fast Ecological Inference Algorithm for the R×C Case".](https://papers.ssrn.com/sol3/papers.cfm?abstract_id=4832834).

The methods to compute the conditional probabilities of the E-Step included in this package are the following:

- **Markov Chain Monte Carlo** (`mcmc`): Performs MCMC to sample vote outcomes for each ballot-box consistent with the observed data. This sample is used to estimate the conditional probability of the E-Step.

- **Multivariate Normal PDF** (`mvn_pdf`): Uses the PDF of a Multivariate Normal to approxi- mate the conditional probability.

- **Multivariate Normal CDF** (`mvn_cdf`): Uses the CDF of a Multivariate Normal to approxi- mate the conditional probability.

- **Multinomial** (`mult`): A single Multinomial is used to approximate the sum of Multinomial distributions.

- **Exact** (`exact`): Solves the E-Step exactly using the Total Probability Law, which requires enumerating an exponential number of terms.

On average, the Multinomial method is the most efficient and precise. Its precision matches the Exact method.

The documentation uses the following notation:

- `b`: number of ballot boxes.
- `g`: number of demographic groups.
- `c`: number of candidates.
- `a`: number of aggregated macro-groups.

To learn more about `fastei`, please consult the available vignettes:

```{r}
browseVignettes("fastei")
```

Aditionally, it is possible to browse the full documentation on the [library website](https://danielhermosilla.github.io/ecological-inference-elections/).
### Installation

It can either be installed from source by running `devtools::install_github("DanielHermosilla/ecological-inference-elections)` or by the CRAN repository `install.packages("fastei")`. Fortran support is required, usually, shipped by R itself. 

## -----------------------------------------------------------------------------
library(fastei)

eim_apo <- get_eim_chile(elect_district = "APOQUINDO")

eim_apo

## ----digits=4-----------------------------------------------------------------
eim_apo <- run_em(eim_apo)
round(eim_apo$prob, 4)

## -----------------------------------------------------------------------------
eim_apo <- bootstrap(eim_apo, seed = 42, nboot = 30)
round(eim_apo$sd, 4)

## -----------------------------------------------------------------------------
eim_nav <- get_eim_chile(elect_district = "NAVIDAD")
eim_nav <- bootstrap(eim_nav, seed = 42, nboot = 30)
round(eim_nav$sd, 4)

## ----message=FALSE, warning=FALSE, echo=TRUE----------------------------------
library(ggplot2)
library(reshape2)
library(viridis)

plot_district <- function(matrix1, district1, matrix2, district2, sd = FALSE) {
    value <- ifelse(sd == FALSE, "prob", "sd")
    df1 <- melt(matrix1)
    df2 <- melt(matrix2)
    df1$Matrix <- district1
    df2$Matrix <- district2
    combined_df <- rbind(df1, df2)
    color <- ifelse(value == "prob", "plasma", "viridis")

    # Add text to each cell of the matrix
    combined_df$label <- sprintf("%.3f", combined_df$value)
    combined_df$text_color <- ifelse(combined_df$value > round(max(combined_df$value) * 0.75 + min(combined_df$value) * 0.25, 2), "black", "white")
    districts <- sort(c(district1, district2))

    # Call the plot
    ggplot(combined_df, aes(x = Var2, y = Var1, fill = value)) +
        geom_tile() +
        geom_text(aes(label = label, color = text_color), size = 2.5) +
        scale_fill_viridis(
            name = value,
            option = color,
        ) +
        scale_color_identity() +
        facet_wrap(~Matrix) +
        coord_fixed() +
        theme_bw() +
        labs(
            title = ifelse(value == "prob",
                paste("Estimated probabilities in districts:", districts[1], "and", districts[2]),
                paste("Standard deviation of estimated probabilities in districts:", districts[1], "and", districts[2])
            ),
            x = "Candidates' votes", y = "Voters' age range", fill = value
        )
}

## ----navidad_apoq_sd_comparison, fig.width = 8, fig.height = 6.5, fig.cap = "Navidad and Apoquindo standard deviation comparison", fig.align = "center", message=FALSE, warning=FALSE, results="hide"----
plot_district(
    matrix1 = eim_nav$sd, district1 = "Navidad",
    matrix2 = eim_apo$sd, district2 = "Apoquindo", sd = TRUE
)

## -----------------------------------------------------------------------------
eim_nav_proxy <- get_agg_proxy(eim_nav, seed = 6, sd_threshold = 0.03, sd_statistic = "maximum")
eim_nav_proxy$group_agg

## -----------------------------------------------------------------------------
mean(eim_nav$sd) - mean(eim_nav_proxy$sd)

## -----------------------------------------------------------------------------
plot_matrix <- function(mat, sd = FALSE, y_labels = NULL) {
    # Initial configurations
    if (!sd) mat <- t(mat)
    df <- reshape2::melt(mat)
    colnames(df) <- c("Row", "Column", "Value")
    df$Row <- factor(df$Row, levels = rev(sort(unique(df$Row))))
    df$Column <- factor(df$Column, levels = sort(unique(df$Column)))
    if (!sd) {
        df$Label <- sprintf("%d", df$Value)
        title_text <- "Voters distribution"
        x_lab <- "Ballot Box"
        y_lab <- "Dem. Group"
        fill_lab <- "Voters"
        df$text_color <- ifelse(df$Value > 30, "black", "white")
        option <- "inferno"
        start <- 0.5
        limits <- NULL
    } else {
        df$Label <- sprintf("%.3f", df$Value)
        title_text <- "Standard deviation of estimated probabilities on district: Navidad"
        x_lab <- "Candidates' votes"
        y_lab <- "Voters' age range"
        fill_lab <- "sd"
        df$text_color <- ifelse(df$Value > 0.13, "black", "white")
        option <- "viridis"
        start <- 0
        limits <- c(0, 0.1)
    }

    # Plot
    p <- ggplot(df, aes(x = Column, y = Row, fill = Value)) +
        geom_tile() +
        geom_text(aes(label = Label, color = text_color), size = 3) +
        scale_color_identity() +
        scale_fill_viridis_c(option = option, begin = start, limits = limits) +
        coord_fixed() +
        theme_bw() +
        theme(axis.text.y = element_text(size = 7), axis.text.x = element_text(size = 7)) +
        labs(
            title = title_text,
            x = x_lab,
            y = y_lab,
            fill = fill_lab
        )
    # Add custom y-axis labels if provided
    if (!is.null(y_labels)) {
        p <- p + scale_y_discrete(labels = y_labels)
    }
    p
}

## ----standard_deviation_proxy, fig.width = 8, fig.height = 3.5, fig.cap = "Navidad aggregated standard deviation with proxy method", fig.align = "center", message=FALSE, warning=FALSE, echo=TRUE----
plot_matrix(eim_nav_proxy$sd, sd = TRUE, y_labels = c("X18.49", "X50."))

## -----------------------------------------------------------------------------
eim_nav_opt <- get_agg_opt(eim_nav, seed = 0, sd_threshold = 0.03, sd_statistic = "maximum")
eim_nav_opt$group_agg

## ----standard_deviation_opt, fig.width = 8, fig.height = 3.5, fig.cap = "Navidad aggregated standard deviation with opt method", fig.align = "center", message=FALSE, warning=FALSE----
plot_matrix(eim_nav_opt$sd, sd = TRUE, y_labels = c("X18.29", "X30.49", "X50."))

## ----granja_lobarnechea_comparison, fig.width = 8, fig.height=6.5, fig.cap = "Lo Barnechea and La Granja comparison", fig.align = "center", message=FALSE----
eim_gra <- get_eim_chile("LA GRANJA")
eim_gra <- run_em(eim_gra)
eim_bar <- get_eim_chile("LO BARNECHEA")
eim_bar <- run_em(eim_bar)

plot_district(eim_gra$prob, "La Granja", eim_bar$prob, "Lo Barnechea")

## -----------------------------------------------------------------------------
comparison2 <- waldtest(
    object1 = eim_gra,
    object2 = eim_bar,
    method = "mult",
    nboot = 30,
    seed = 42,
)

round(comparison2$pvals, 3)

## ----half_comparison, fig.width = 8, fig.height=6.5, fig.cap = "Equally sorted districts comparison", fig.align = "center", message=FALSE, warning=FALSE----
eimRM <- get_eim_chile(region = "METROPOLITANA DE SANTIAGO")

n <- nrow(eimRM$X)
n_train <- floor(n * 0.5)
train_indices <- sample(seq_len(n), n_train)

eimRMhalf1 <- eim(X = eimRM$X[train_indices, ], W = eimRM$W[train_indices, ])
eimRMhalf2 <- eim(X = eimRM$X[-train_indices, ], W = eimRM$W[-train_indices, ])

eimRMhalf1 <- run_em(eimRMhalf1)
eimRMhalf2 <- run_em(eimRMhalf2)

plot_district(eimRMhalf1$prob, "Half 1", eimRMhalf2$prob, "Half 2")

## -----------------------------------------------------------------------------
comparison2 <- waldtest(
    object1 = eimRMhalf1,
    object2 = eimRMhalf2,
    nboot = 10,
    seed = 42
)

round(comparison2$pvals, 4)

## -----------------------------------------------------------------------------
eim_sim <- simulate_election(num_ballots = 15, num_groups = 2, num_candidates = 3, seed = 42)
eim_sim

## -----------------------------------------------------------------------------
eim_sim$real_prob

## -----------------------------------------------------------------------------
eim_sim <- run_em(eim_sim)
eim_sim$prob

## -----------------------------------------------------------------------------
input_probability <- matrix(c(0.9, 0.05, 0.05, 0.2, 0.3, 0.5), nrow = 2, byrow = TRUE)
input_probability

## -----------------------------------------------------------------------------
eim_sim2 <- simulate_election(
    num_ballots = 30, num_groups = 2, num_candidates = 3, seed = 42,
    prob = input_probability
)
eim_sim2
eim_sim2$real_prob

## -----------------------------------------------------------------------------
eim_sim3 <- simulate_election(
    num_ballots = 20, num_groups = 4, num_candidates = 2, seed = 42,
    lambda = 0.1
)

## ----eim_sim3_heatmap, fig.width = 8, fig.height = 3.5, fig.cap = "Voters' heatmap for a low lambda value", fig.align = "center", message=FALSE, warning=FALSE, results="hide"----
plot_matrix(eim_sim3$W)

## -----------------------------------------------------------------------------
run_em(eim_sim3)$prob

## -----------------------------------------------------------------------------
eim_sim4 <- simulate_election(
    num_ballots = 20, num_groups = 4, num_candidates = 2, seed = 42,
    lambda = 0.9
)

## ----eim_sim4_heatmap, fig.width = 8, fig.height = 3.5, fig.cap = "Voters' heatmap for a high lambda value", fig.align = "center", message=FALSE, warning=FALSE, results="hide"----
plot_matrix(eim_sim4$W)

## -----------------------------------------------------------------------------
run_em(eim_sim4)$prob

library(jsonlite)

#' S3 Object for the Expectation-Maximization Algorithm
#'
#' This constructor creates an `eim` S3 object, either by using matrices
#' `X` and `W` directly or by reading them from a JSON file. Each
#' `eim` object encapsulates the data (votes for candidates and demographic groups) required by the underlying Expectation-Maximization algorithm.
#'
#' @param X A `(b x c)` matrix representing candidate votes per ballot box.
#'
#' @param W A `(b x g)` matrix representing group votes per ballot box.
#'
#' @param json_path A path to a JSON file containing `X` and `W` fields, stored as nested arrays. It may contain additional fields with other attributes, which will be added to the returned object.
#'
#' @details
#' If `X` and `W` are directly supplied, they must match the
#' dimensions of ballot boxes `(b)`. Alternatively, if `json_path` is provided, the function expects
#' the JSON file to contain elements named `"X"` and `"W"` under the
#' top-level object. This two approaches are **mutually exclusable**, yielding an error otherwise.
#'
#' Internally, this function also initializes the corresponding instance within
#' the low-level (C-based) API, ensuring the data is correctly registered for
#' further processing by the EM algorithm.
#'
#' @return A list of class `eim` containing:
#' \describe{
#'   \item{\code{X}}{The candidate votes matrix \code{(b x c)}.}
#'   \item{\code{W}}{The group votes matrix \code{(b x g)}.}
#' }
#'
#' @note
#' A way to generate synthetic data for `X` and `W` is by using the [simulate_election] function. See Example 2 below.
#'
#' @section Methods:
#' In addition to this constructor, the "eim" class provides several
#' S3 methods for common operations. Some of these methods are fully documented,
#' while others are ommited due to its straightfoward implementantion. The available methods are:
#'
#' \itemize{
#'   \item \code{\link{run_em}} - Runs the EM algorithm.
#'   \item \code{\link{bootstrap}} - Estimates the standard deviation.
#'   \item \code{\link{save_eim}} - Saves the object to a file.
#'   \item \code{\link{get_agg_proxy}} - Estimates an ideal group aggregation given their standard deviations.
#'   \item \code{\link{get_agg_opt}} - Estimates an ideal group aggregation among all combinations, given the log-likelihood.
#' 	 \item \code{\link{plot.eim}} - Plots the probability matrix.
#'   \item \code{print.eim} - Print info about the object.
#'   \item \code{summary.eim} - Summarize the object.
#'   \item \code{as.matrix.eim} - Returns the probability matrix.
#'   \item \code{logLik.eim} - Returns the final log-likelihood, or evaluates log-likelihood at a fixed probability matrix.
#' }
#'
#' @examples
#'
#' # Example 1: Create an eim object from a JSON file
#' \dontrun{
#' model1 <- eim(json_path = "path/to/file.json")
#' }
#'
#' # Example 2: Use simulate_election with optional parameters, then create an eim object
#' # from matrices
#'
#' # Simulate data for 500 ballot boxes, 4 candidates and 5 groups
#' sim_result <- simulate_election(
#'     num_ballots = 500,
#'     num_candidates = 3,
#'     num_groups = 5,
#'     group_proportions = c(0.2, 0.2, 0.4, 0.1, 0.1)
#' )
#'
#' model2 <- eim(X = sim_result$X, W = sim_result$W)
#'
#' # Example 3: Create an object from a user defined matrix with 8 ballot boxes,
#' # 2 candidates and 7 groups.
#'
#' x_mat <- matrix(c(
#'     57, 90,
#'     60, 84,
#'     43, 102,
#'     72, 71,
#'     63, 94,
#'     52, 80,
#'     60, 72,
#'     54, 77
#' ), nrow = 8, ncol = 2, byrow = TRUE)
#'
#' w_mat <- matrix(c(
#'     10, 15, 25, 21, 10, 40, 26,
#'     11, 21, 37, 32, 8, 23, 12,
#'     17, 12, 43, 27, 12, 19, 15,
#'     20, 18, 25, 15, 22, 17, 26,
#'     21, 19, 27, 16, 23, 22, 29,
#'     18, 16, 20, 14, 19, 22, 23,
#'     10, 15, 21, 18, 20, 16, 32,
#'     12, 17, 19, 22, 15, 18, 28
#' ), nrow = 8, ncol = 7, byrow = TRUE)
#'
#' model3 <- eim(X = x_mat, W = w_mat)
#'
#' @export
#' @aliases eim()
eim <- function(X = NULL, W = NULL, json_path = NULL) {
    x_provided <- !is.null(X)
    w_provided <- !is.null(W)
    xw_provided <- x_provided || w_provided
    json_provided <- !is.null(json_path)

    if (sum(x_provided, w_provided) == 1) {
        stop("eim: If providing a matrix, 'X' and 'W' must be provided.")
    }

    if (sum(xw_provided, json_provided) != 1) {
        stop(paste(
            "eim: You must provide exactly one of the following:\n",
            "(1)\tA json path\n",
            "(2)\t`X` and `W`"
        ))
    }
    extra_params <- list()

    # Load data from JSON if a path is provided
    if (json_provided) {
        matrices <- .validate_json_eim(json_path) # nolint
        X <- as.matrix(matrices$X)
        W <- as.matrix(matrices$W)
        allowed_params <- c(
            "prob",
            "avg_prob",
            "initial_prob",
            "iterations",
            "nboot",
            "logLik",
            "convergence",
            "maxiter",
            "maxtime",
            "ll_threshold",
            "cond_prob",
            "sd",
            "group_agg",
            "message",
            "status",
            "time",
            "method",
            "W_agg",
            "mcmc_samples",
            "mcmc_stepsize",
            "mvncdf_method",
            "mvncdf_samples",
            "mvncdf_error",
            "miniter",
            "adjust_prob_cond_method",
            "adjust_prob_cond_every",
            "prob_inv",
            "cond_prob_inv",
            "expected_outcome_inv",
            "LL_ind",
            "LL_rev_ind",
            "dLL",
            "dLL_rev",
            "nu",
            "nu_rev",
            "err_forward",
            "err_inverse",
            "symmetric_weights",
            "symmetric_weight_method"
        )
        extra_params <- matrices[names(matrices) %in% allowed_params] # TODO: Validate them
    }

    # Perform matricial validation
    .validate_eim(X, W) # nolint

    # Create the S3 object
    obj <- list(
        X = X,
        W = W
    )
    # Add optional parameters if they exist
    if (length(extra_params) > 0) {
        obj <- c(obj, extra_params)
    }

    class(obj) <- "eim"
    obj
}
#' @title Compute the Expected-Maximization Algorithm
#'
#' @description
#' Executes the Expectation-Maximization (EM) algorithm indicating the approximation method to use in the E-step.
#' Certain methods may require additional arguments, which can be passed through `...` (see [fastei-package] for more details).
#'
#' @param object An object of class `eim`, which can be created using the [eim] function. This parameter should not be used if either (i) `X` and `W` matrices or (ii) `json_path` is supplied. See **Note**.
#'
#' @inheritParams eim
#'
#' @param method An optional string specifying the method used for estimating the E-step. Valid
#'   options are:
#' - `mult`: The default method, using a single sum of Multinomial distributions.
#' - `mvn_cdf`: Uses a Multivariate Normal CDF distribution to approximate the conditional probability.
#' - `mvn_pdf`: Uses a Multivariate Normal PDF distribution to approximate the conditional probability.
#' - `mcmc`: Uses MCMC to sample vote outcomes. This is used to estimate the conditional probability of the E-step.
#' - `exact`: Solves the E-step using the Total Probability Law.
#'
#' For a detailed description of each method, see [fastei-package] and **References**.
#'
#' @param initial_prob An optional string specifying the method used to obtain the initial
#'   probability. Accepted values are:
#' - `uniform`: Assigns equal probability to every candidate within each group.
#' - `proportional`: Assigns probabilities to each group based on the proportion of candidates votes.
#' - `group_proportional`: Computes the probability matrix by taking into account both group and candidate proportions. This is the default method.
#' - `random`: Use randomized values to fill the probability matrix.
#' @param allow_mismatch Boolean, if `TRUE`, allows a mismatch between the voters and votes for each ballot-box. If `FALSE`, throws an error if there is a mismatch. By default it is `TRUE`. See **Notes** for more details.
#'
#' @param maxiter An optional integer indicating the maximum number of EM iterations.
#'   The default value is `1000`.
#'
#' @param miniter An optional integer indicating the minimum number of EM iterations. The default value is `0`.
#'
#' @param maxtime An optional numeric specifying the maximum running time (in seconds) for the
#'   algorithm. This is checked at every iteration of the EM algorithm. The default value is `3600`, which corresponds to an hour.
#'
#' @param param_threshold An optional numeric value indicating the maximum Frobenius norm
#'   of the difference between consecutive probability matrices required to stop iterating. The default value is `0.001`. Note that the algorithm will stop if either `ll_threshold` **or** `param_threshold` is accomplished.
#'
#' @param ll_threshold An optional numeric value indicating the minimum difference between consecutive log-likelihood values to stop iterating. The default value is `inf`, essentially deactivating
#' the threshold. Note that the algorithm will stop if either `ll_threshold` **or** `param_threshold` is accomplished.
#'
#' @param compute_ll An optional boolean indicating whether to compute the log-likelihood at each iteration. The default value is `TRUE`.
#'
#' @param adjust_prob_cond_method An optional string indicating the method to adjust the conditional probability so that for each candidate, the sum product of voters and conditional probabilities across groups equals the votes obtained by the candidate. It can take values: `""` if no adjusting is made, `"lp"` if the adjustment is based on a linear programming that penalizes with L1-norm, `"project_lp"` if the adjustment is performed using projection and linear programming (this is the default)
#'
#' @param adjust_prob_cond_every An optional boolean indicating whether to adjust the conditional probability on every iteration (if `TRUE`), or only at the conditional probabilities obtained at the end of the EM algorithm (if `FALSE`, this is the default). This parameter applies only if `adjust_prob_conditional_method` is `lp` or `project_lp`.
#'
#' @param verbose An optional boolean indicating whether to print informational messages during the EM
#'   iterations. The default value is `FALSE`.
#'
#' @param seed An optional integer indicating the random seed for the randomized algorithms. This argument is only applicable if `initial_prob = "random"` or `method` is either `"mcmc"` or `"mvn_cdf"`.
#'
#' @param group_agg An optional vector of increasing integers from 1 to the number of columns in `W`, specifying how to aggregate groups in `W` before running the EM algorithm. Each value represents the highest column index included in each aggregated group. For example, if `W` has four columns, `group_agg = c(2, 4)` indicates that columns 1 and 2 should be combined into one group, and columns 3 and 4 into another. Defaults to `NULL`, in which case no group aggregation is performed.
#'
#' @param mcmc_stepsize An optional integer specifying the step size for the `mcmc`
#'   algorithm. This parameter is only applicable when `method = "mcmc"` and will
#'   be ignored otherwise. The default value is `3000`.
#'
#' @param mcmc_samples An optional integer indicating the number of samples to generate for the
#'   **MCMC** method. This parameter is only relevant when `method = "mcmc"`.
#'   The default value is `1000`.
#'
#' @param mvncdf_method An optional string specifying the method used to estimate the `mvn_cdf` method
#'   via a Monte Carlo simulation. Accepted values are `genz` and `genz2`, with `genz`
#'   set as the default. This parameter is only applicable when `method = "mvn_cdf"`. See **References** for more details.
#'
#' @param mvncdf_error An optional numeric value defining the error threshold for the Monte Carlo
#'   simulation when estimating the `mvn_cdf` method. The default value is `1e-6`. This parameter is only relevant
#' when `method = "mvn_cdf"`.
#'
#' @param mvncdf_samples An optional integer specifying the number of Monte Carlo
#'   samples for the `mvn_cdf` method. The default value is `5000`. This argument is only applicable when `method = "mvn_cdf"`.
#'
#' @param scale_factor An optional numeric value used to scale down the `X` and `W` matrices before executing the EM algorithm. This scaling can help improve performance when dealing with large vote counts. For example if `scale_factor = 2` all elements of `X` and `W` are divided by two and rounded. The default value is `1`, which means no scaling is applied. In case the scaling results in mismatch between `W` and `X`, ensure that `allow_mismatch = TRUE`.
#'
#' @param symmetric A boolean indicating whether to perform a symmetric estimation. If `TRUE`, the algorithm runs twice: first estimating the probabilities of candidates given groups, and then estimating the probabilities of groups given candidates. The final probabilities are obtained by combining both directions using `symmetric_weight_method` (`"joint"` by default). The default value is `FALSE`.
#'
#' @param symmetric_weight_method Character string indicating how to combine both directions when `symmetric = TRUE`. Valid values are `"joint"` (default behavior: joint EM where forward/reverse E-steps are averaged each iteration using equal weights), `"average"`, `"delta_ll"` (weights based on the relative likelihood gaps), and `"mae_inverse"` (weights based on reconstruction errors).
#'
#' @param ... Added for compability
#'
#' @references
#' [Thraves, C., Ubilla, P. and Hermosilla, D.: *"Fast Ecological Inference Algorithm for the RxC Case"*](https://papers.ssrn.com/sol3/papers.cfm?abstract_id=4832834). Aditionally, the MVN CDF is computed by the methods introduced in [Genz, A. (2000). Numerical computation of multivariate normal probabilities. *Journal of Computational and Graphical Statistics*](https://www.researchgate.net/publication/2463953_Numerical_Computation_Of_Multivariate_Normal_Probabilities)
#'
#' @note
#' This function can be executed using one of three mutually exclusive approaches:
#'
#' 1. By providing an existing `eim` object.
#' 2. By supplying both input matrices (`X` and `W`) directly.
#' 3. By specifying a JSON file (`json_path`) containing the matrices.
#'
#' These input methods are **mutually exclusive**, meaning that you must provide **exactly one** of
#' these options. Attempting to provide more than one or none of these inputs will result in an error.
#'
#' When called with an `eim` object, the function updates the object with the computed results.
#' If an `eim` object is not provided, the function will create one internally using either the
#' supplied matrices or the data from the JSON file before executing the algorithm.
#'
#' If there are ballot-boxes with mismatch between `W` and `X`, and `allow_mismatch = TRUE`, then: if `method = "exact"`, at each ballot-box with mismatch D'Hont is applied to add or remove the necessary voters from (`W`) so that its total match the total number of votes (`X`); if method is "`mvn_pdf`", "`mvn_cdf`" or "`mcmc`", the number of voters (`W`) of the ballot-box with mismatch is scaled to match its total number of votes (`X`).
#'
#' @seealso The [eim] object implementation.
#'
#' @return
#' The function returns an `eim` object with the function arguments and the following attributes:
#' \describe{
#'   \item{prob}{
#'     The estimated global probability matrix `(g x c)`.
#'   }
#' 	 \item{cond_prob}{
#'     A `(g x c x b)` 3d-array with the probability that at each ballot-box a voter of each group voted for each candidate,
#'     given the observed outcome at the particular ballot-box.
#'   }
#' 	 \item{expected_outcome}{
#'     A `(g x c x b)` 3d-array with the expected votes cast for each ballot-box.
#'   }
#'   \item{logLik}{The log-likelihood value from the last iteration.}
#'   \item{iterations}{The total number of iterations performed by the EM algorithm.}
#'   \item{time}{The total execution time of the algorithm in seconds.}
#'   \item{status}{
#'     The final status ID of the algorithm upon completion:
#'     \itemize{
#'       \item \code{0}: Converged
#'       \item \code{1}: Maximum time reached.
#'       \item \code{2}: Maximum iterations reached.
#'     }
#'   }
#'   \item{message}{The finishing status displayed as a message, matching the status ID value.}
#'   \item{method}{The method for estimating the conditional probability in the E-step.}
#' }
#' Aditionally, it will create `mcmc_samples` and `mcmc_stepsize` parameters if the specified `method = "mcmc"`, or `mvncdf_method`, `mvncdf_error` and `mvncdf_samples` if `method = "mvn_cdf"`.
#'
#' Also, if the eim object supplied is created with the function [simulate_election], it also returns the real probability and unobserved votes with the name `real_prob` and `outcome` respectively. See [simulate_election].
#'
#' If `group_agg` is different than `NULL`, two values are returned: `W_agg` a `(b x a)` matrix with the number of voters of each aggregated group o each ballot-box, and `group_agg` the same input vector.
#'
#' Furthermore, if `symmetric = TRUE`, the following additional attributes are included:
#' \describe{
#' 		\item{prob_inv}{The estimated probability matrix `(c x g)`, obtained by swapping `X` and `W`, when the symmetric scheme performs an explicit reverse run (for example `"average"`, `"delta_ll"`, and `"mae_inverse"`).}
#' 		\item{cond_prob_inv}{A `(c x g x b)` 3d-array with the probability that at each ballot-box a voter of each candidate voted for each group, given the observed outcome at the particular ballot-box, when the symmetric scheme performs an explicit reverse run.}
#' }
#' If `symmetric_weight_method = "delta_ll"` and both `logLik` values are available, the object also includes `LL_ind`, `LL_rev_ind`, `dLL`, `dLL_rev`, `nu`, `nu_rev`, and `symmetric_weights`.
#' If `symmetric_weight_method = "mae_inverse"`, the object also includes `err_forward`, `err_inverse`, and `symmetric_weights`.
#' Under this argument, the conditional probabilities will be obtained by dividing new expected outcomes by `W`. The probabilities will be calculated by performing an M-step.
#'
#' @examples
#' \donttest{
#' # Example 1: Compute the Expected-Maximization with default settings
#' simulations <- simulate_election(
#'     num_ballots = 300,
#'     num_candidates = 5,
#'     num_groups = 3,
#' )
#' model <- eim(simulations$X, simulations$W)
#' model <- run_em(model) # Returns the object with updated attributes
#'
#' # Example 2: Compute the Expected-Maximization using the mvn_pdf method
#' model <- run_em(
#'     object = model,
#'     method = "mvn_pdf",
#' )
#'
#' # Example 3: Run the mvn_cdf method with default settings
#' model <- run_em(object = model, method = "mvn_cdf")
#' }
#' \dontrun{
#' # Example 4: Perform an Exact estimation using user-defined parameters
#'
#' run_em(
#'     json_path = "a/json/file.json",
#'     method = "exact",
#'     initial_prob = "uniform",
#'     maxiter = 10,
#'     maxtime = 600,
#'     param_threshold = 1e-3,
#'     ll_threshold = 1e-5,
#'     verbose = TRUE
#' )
#' }
#'
#' @name run_em
#' @aliases run_em()
#' @export
run_em <- function(object = NULL,
                   X = NULL,
                   W = NULL,
                   json_path = NULL,
                   method = "mult",
                   initial_prob = "group_proportional",
                   allow_mismatch = TRUE,
                   maxiter = 1000,
                   miniter = 0,
                   maxtime = 3600,
                   param_threshold = 0.001,
                   ll_threshold = as.double(-Inf),
                   compute_ll = TRUE,
                   seed = NULL,
                   verbose = FALSE,
                   group_agg = NULL,
                   mcmc_samples = 1000,
                   mcmc_stepsize = 3000,
                   mvncdf_method = "genz",
                   mvncdf_error = 1e-3,
                   mvncdf_samples = 5000,
                   adjust_prob_cond_method = "project_lp",
                   adjust_prob_cond_every = FALSE,
                   scale_factor = 1,
                   symmetric = FALSE,
                   symmetric_weight_method = "joint",
                   ...) {
    base_call <- match.call()
    all_params <- lapply(as.list(match.call(expand.dots = TRUE)), eval, parent.frame())
    .validate_compute(all_params) # nolint

    if (!is.null(seed)) {
        set.seed(seed)
    }

    object <- .run_em_prepare_object(
        object = object,
        X = X,
        W = W,
        json_path = json_path,
        scale_factor = scale_factor,
        method = method,
        allow_mismatch = allow_mismatch,
        group_agg = group_agg
    )

    control <- list(
        base_call = base_call,
        caller_env = parent.frame(),
        all_params = all_params,
        method = method,
        initial_prob = initial_prob,
        maxiter = maxiter,
        miniter = miniter,
        maxtime = maxtime,
        param_threshold = param_threshold,
        ll_threshold = ll_threshold,
        compute_ll = compute_ll,
        verbose = verbose,
        adjust_prob_cond_method = adjust_prob_cond_method,
        adjust_prob_cond_every = adjust_prob_cond_every,
        symmetric = symmetric,
        symmetric_weight_method = symmetric_weight_method
    )

    result <- .run_em_core(object, control)

    invisible(result)
}

#' Runs a Bootstrap to Estimate the **Standard Deviation** of Predicted Probabilities
#'
#' @description
#' This function computes the Expected-Maximization (EM) algorithm "`nboot`" times. It then computes the standard deviation from the `nboot` estimated probability matrices on each component.
#' @param nboot Integer specifying how many times to run the
#'   EM algorithm.
#'
#' @inheritParams run_em
#'
#' @inheritParams simulate_election
#'
#' @param seed An optional integer indicating the random seed for the randomized algorithms. This argument is only applicable if `initial_prob = "random"` or `method` is either `"mcmc"` or `"mvn_cdf"`. Aditionally, it sets the random draws of the ballot boxes.
#'
#' @param ... Additional arguments passed to the [run_em] function that will execute the EM algorithm.
#'
#' @inherit run_em note
#'
#' @seealso The [eim] object and [run_em] implementation.
#'
#' @return
#' Returns an `eim` object with the `sd` field containing the estimated standard deviations of the probabilities and the `avg_prob` field with the average bootstrapped probability matrix. If an `eim` object is provided, its attributes (see [run_em]) are retained in the returned object.
#'
#' @examples
#' \donttest{
#' # Example 1: Using an 'eim' object directly
#' simulations <- simulate_election(
#'     num_ballots = 200,
#'     num_candidates = 5,
#'     num_groups = 3,
#' )
#'
#' model <- eim(X = simulations$X, W = simulations$W)
#'
#' model <- bootstrap(
#'     object = model,
#'     nboot = 30,
#'     method = "mult",
#'     maxiter = 500,
#'     verbose = FALSE,
#' )
#'
#' # Access standard deviation throughout 'model'
#' print(model$sd)
#'
#'
#' # Example 2: Providing 'X' and 'W' matrices directly
#' model <- bootstrap(
#'     X = simulations$X,
#'     W = simulations$W,
#'     nboot = 15,
#'     method = "mvn_pdf",
#'     maxiter = 100,
#'     maxtime = 5,
#'     param_threshold = 0.01,
#'     allow_mismatch = FALSE
#' )
#'
#' print(model$sd)
#' }
#'
#' # Example 3: Using a JSON file as input
#'
#' \dontrun{
#' model <- bootstrap(
#'     json_path = "path/to/election_data.json",
#'     nboot = 70,
#'     method = "mult",
#' )
#'
#' print(model$sd)
#' }
#'
#' @name bootstrap
#' @aliases bootstrap()
#' @export
bootstrap <- function(object = NULL,
                      X = NULL,
                      W = NULL,
                      json_path = NULL,
                      nboot = 100,
                      allow_mismatch = TRUE,
                      seed = NULL,
                      ...) {
    # Retrieve the default values from run_em() as a list
    all_params <- lapply(as.list(match.call(expand.dots = TRUE)), eval, parent.frame())
    .validate_compute(all_params) # nolint # It would validate nboot too.

    # Initialize eim object if needed
    if (is.null(object)) {
        object <- eim(X = X, W = W, json_path = json_path)
    } else if (!inherits(object, "eim")) {
        stop("Bootstrap: The object must be initialized with the `eim()` function.")
    }

    # Handle the group aggregation, if provided
    if (!is.null(all_params$group_agg)) {
        sizes <- diff(c(0, all_params$group_agg))
        rep_labels <- rep(seq_along(sizes), sizes)
        groups <- split(seq_len(ncol(object$W)), rep_labels)
        Wagg <- do.call(
            cbind,
            lapply(groups, function(cols) rowSums(object$W[, cols, drop = FALSE]))
        )
        rownames(Wagg) <- rownames(object$W)
        object$W_agg <- Wagg
        object$group_agg <- all_params$group_agg
    }

    # I need to define the method before on this case
    method <- if (!is.null(all_params$method)) all_params$method else "mult"

    # Applies a scaling
    if (!is.null(all_params$scale_factor) && all_params$scale_factor != 1) {
        object$X <- round(object$X / all_params$scale_factor)
        object$W <- round(object$W / all_params$scale_factor)
    }

    # Note: Mismatch restricted methods are checked inside .validate_compute
    if (!allow_mismatch) {
        mismatch_rows <- which(rowSums(object$X) != rowSums(object$W))

        if (length(mismatch_rows) > 0) {
            stop(
                "bootstrap: Row-wise mismatch in vote totals detected.\n",
                "Rows with mismatches: ", paste(mismatch_rows, collapse = ", "), "\n",
                "To allow mismatches, set `allow_mismatch = TRUE`."
            )
        }
    } else {
        if (method == "exact") {
            W <- .dhondt_correction(object$W, object$X)
        }
        message("Applying a D'Hondt correction for correcting mismatches in W")

        # stop("run_em: Exact method isn't supported with mismatch")
    }
    # Set seed for reproducibility
    if (!is.null(seed)) set.seed(seed)

    # Extract parameters with defaults if missing
    W <- if (is.null(object$W_agg)) object$W else object$W_agg
    initial_prob <- if (!is.null(all_params$initial_prob)) all_params$initial_prob else "group_proportional"
    maxiter <- if (!is.null(all_params$maxiter)) all_params$maxiter else 1000
    miniter <- if (!is.null(all_params$miniter)) all_params$miniter else 0
    maxtime <- if (!is.null(all_params$maxtime)) all_params$maxtime else 3600
    param_threshold <- if (!is.null(all_params$param_threshold)) all_params$param_threshold else 0.001
    verbose <- if (!is.null(all_params$verbose)) all_params$verbose else FALSE
    compute_ll <- if (!is.null(all_params$compute_ll)) all_params$compute_ll else TRUE
    adjust_prob_cond_method <- if (!is.null(all_params$adjust_prob_cond_method)) all_params$adjust_prob_cond_method else ""
    adjust_prob_cond_every <- if (!is.null(all_params$adjust_prob_cond_every)) all_params$adjust_prob_cond_every else FALSE

    # R does a subtle type conversion when handing -Inf. Hence, we'll use a direct assignment
    if ("ll_threshold" %in% names(all_params)) {
        ll_threshold <- all_params$ll_threshold
    } else {
        ll_threshold <- as.double(-Inf)
    }

    # Handle method-specific defaults
    mcmc_stepsize <- 0L
    mcmc_samples <- 0L
    mvncdf_method <- ""
    mvncdf_samples <- 0L
    mvncdf_error <- 0.0

    if (method == "mcmc") {
        mcmc_stepsize <- if (!is.null(all_params$mcmc_stepsize)) all_params$mcmc_stepsize else 3000
        mcmc_samples <- if (!is.null(all_params$mcmc_samples)) all_params$mcmc_samples else 1000
        burn_in <- if (!is.null(all_params$burn_in)) all_params$burn_in else 10000
    } else if (method == "mvn_cdf") {
        mvncdf_method <- if (!is.null(all_params$mvncdf_method)) all_params$method else "genz"
        mvncdf_samples <- if (!is.null(all_params$mvncdf_samples)) all_params$mvncdf_samples else 5000
        mvncdf_error <- if (!is.null(all_params$mvncdf_error)) all_params$mvncdf_error else 1e-6
    } # Call C bootstrap function

    result <- bootstrapAlg(
        t(object$X),
        W,
        as.integer(nboot),
        as.character(method),
        if (is.character(initial_prob)) as.character(initial_prob) else "custom",
        as.integer(maxiter),
        as.double(maxtime),
        as.double(param_threshold),
        as.double(ll_threshold),
        as.logical(compute_ll),
        as.logical(verbose),
        as.integer(mcmc_stepsize),
        as.integer(mcmc_samples),
        as.character(mvncdf_method),
        as.double(mvncdf_error),
        as.integer(mvncdf_samples),
        as.integer(miniter),
        as.character(adjust_prob_cond_method),
        as.logical(adjust_prob_cond_every),
        if (is.matrix(initial_prob)) initial_prob else matrix(-1, nrow = 1, ncol = 1)
    )

    object$sd <- result$sd
    dimnames(object$sd) <- list(colnames(W), colnames(object$X))
    object$avg_prob <- result$avg_prob
    dimnames(object$avg_prob) <- list(colnames(W), colnames(object$X))
    object$nboot <- nboot
    object$sd[object$sd == 9999] <- Inf
    object$avg_prob[object$avg_prob == 9999] <- Inf

    class(object) <- "eim"
    return(object)
}

#' Runs the EM algorithm aggregating adjacent groups, maximizing the variability of macro-group allocation in ballot boxes.
#'
#' This function estimates the voting probabilities (computed using [run_em]) aggregating adjacent groups so that the estimated probabilities' standard deviation (computed using [bootstrap]) is below a given threshold. See **Details** for more information.
#'
#' Groups need to have an order relation so that adjacent groups can be merged. Groups of consecutive column indices in the matrix W are considered adjacent. For example, consider the following seven groups defined by voters' age ranges: 20-29, 30-39, 40-49, 50-59, 60-69, 70-79, and 80+. A possible group aggregation can be a macro-group composed of the three following age ranges: 20-39, 40-59, and 60+. Since there are multiple group aggregations, even for a fixed number of macro-groups, a Dynamic Program (DP) mechanism is used to find the group aggregation that maximizes the sum of the standard deviation of the macro-groups proportions among ballot boxes for a specific number of macro-groups. If no group aggregation standard deviation statistic meets the threshold condition, `NULL` is returned.
#'
#' To find the best group aggregation, the function runs the DP iteratively, starting with all groups (this case is trivial since the group aggregation is such that all macro-groups match exactly the original groups). If the standard deviation statistic (`sd_statistic`) is below the threshold (`sd_threshold`), it stops. Otherwise, it runs the DP such that the number of macro-groups is one unit less than the original number of macro-groups. If the standard deviation statistic is below the threshold, it stops. This continues until either the algorithm stops, or until no group aggregation obtained by the DP satisfies the threshold condition. If the former holds, then the last group aggregation obtained (before stopping) is returned; while if the latter holds, then no output is returned unless the user sets the input parameter `feasible=FALSE`, in which case it returns the group aggregation that has the least standard deviation statistic, among the group-aggregations obtained from the DP.
#'
#' @param object An object of class `eim`, which can be created using the [eim] function. This parameter should not be used if either (i) `X` and `W` matrices or (ii) `json_path` is supplied. See **Note** in [run_em].
#'
#' @param sd_statistic String indicates the statistic for the standard deviation `(g x c)` matrix for the stopping condition, i.e., the algorithm stops when the statistic is below the threshold. It can take the value `maximum`, in which case computes the maximum over the standard deviation matrix, or `average`, in which case computes the average.
#'
#' @param sd_threshold Numeric with the value to use as a threshold for the statistic (`sc_statistic`) of the standard deviation of the estimated probabilities. Defaults to 0.05.
#'
#' @param method An optional string specifying the method used for estimating the E-step. Valid
#'   options are:
#' - `mult`: The default method, using a single sum of Multinomial distributions.
#' - `mvn_cdf`: Uses a Multivariate Normal CDF distribution to approximate the conditional probability.
#' - `mvn_pdf`: Uses a Multivariate Normal PDF distribution to approximate the conditional probability.
#' - `mcmc`: Uses MCMC to sample vote outcomes. This is used to estimate the conditional probability of the E-step.
#' - `exact`: Solves the E-step using the Total Probability Law.
#'
#' @param feasible Logical indicating whether the returned matrix must strictly satisfy the `sd_threshold`.
#' If `TRUE`, no output is returned if the method does not find a group aggregation whose standard deviation statistic is below the threshold. If `FALSE` and the latter holds, it returns the group aggregation obtained from the DP with the the lowest standard deviation statistic. See **Details** for more information. Default is `TRUE`.
#'
#' @inheritParams bootstrap
#'
#' @param allow_mismatch Boolean, if `TRUE`, allows a mismatch between the voters and votes for each ballot-box. If `FALSE`, throws an error if there is a mismatch. By default it is `TRUE`. See **Notes** in [run_em] for more details.
#'
#' @param ... Additional arguments passed to the [run_em] function that will execute the EM algorithm.
#'
#' @seealso The [eim] object and [run_em] implementation.
#'
#' @return
#' It returns an eim object with the same attributes as the output of [run_em], plus the attributes:
#'
#' - **sd**: A `(a x c)` matrix with the standard deviation of the estimated probabilities computed with bootstrapping. Note that `a` denotes the number of macro-groups of the resulting group aggregation, it should be between `1` and `g`.
#' - **nboot**: Number of samples used for the [bootstrap] method.
#' - **seed**: Random seed used (if specified).
#' - **sd_statistic**: The statistic used as input.
#' - **sd_threshold**: The threshold used as input.
#' - **is_feasible**:  Boolean indicating whether the statistic of the standard deviation matrix is below the threshold.
#' - **group_agg**: Vector with the resulting group aggregation. See **Examples** for more details.
#'
#' Additionally, it will create the `W_agg` attribute with the aggregated groups, along with the attributes corresponding to running [run_em] with the aggregated groups.
#'
#' @examples
#' # Example 1: Using a simulated instance
#' simulations <- simulate_election(
#'     num_ballots = 400,
#'     num_candidates = 3,
#'     num_groups = 6,
#'     group_proportions = c(0.4, 0.1, 0.1, 0.1, 0.2, 0.1),
#'     lambda = 0.7,
#'     seed = 42
#' )
#'
#' result <- get_agg_proxy(
#'     X = simulations$X,
#'     W = simulations$W,
#'     sd_threshold = 0.015,
#'     seed = 42
#' )
#'
#' result$group_agg # c(2 6)
#' # This means that the resulting group aggregation is conformed by
#' # two macro-groups: one that has the original groups 1 and 2; and
#' # a second that has the original groups 3, 4, 5, and 6.
#'
#' # Example 2: Using the chilean election results
#' data(chile_election_2021)
#'
#' niebla_df <- chile_election_2021[chile_election_2021$ELECTORAL.DISTRICT == "NIEBLA", ]
#'
#' # Create the X matrix with selected columns
#' X <- as.matrix(niebla_df[, c("C1", "C2", "C3", "C4", "C5", "C6", "C7")])
#'
#' # Create the W matrix with selected columns
#' W <- as.matrix(niebla_df[, c(
#'     "X18.19", "X20.29",
#'     "X30.39", "X40.49",
#'     "X50.59", "X60.69",
#'     "X70.79", "X80."
#' )])
#'
#' solution <- get_agg_proxy(
#'     X = X, W = W,
#'     allow_mismatch = TRUE, sd_threshold = 0.03,
#'     sd_statistic = "average", seed = 42
#' )
#'
#' solution$group_agg # c(3, 4, 5, 6, 8)
#' # This means that the resulting group aggregation consists of
#' # five macro-groups: one that includes the original groups 1, 2, and 3;
#' # three singleton groups (4, 5, and 6); and one macro-group that includes groups 7 and 8.
#'
#' @export
get_agg_proxy <- function(object = NULL,
                          X = NULL,
                          W = NULL,
                          json_path = NULL,
                          sd_statistic = "maximum",
                          sd_threshold = 0.05,
                          method = "mult",
                          feasible = TRUE,
                          nboot = 100,
                          allow_mismatch = TRUE,
                          seed = NULL, ...) {
    # Retrieve the default values from run_em() as a list
    all_params <- lapply(as.list(match.call(expand.dots = TRUE)), eval, parent.frame())
    .validate_compute(all_params) # nolint # It would validate nboot too.

    # Retrieve default values from run_em() and update with user parameters
    run_em_defaults <- formals(run_em)
    run_em_args <- modifyList(as.list(run_em_defaults), all_params)
    run_em_args <- run_em_args[names(run_em_args) != "..."] # Remove ellipsis

    # Set seed for reproducibility
    if (!is.null(seed)) set.seed(seed)

    # Initialize eim object if needed
    if (is.null(object)) {
        object <- eim(X = X, W = W, json_path = json_path)
    } else if (!inherits(object, "eim")) {
        stop("Bootstrap: The object must be initialized with the `eim()` function.")
    }
    # I need to define the method before
    if (!is.null(all_params$scale_factor) && all_params$scale_factor != 1) {
        object$X <- round(object$X / all_params$scale_factor)
        object$W <- round(object$W / all_params$scale_factor)
    }

    method <- if (!is.null(all_params$method)) all_params$method else "mult"
    # Note: Mismatch restricted methods are checked inside .validate_compute
    if (!allow_mismatch) {
        mismatch_rows <- which(rowSums(object$X) != rowSums(object$W))

        if (length(mismatch_rows) > 0) {
            stop(
                "get_agg_proxy: Row-wise mismatch in vote totals detected.\n",
                "Rows with mismatches: ", paste(mismatch_rows, collapse = ", "), "\n",
                "To allow mismatches, set `allow_mismatch = TRUE`."
            )
        }
    } else {
        if (method == "exact") {
            W <- .dhondt_correction(object$W, object$X)
        }
        message("Applying a D'Hondt correction for correcting mismatches in W")

        # stop("run_em: Exact method isn't supported with mismatch")
    }

    # Extract parameters with defaults if missing
    initial_prob <- if (!is.null(all_params$initial_prob)) all_params$initial_prob else "group_proportional"
    maxiter <- if (!is.null(all_params$maxiter)) all_params$maxiter else 1000
    miniter <- if (!is.null(all_params$miniter)) all_params$miniter else 0
    maxtime <- if (!is.null(all_params$maxtime)) all_params$maxtime else 3600
    param_threshold <- if (!is.null(all_params$param_threshold)) all_params$param_threshold else 0.01
    verbose <- if (!is.null(all_params$verbose)) all_params$verbose else FALSE
    compute_ll <- if (!is.null(all_params$compute_ll)) all_params$compute_ll else TRUE
    adjust_prob_cond_method <- if (!is.null(all_params$adjust_prob_cond_method)) all_params$adjust_prob_cond_method else ""
    adjust_prob_cond_every <- if (!is.null(all_params$adjust_prob_cond_every)) all_params$adjust_prob_cond_every else FALSE


    # R does a subtle type conversion when handing -Inf. Hence, we'll use a direct assignment
    if ("ll_threshold" %in% names(all_params)) {
        ll_threshold <- all_params$ll_threshold
    } else {
        ll_threshold <- as.double(-Inf)
    }

    # Handle method-specific defaults
    mcmc_stepsize <- 0L
    mcmc_samples <- 0L
    mvncdf_method <- ""
    mvncdf_samples <- 0L
    mvncdf_error <- 0.0

    if (method == "mcmc") {
        mcmc_stepsize <- if (!is.null(all_params$mcmc_stepsize)) all_params$mcmc_stepsize else 3000
        mcmc_samples <- if (!is.null(all_params$mcmc_samples)) all_params$mcmc_samples else 1000
        burn_in <- if (!is.null(all_params$burn_in)) all_params$burn_in else 10000
    } else if (method == "mvn_cdf") {
        mvncdf_method <- if (!is.null(all_params$mvncdf_method)) all_params$method else "genz"
        mvncdf_samples <- if (!is.null(all_params$mvncdf_samples)) all_params$mvncdf_samples else 5000
        mvncdf_error <- if (!is.null(all_params$mvncdf_error)) all_params$mvncdf_error else 1e-6
    }

    result <- groupAgg(
        as.character(sd_statistic),
        as.double(sd_threshold),
        as.logical(feasible),
        t(object$X),
        object$W,
        as.integer(nboot),
        as.character(method),
        if (is.character(initial_prob)) as.character(initial_prob) else "custom",
        as.integer(maxiter),
        as.double(maxtime),
        as.double(param_threshold),
        as.double(ll_threshold),
        as.logical(compute_ll),
        as.logical(verbose),
        as.integer(mcmc_stepsize),
        as.integer(mcmc_samples),
        as.character(mvncdf_method),
        as.double(mvncdf_error),
        as.integer(mvncdf_samples),
        as.integer(miniter),
        as.character(adjust_prob_cond_method),
        as.logical(adjust_prob_cond_every),
        if (is.matrix(initial_prob)) initial_prob else matrix(-1, nrow = 1, ncol = 1)
    )

    # If the returned matrix isn't the best non-feasible result
    # best_result <- TRUE if it's unfeasible
    if (!result$best_result || !feasible) {
        # Convert the 'W' matrix by merging columns
        # We add '2' to indices since it's originally 0-based.
        col_groups <- split(seq_len(ncol(object$W)), findInterval(seq_len(ncol(object$W)), c(1, result$indices + 2)))
        # Lambda function to add the columns, if there wasn't a group aggregation, return object$W
        # object$W_agg <- as.matrix(sapply(col_groups, function(cols) rowSums(object$W[, cols, drop = FALSE])))
        object$W_agg <- as.matrix(do.call(cbind, lapply(col_groups, function(cols) rowSums(object$W[, cols, drop = FALSE]))))
        rownames(object$W_agg) <- rownames(object$W) # Ballot boxes
        # if (result$indices[1] != -1) {
        object$group_agg <- unique(result$indices + 1)
        # } # Use R's index system
        object$sd <- result$bootstrap_result
        dimnames(object$sd) <- list(colnames(object$W_agg), colnames(object$X))
        object$sd[object$sd == 9999] <- Inf
        object$sd_statistic <- sd_statistic
        object$sd_threshold <- sd_threshold
        object$is_feasible <- !result$best_result

        # Add EM arguments aswell
        # core_args <- all_params[!names(all_params) %in% c("object", "X", "W", "X2", "W2", "json_path", "verbose")]
        # final_args <- c(core_args, list(object = NULL, json_path = NULL, X = object$X, W = object$W_agg, verbose = FALSE))
        em_results <- run_em(X = object$X, W = object$W_agg, method = method, allow_mismatch = TRUE)
        object <- c(
            object,
            em_results[setdiff(names(em_results), names(object))]
        )
    }

    class(object) <- "eim"
    return(object)
}

#' Runs the EM algorithm **over all possible group aggregating**, returning the one with higher likelihood while constraining the standard deviation of the probabilities.
#'
#' This function estimates the voting probabilities (computed using [run_em]) by trying all group aggregations (of adjacent groups), choosing
#' the one that achieves the higher likelihood as long as the standard deviation (computed using [bootstrap]) of the estimated probabilities
#' is below a given threshold. See **Details** for more informacion on adjacent groups.
#'
#' Groups of consecutive column indices in the matrix `W` are considered adjacent. For example, consider the following seven groups defined by voters' age
#' ranges: 20-29, 30-39, 40-49, 50-59, 60-69, 70-79, and 80+. A possible group aggregation can be a macro-group composed of the three following age
#' ranges: 20-39, 40-59, and 60+. Since there are multiple group aggregations, the method evaluates all possible group aggregations (merging only adjacent groups).
#'
#' @inheritParams get_agg_proxy
#'
#' @param ... Additional arguments passed to the [run_em] function that will execute the EM algorithm.
#'
#' @param allow_mismatch Boolean, if `TRUE`, allows a mismatch between the voters and votes for each ballot-box. If `FALSE`, throws an error if there is a mismatch. By default it is `TRUE`. See **Notes** in [run_em] for more details.
#'
#' @return
#' It returns an eim object with the same attributes as the output of [run_em], plus the attributes:
#'
#' - **sd**: A `(a x c)` matrix with the standard deviation of the estimated probabilities computed with bootstrapping. Note that `a` denotes the number of macro-groups of the resulting group aggregation, it should be between `1` and `g`.
#' - **nboot**: Number of samples used for the [bootstrap] method.
#' - **seed**: Random seed used (if specified).
#' - **sd_statistic**: The statistic used as input.
#' - **sd_threshold**: The threshold used as input.
#' - **group_agg**: Vector with the resulting group aggregation. See **Examples** for more details.
#'
#' Additionally, it will create the `W_agg` attribute with the aggregated groups, along with the attributes corresponding to running [run_em] with the aggregated groups.
#'
#' @examples
#' # Example 1: Using a simulated instance
#' simulations <- simulate_election(
#'     num_ballots = 20,
#'     num_candidates = 3,
#'     num_groups = 8,
#'     seed = 42
#' )
#'
#' result <- get_agg_opt(
#'     X = simulations$X,
#'     W = simulations$W,
#'     sd_threshold = 0.05,
#'     seed = 42
#' )
#'
#' result$group_agg # c(3,8)
#' # This means that the resulting group aggregation consists of
#' # two macro-groups: one that includes the original groups 1, 2, and 3;
#' # the remaining one with groups 4, 5, 6, 7 and 8.
#'
#' \donttest{
#' # Example 2: Getting an unfeasible result
#' result2 <- get_agg_opt(
#'     X = simulations$X,
#'     W = simulations$W,
#'     sd_threshold = 0.001
#' )
#'
#' result2$group_agg # Error
#' result2$X # Input candidates' vote matrix
#' result2$W # Input group-level voter matrix
#' }
#' @export
get_agg_opt <- function(object = NULL,
                        X = NULL,
                        W = NULL,
                        json_path = NULL,
                        sd_statistic = "maximum",
                        sd_threshold = 0.05,
                        method = "mult",
                        nboot = 100,
                        allow_mismatch = TRUE,
                        seed = NULL,
                        ...) {
    # Retrieve the default values from run_em() as a list
    all_params <- lapply(as.list(match.call(expand.dots = TRUE)), eval, parent.frame())
    .validate_compute(all_params) # nolint # It would validate nboot too.

    # Retrieve default values from run_em() and update with user parameters
    run_em_defaults <- formals(run_em)
    run_em_args <- modifyList(as.list(run_em_defaults), all_params)
    run_em_args <- run_em_args[names(run_em_args) != "..."] # Remove ellipsis

    if (!is.null(seed)) set.seed(seed) # Set seed for reproducibility
    # Initialize eim object if needed
    if (is.null(object)) {
        object <- eim(X = X, W = W, json_path = json_path)
    } else if (!inherits(object, "eim")) {
        stop("get_agg_opt: The object must be initialized with the `eim()` function.")
    }
    # Note: Mismatch restricted methods are checked inside .validate_compute
    # Method needs to be defined before

    if (!is.null(all_params$scale_factor) && all_params$scale_factor != 1) {
        object$X <- round(object$X / all_params$scale_factor)
        object$W <- round(object$W / all_params$scale_factor)
    }

    method <- if (!is.null(all_params$method)) all_params$method else "mult"
    if (!allow_mismatch) {
        mismatch_rows <- which(rowSums(object$X) != rowSums(object$W))

        if (length(mismatch_rows) > 0) {
            stop(
                "get_agg_opt: Row-wise mismatch in vote totals detected.\n",
                "Rows with mismatches: ", paste(mismatch_rows, collapse = ", "), "\n",
                "To allow mismatches, set `allow_mismatch = TRUE`."
            )
        }
    } else {
        if (method == "exact") {
            W <- .dhondt_correction(object$W, object$X)
        }
        message("Applying a D'Hondt correction for correcting mismatches in W")

        # stop("run_em: Exact method isn't supported with mismatch")
    }

    initial_prob <- if (!is.null(all_params$initial_prob)) all_params$initial_prob else "group_proportional"
    maxiter <- if (!is.null(all_params$maxiter)) all_params$maxiter else 1000
    miniter <- if (!is.null(all_params$miniter)) all_params$miniter else 0
    maxtime <- if (!is.null(all_params$maxtime)) all_params$maxtime else 3600
    param_threshold <- if (!is.null(all_params$param_threshold)) all_params$param_threshold else 0.01
    verbose <- if (!is.null(all_params$verbose)) all_params$verbose else FALSE
    compute_ll <- if (!is.null(all_params$compute_ll)) all_params$compute_ll else TRUE
    adjust_prob_cond_method <- if (!is.null(all_params$adjust_prob_cond_method)) all_params$adjust_prob_cond_method else ""
    adjust_prob_cond_every <- if (!is.null(all_params$adjust_prob_cond_every)) all_params$adjust_prob_cond_every else FALSE

    # R does a subtle type conversion when handing -Inf. Hence, we'll use a direct assignment
    if ("ll_threshold" %in% names(all_params)) {
        ll_threshold <- all_params$ll_threshold
    } else {
        ll_threshold <- as.double(-Inf)
    }

    # Handle method-specific defaults
    mcmc_stepsize <- 0L
    mcmc_samples <- 0L
    mvncdf_method <- ""
    mvncdf_samples <- 0L
    mvncdf_error <- 0.0

    if (method == "mcmc") {
        mcmc_stepsize <- if (!is.null(all_params$mcmc_stepsize)) all_params$mcmc_stepsize else 3000
        mcmc_samples <- if (!is.null(all_params$mcmc_samples)) all_params$mcmc_samples else 1000
        burn_in <- if (!is.null(all_params$burn_in)) all_params$burn_in else 10000
    } else if (method == "mvn_cdf") {
        mvncdf_method <- if (!is.null(all_params$mvncdf_method)) all_params$mvncdf_method else "genz"
        mvncdf_samples <- if (!is.null(all_params$mvncdf_samples)) all_params$mvncdf_samples else 5000
        mvncdf_error <- if (!is.null(all_params$mvncdf_error)) all_params$mvncdf_error else 1e-6
    }

    result <- groupAggGreedy(
        as.character(sd_statistic),
        as.double(sd_threshold),
        t(object$X),
        object$W,
        as.integer(nboot),
        as.character(method),
        if (is.character(initial_prob)) as.character(initial_prob) else "group_proportional",
        as.integer(maxiter),
        as.double(maxtime),
        as.double(param_threshold),
        as.double(ll_threshold),
        as.logical(compute_ll),
        as.logical(verbose),
        as.integer(mcmc_stepsize),
        as.integer(mcmc_samples),
        as.character(mvncdf_method),
        as.double(mvncdf_error),
        as.integer(mvncdf_samples),
        as.integer(miniter),
        as.character(adjust_prob_cond_method),
        as.logical(adjust_prob_cond_every)
        # if (is.matrix(initial_prob)) initial_prob else matrix(-1, nrow = 1, ncol = 1)
    )

    if (result$indices[[1]] == -1) {
        return(object)
    }

    # Convert the 'W' matrix by merging columns
    # We add '2' to indices since it's originally 0-based.
    col_groups <- split(seq_len(ncol(object$W)), findInterval(seq_len(ncol(object$W)), c(1, result$indices + 2)))
    # Lambda function to add the columns
    # object$W_agg <- as.matrix(sapply(col_groups, function(cols) rowSums(object$W[, cols, drop = FALSE])))
    object$cond_prob <- result$q
    # object$cond_prob <- aperm(result$q, perm = c(2, 3, 1)) # Correct dimensions
    dimnames(object$cond_prob) <- list(
        NULL,
        colnames(object$X),
        rownames(object$X)
    )
    object$expected_outcome <- result$expected_outcome
    dimnames(object$expected_outcome) <- list(
        NULL,
        colnames(object$X),
        rownames(object$X)
    )
    object$W_agg <- do.call(cbind, lapply(col_groups, function(cols) rowSums(object$W[, cols, drop = FALSE])))
    rownames(object$W_agg) <- rownames(object$W)
    object$group_agg <- result$indices + 1 # Use R's index system
    object$prob <- as.matrix(result$probabilities)
    dimnames(object$prob) <- list(NULL, colnames(object$X))
    object$iterations <- as.numeric(result$total_iterations)
    object$logLik <- as.numeric(result$log_likelihood)
    object$time <- result$total_time
    object$message <- result$stopping_reason
    object$status <- as.integer(result$finish_id)
    object$sd <- result$bootstrap_sol
    dimnames(object$sd) <- dimnames(object$prob)
    object$sd[object$sd == 9999] <- Inf
    object$method <- method
    object$ll_threshold <- ll_threshold
    object$param_threshold <- param_threshold
    object$maxtime <- maxtime
    object$maxiter <- maxiter
    object$miniter <- miniter
    object$initial_prob <- initial_prob
    object$adjust_prob_cond_method <- adjust_prob_cond_method
    object$adjust_prob_cond_every <- adjust_prob_cond_every

    if (method == "mvn_cdf") {
        object$mvncdf_error <- mvncdf_error
        object$mvncdf_samples <- mvncdf_samples
        object$mvncdf_method <- mvncdf_method
    } else if (method == "mcmc") {
        object$mcmc_stepsize <- mcmc_stepsize
        object$mcmc_samples <- mcmc_samples
    }

    class(object) <- "eim"
    return(object)
}

#' Performs a matrix-wise Wald test for two eim objects
#'
#'
#' This function compares two `eim` objects (or sets of matrices that can be converted to such objects) by computing a Wald test on each component
#' of their estimated probability matrices. The Wald test is applied using bootstrap-derived standard deviations, and the result is a matrix
#' of p-values corresponding to each group-candidate combination.
#'
#' It uses Wald test to analyze if there is a significant difference between the estimated probabilities between a treatment and a control set. The test is performed independently for each component of the probability matrix.
#'
#' @inheritParams bootstrap
#'
#' @param object1 An `eim` object, as returned by [eim].
#' @param object2 A second `eim` object to compare with `object`.
#' @param X1 A `(b x c)` matrix representing candidate votes per ballot box.
#' @param W1 A `(b x g)` matrix representing group votes per ballot box.
#' @param X2 A second `(b x c)` matrix to compare with `X`.
#' @param W2 A second `(b x g)` matrix to compare with `W`.
#' @param nboot Integer specifying how many times to run the EM algorithm per object.
#' @param alternative Character string specifying the type of alternative hypothesis to test. Must be one of `"two.sided"` (default), `"greater"`, or `"less"`. If `"two.sided"`, the test checks for any difference in estimated probabilities. If `"greater"`, it tests whether the first object has systematically higher probabilities than the second. If `"less"`, it tests whether the first has systematically lower probabilities.
#' @param ... Additional arguments passed to [bootstrap] and [run_em].
#'
#' @return A list with components:
#'   - `pvals`: a numeric matrix of p-values with the same dimensions as the estimated probability matrices (`pvals`) from the input objects.
#'   - `statistic`: a numeric matrix of z-statistics with the same dimensions as the estimated probability matrices (`pvals`).
#'   - `eim1` and `eim2`: the original `eim` objects used for comparison.
#'
#' Each entry in the pvals matrix is the p-value from Wald test between the corresponding
#' entries of the two estimated probability matrices.
#'
#' @details
#' The user must provide either of the following (but not both):
#' - Two `eim` objects via `object1` and `object2`, or
#' - Four matrices: `X1`, `W1`, `X2`, and `W2`, which will be converted into `eim` objects internally.
#'
#' The Wald test is computed using the formula:
#'
#' \deqn{
#' z_{ij} = \frac{p_{1,ij}-p_{2,ij}}{\sqrt{s_{1,ij}^2+s_{2,ij}^2}}
#' }
#' In this expression, \eqn{s_{1,ij}^2} and \eqn{s_{2,ij}^2} represent the bootstrap sample variances for the treatment and control sets, respectively, while \eqn{p_{1,ij}} and \eqn{p_{2,ij}} are the corresponding estimated probability matrices obtained via the EM algorithm.
#'
#' @examples
#' sim1 <- simulate_election(num_ballots = 100, num_candidates = 3, num_groups = 5, seed = 123)
#' sim2 <- simulate_election(num_ballots = 100, num_candidates = 3, num_groups = 5, seed = 124)
#'
#' result <- waldtest(sim1, sim2, nboot = 100)
#'
#' # Check which entries are significantly different
#' which(result$pvals < 0.05, arr.ind = TRUE)
#'
#' @export
waldtest <- function(object1 = NULL,
                     object2 = NULL,
                     X1 = NULL,
                     W1 = NULL,
                     X2 = NULL,
                     W2 = NULL,
                     nboot = 100,
                     seed = NULL,
                     alternative = "two.sided",
                     ...) {
    object <- object1
    X <- X1
    W <- W1
    provided <- c(!is.null(X1), !is.null(W1), !is.null(X2), !is.null(W2))
    invalidMat <- any(provided) && !all(provided)
    using_objects <- !is.null(object1) && !is.null(object2)
    using_matrices <- all(provided)
    input_modes <- sum(using_objects, using_matrices)

    if (input_modes == 0) {
        stop("Invalid input: must supply two objects or four matrices.")
    } else if (input_modes > 1 || invalidMat) {
        stop("Invalid input: you must provide either: two eim objects (object1, object2), or four matrices (X1, X2, W1, W2), but not both.")
    }

    all_params <- lapply(as.list(match.call(expand.dots = TRUE)), eval, parent.frame())
    # .validate_compute(all_params) # nolint # It would validate nboot too.

    # Retrieve default values from bootstrap() and update with user parameters
    bootstrap_defaults <- formals(bootstrap)
    bootstrap_args <- modifyList(as.list(bootstrap_defaults), all_params)
    bootstrap_args <- bootstrap_args[names(bootstrap_args) != "..."] # Remove ellipsis

    if (using_matrices) {
        object <- eim(X, W)
        object2 <- eim(X2, W2)
    }

    if (ncol(object$X) != ncol(object2$X) || ncol(object$W) != ncol(object2$W)) {
        stop("Column dimensions must be the same for both 'eim' objects")
    }

    # First object
    if (!is.null(all_params$verbose) && all_params$verbose) {
        message("Obtaining the values of the first object.\n")
    }
    boot1 <- do.call(bootstrap, c(
        list(object = object),
        bootstrap_args[!names(bootstrap_args) %in% c("object", "object2", "X", "X1", "X2", "W", "W1", "W2", "json_path")],
        list(verbose = FALSE)
    ))
    em1 <- do.call(run_em, c(
        list(object = object),
        bootstrap_args[!names(bootstrap_args) %in% c("object", "object2", "X1", "X", "X2", "W1", "W", "W2", "json_path")],
        list(verbose = FALSE)
    ))

    if (!is.null(all_params$verbose) && all_params$verbose) {
        message("Obtaining the values of the second object.\n")
    }
    # Second object
    boot2 <- do.call(bootstrap, c(
        list(object = object2),
        bootstrap_args[!names(bootstrap_args) %in% c("object", "object2", "X", "X1", "X2", "W", "W1", "W2", "json_path")],
        list(verbose = FALSE)
    ))
    em2 <- do.call(run_em, c(
        list(object = object2),
        bootstrap_args[!names(bootstrap_args) %in% c("object", "object2", "X", "X1", "X2", "W", "W1", "W2", "json_path")],
        list(verbose = FALSE)
    ))

    # Matrix-wise p-values
    var1 <- boot1$sd^2
    var2 <- boot2$sd^2

    delta <- em1$prob - em2$prob
    se_delta <- sqrt(var1 + var2)

    z <- delta / se_delta

    pvals <- switch(alternative,
        "two.sided" = 2 * pnorm(-abs(z)),
        "greater"   = pnorm(-z),
        "less"      = pnorm(z)
    )

    em1$sd <- boot1$sd
    em2$sd <- boot2$sd

    result <- list()
    result$pvals <- pvals
    result$statistic <- z
    class(em1) <- "eim"
    class(em2) <- "eim"
    result$eim1 <- em1
    result$eim2 <- em2

    return(result)
}


#' Save an `eim` object to a file
#'
#' This function saves an `eim` object to a specified file format. Supported formats are
#' **RDS**, **JSON**, and **CSV**. The function dynamically extracts and saves all available
#' attributes when exporting to JSON. If the `prob` field exists, it is saved when using CSV;
#' otherwise, it yields an error.
#'
#' @param object An `eim` object.
#' @param filename A character string specifying the file path, including the desired file extension (`.rds`, `.json`, or `.csv`).
#' @param ... Additional arguments (currently unused but included for compatibility).
#'
#' @usage save_eim(object, filename, ...)
#'
#' @details
#' - If the file extension is **RDS**, the entire object is saved using `saveRDS()`.
#' - If the file extension is **JSON**, all available attributes of the object are stored in JSON format.
#' - If the file extension is **CSV**:
#'   - If the object contains a `prob` field, only that field is saved as a CSV.
#'   - Otherwise, returns an error.
#'
#' @return The function does not return anything explicitly but saves the object to the specified file.
#'
#' @seealso The [eim] object implementation.
#'
#' @examples
#' \donttest{
#' model <- eim(X = matrix(1:9, 3, 3), W = matrix(1:9, 3, 3))
#'
#' model <- run_em(model)
#'
#' td <- tempdir()
#' out_rds <- file.path(td, "model_results.rds")
#' out_json <- file.path(td, "model_results.json")
#' out_csv <- file.path(td, "model_results.csv")
#'
#' # Save as RDS
#' save_eim(model, filename = out_rds)
#'
#' # Save as JSON
#' save_eim(model, filename = out_json)
#'
#' # Save as CSV
#' save_eim(model, filename = out_csv)
#'
#' # Remove the files
#' files <- c(out_rds, out_json, out_csv)
#' file.remove(files)
#' }
#'
#' @name save_eim
#' @aliases save_eim()
#' @export
save_eim <- function(object, filename, ...) {
    # Ensure filename is a valid string
    if (!is.character(filename) || length(filename) != 1) {
        stop("Invalid filename. Please provide a valid file path as a character string.")
    }

    if (!inherits(object, "eim")) {
        stop("The object must be initialized with the `eim()` function.")
    }

    # Get file extension
    file_ext <- tools::file_ext(filename)

    # Save as RDS
    if (file_ext == "rds") {
        saveRDS(object, file = filename)
        message("Results saved as RDS: ", filename)

        # Save as JSON (with all available attributes)
    } else if (file_ext == "json") {
        json_data <- list()

        # Dynamically extract all attributes and store them
        for (name in names(object)) {
            val <- object[[name]]

            # if it's our 3-D array cond_prob, swap dim 1 <-> dim 3
            if (identical(name, "cond_prob") &&
                is.array(val) &&
                length(dim(val)) == 3) {
                val <- aperm(val, perm = c(3, 1, 2))
            }

            json_data[[name]] <- val
        }
        # Add the names of ballot boxes
        if (!is.null(object$X) && !is.null(rownames(object$X))) {
            json_data$ballotbox_id <- rownames(object$X)
        }
        # Add the names of candidates
        if (!is.null(object$X) && !is.null(colnames(object$X))) {
            json_data$candidates_id <- colnames(object$X)
        }
        # Add the names of ballot boxes
        if (!is.null(object$W) && !is.null(colnames(object$W))) {
            json_data$group_id <- colnames(object$W)
        }

        jsonlite::write_json(json_data, filename, pretty = TRUE, auto_unbox = TRUE, digits = 10)
        message("Results saved as JSON: ", filename)

        # Save as CSV
    } else if (file_ext == "csv") {
        if (!is.null(object$prob)) {
            write.csv(as.matrix(object$prob), filename, row.names = TRUE)
            message("Probability matrix saved as CSV: ", filename)
        } else {
            stop("The `run_em()` method must be called for saving a '.csv' file.")
        }
    } else {
        stop("Unsupported file format. Use '.rds', '.json', or '.csv'.")
    }
}

#' @noRd
#' @export
# write.csv.eim <- function(object, filename, ...) {
#    if (!inherits(object, "eim")) {
#        stop("The object must be initialized with the `eim()` function.")
#    }
#    if (!is.character(filename) || length(filename) != 1) {
#        stop("Invalid filename. Please provide a valid file path as a character string.")
#    }
#
#    # Get file extension
#    file_ext <- tools::file_ext(filename)
#
#    if (file_ext != "csv") {
#        stop("The filepath provided must end with '.csv'")
#    }
#
#    if (!is.null(object$prob)) {
#        write.csv(as.matrix(object$prob), filename, row.names = TRUE)
#        message("Probability matrix saved as CSV: ", filename)
#    } else {
#        stop("The `run_em()` method must be called for saving a '.csv' file.")
#    }
# }

#' @noRd
#' @export
# dput.eim <- function(object, filename, ...) {
#    if (!inherits(object, "eim")) {
#        stop("The object must be initialized with the `eim()` function.")
#    }
#    if (!is.character(filename) || length(filename) != 1) {
#        stop("Invalid filename. Please provide a valid file path as a character string.")
#    }
#
#    # Get file extension
#    file_ext <- tools::file_ext(filename)
#
#    if (file_ext != "rda") {
#        stop("The filepath provided must end with '.rda'")
#    }
#    saveRDS(object, file = filename)
#    message("Results saved as RDS: ", filename)
# }


# Maybe for a future patch, if it's needed, add the option to get the eaxct log-likelihood
# exactLL <- function(object, scale_factor = 1) {
#     if (is.null(object$X) || is.null(object$W) || is.null(object$prob)) {
#         stop("The object must contain X, W and prob matrices.")
#     }
#
#     # Applies a scaling
#     if (scale_factor != 1) {
#         object$X <- round(object$X / scale_factor)
#         object$W <- round(object$W / scale_factor)
#         object$W <- .dhondt_correction(object$W, object$X)
#     }
#
#     ll <- computeExactLL(t(object$X), object$W, object$prob)
#     ll
# }

test_that("LP adjustment matches candidate totals", {
    sim <- simulate_election(
        num_ballots = 8,
        num_candidates = 3,
        num_groups = 2,
        ballot_voters = rep(60, 8),
        lambda = 0.3,
        seed = 130
    )

    fit_lp <- run_em(
        X = sim$X,
        W = sim$W,
        method = "mult",
        maxiter = 4,
        maxtime = 2,
        compute_ll = FALSE,
        adjust_prob_cond_method = "lp",
        adjust_prob_cond_every = TRUE
    )

    fit_project <- run_em(
        X = sim$X,
        W = sim$W,
        method = "mult",
        maxiter = 4,
        maxtime = 2,
        compute_ll = FALSE,
        adjust_prob_cond_method = "project_lp",
        adjust_prob_cond_every = TRUE
    )

    xhat_lp <- expected_votes_from_q(sim$W, fit_lp$cond_prob)
    xhat_project <- expected_votes_from_q(sim$W, fit_project$cond_prob)

    expect_equal(xhat_lp, sim$X, tolerance = 1e-4)
    expect_equal(xhat_project, sim$X, tolerance = 1e-4)
}) 

test_that("bootstrap returns standard deviation and average probability", {
    sim <- simulate_election(
        num_ballots = 8,
        num_candidates = 3,
        num_groups = 2,
        ballot_voters = rep(40, 8),
        seed = 150
    )

    boot <- bootstrap(
        X = sim$X,
        W = sim$W,
        nboot = 3,
        method = "mult",
        maxiter = 4,
        maxtime = 2,
        compute_ll = FALSE
    )

    expect_s3_class(boot, "eim")
    expect_equal(dim(boot$sd), c(2, 3))
    expect_equal(dim(boot$avg_prob), c(2, 3))
    expect_true(all(boot$sd >= 0))
    expect_prob_matrix(boot$avg_prob)
})

test_that("run_em respects iteration controls", {
    sim <- simulate_election(
        num_ballots = 6,
        num_candidates = 3,
        num_groups = 2,
        ballot_voters = rep(30, 6),
        lambda = 0.4,
        seed = 120
    )

    fit <- run_em(
        X = sim$X,
        W = sim$W,
        method = "mult",
        maxiter = 3,
        miniter = 2,
        maxtime = 2,
        compute_ll = FALSE
    )

    expect_true(fit$iterations >= 2)
    expect_true(fit$iterations <= 3)
})

test_that("run_em rejects mismatched totals when disabled", {
    sim <- simulate_election(
        num_ballots = 4,
        num_candidates = 3,
        num_groups = 2,
        ballot_voters = rep(25, 4),
        seed = 121
    )

    sim$X[1, 1] <- sim$X[1, 1] + 1

    expect_error(
        run_em(X = sim$X, W = sim$W, allow_mismatch = FALSE),
        "Row-wise mismatch"
    )
})

test_that("simulate_election returns data", {
    sim <- simulate_election(
        num_ballots = 10,
        num_candidates = 3,
        num_groups = 2,
        seed = 123
    )

    expect_s3_class(sim, "eim")
    expect_equal(dim(sim$X), c(10, 3))
    expect_equal(dim(sim$W), c(10, 2))
    expect_true(is.matrix(sim$real_prob))
    expect_equal(dim(sim$real_prob), c(2, 3))
})

test_that("run_em returns probability matrix", {
    sim <- simulate_election(
        num_ballots = 10,
        num_candidates = 3,
        num_groups = 2,
        seed = 124
    )

    model <- eim(X = sim$X, W = sim$W)
    fit <- run_em(
        object = model,
        method = "mult",
        maxiter = 5,
        maxtime = 2,
        compute_ll = FALSE
    )

    expect_s3_class(fit, "eim")
    expect_true(is.matrix(fit$prob))
    expect_equal(dim(fit$prob), c(2, 3))
    expect_true(is.array(fit$cond_prob))
    expect_equal(dim(fit$cond_prob), c(2, 3, 10))
})

test_that("bootstrap returns standard deviations", {
    sim <- simulate_election(
        num_ballots = 8,
        num_candidates = 3,
        num_groups = 2,
        seed = 125
    )

    model <- eim(X = sim$X, W = sim$W)
    boot <- bootstrap(
        object = model,
        nboot = 3,
        method = "mult",
        maxiter = 3,
        maxtime = 2,
        compute_ll = FALSE
    )

    expect_s3_class(boot, "eim")
    expect_true(is.matrix(boot$sd))
    expect_equal(dim(boot$sd), c(2, 3))
})

test_that("save_eim writes JSON, RDS, and CSV", {
    sim <- simulate_election(
        num_ballots = 5,
        num_candidates = 3,
        num_groups = 2,
        ballot_voters = rep(20, 5),
        seed = 200
    )

    fit <- run_em(
        X = sim$X,
        W = sim$W,
        method = "mult",
        maxiter = 3,
        maxtime = 2,
        compute_ll = FALSE
    )

    out_rds <- tempfile(fileext = ".rds")
    out_json <- tempfile(fileext = ".json")
    out_csv <- tempfile(fileext = ".csv")

    save_eim(fit, out_rds)
    save_eim(fit, out_json)
    save_eim(fit, out_csv)

    expect_true(file.exists(out_rds))
    expect_true(file.exists(out_json))
    expect_true(file.exists(out_csv))

    json_data <- jsonlite::fromJSON(out_json)
    expect_true(all(c("X", "W", "prob") %in% names(json_data)))

    reloaded <- eim(json_path = out_json)
    expect_s3_class(reloaded, "eim")
    expect_equal(dim(reloaded$X), dim(fit$X))
    expect_equal(dim(reloaded$W), dim(fit$W))

    csv_data <- read.csv(out_csv)
    expect_equal(nrow(csv_data), nrow(fit$prob))
    expect_equal(ncol(csv_data), ncol(fit$prob) + 1)
})

test_that("S3 methods behave as expected", {
    sim <- simulate_election(
        num_ballots = 4,
        num_candidates = 3,
        num_groups = 2,
        ballot_voters = rep(20, 4),
        seed = 210
    )

    model <- eim(X = sim$X, W = sim$W)
    expect_error(as.matrix(model), "Probability matrix not available")

    fit <- run_em(
        object = model,
        method = "mult",
        maxiter = 3,
        maxtime = 2,
        compute_ll = FALSE
    )

    expect_true(is.matrix(as.matrix(fit)))
    summary_out <- summary(fit)
    expect_true(all(c("candidates", "groups", "ballots") %in% names(summary_out)))
    expect_error(logLik(fit), "run_em", fixed = TRUE)

    fit_ll <- run_em(
        X = sim$X,
        W = sim$W,
        method = "mult",
        maxiter = 3,
        maxtime = 2,
        compute_ll = TRUE
    )
    expect_true(is.numeric(logLik(fit_ll)))

    ll_fixed <- logLik(fit_ll, prob = fit_ll$prob, method = "multinomial")
    expect_true(is.numeric(ll_fixed))
    expect_true(is.finite(ll_fixed))

    expect_error(
        logLik(fit_ll, prob = matrix(1 / 3, nrow = 3, ncol = 3)),
        "Invalid 'prob' dimensions"
    )
})

test_that("waldtest returns p-values matrix", {
    sim1 <- simulate_election(
        num_ballots = 6,
        num_candidates = 3,
        num_groups = 2,
        ballot_voters = rep(30, 6),
        seed = 190
    )
    sim2 <- simulate_election(
        num_ballots = 6,
        num_candidates = 3,
        num_groups = 2,
        ballot_voters = rep(30, 6),
        seed = 191
    )

    result <- waldtest(
        X1 = sim1$X,
        W1 = sim1$W,
        X2 = sim2$X,
        W2 = sim2$W,
        nboot = 2,
        method = "mult",
        maxiter = 3,
        maxtime = 2,
        compute_ll = FALSE
    )

    expect_true(is.matrix(result$pvals))
    expect_equal(dim(result$pvals), c(2, 3))
    expect_s3_class(result$eim1, "eim")
    expect_s3_class(result$eim2, "eim")
})

test_that("run_em symmetric uses joint by default", {
    sim <- simulate_election(
        num_ballots = 6,
        num_candidates = 3,
        num_groups = 2,
        ballot_voters = rep(40, 6),
        lambda = 0.25,
        seed = 140
    )

    fit <- run_em(
        X = sim$X,
        W = sim$W,
        method = "mult",
        symmetric = TRUE,
        maxiter = 6,
        maxtime = 2,
        compute_ll = FALSE
    )

    expect_equal(fit$symmetric_weight_method, "joint")
    expect_equal(fit$symmetric_weights, c(original = 0.5, reverse = 0.5))
    expect_null(fit$prob_inv)
    expect_null(fit$cond_prob_inv)
    expect_null(fit$expected_outcome_inv)
    expect_prob_matrix(fit$prob)
    expect_prob_array(fit$cond_prob)
})

test_that("run_em symmetric average preserves inverse outputs", {
    sim <- simulate_election(
        num_ballots = 6,
        num_candidates = 3,
        num_groups = 2,
        ballot_voters = rep(40, 6),
        lambda = 0.25,
        seed = 146
    )

    fit <- run_em(
        X = sim$X,
        W = sim$W,
        method = "mult",
        symmetric = TRUE,
        symmetric_weight_method = "average",
        maxiter = 4,
        maxtime = 2,
        compute_ll = FALSE
    )

    expect_equal(fit$symmetric_weight_method, "average")
    expect_equal(fit$symmetric_weights, c(original = 0.5, reverse = 0.5))
    expect_true(is.matrix(fit$prob_inv))
    expect_true(is.array(fit$cond_prob_inv))
    expect_equal(dim(fit$prob_inv), c(ncol(sim$X), ncol(sim$W)))
    expect_equal(dim(fit$cond_prob_inv), c(ncol(sim$X), ncol(sim$W), nrow(sim$X)))
    expect_prob_matrix(fit$prob)
    expect_prob_array(fit$cond_prob)

    sums_inv <- apply(fit$cond_prob_inv, c(1, 3), sum)
    expect_true(all(abs(sums_inv - 1) < 1e-6))
})

test_that("run_em symmetric supports delta_ll weighting", {
    sim <- simulate_election(
        num_ballots = 6,
        num_candidates = 3,
        num_groups = 2,
        ballot_voters = rep(40, 6),
        lambda = 0.25,
        seed = 142
    )

    fit <- run_em(
        X = sim$X,
        W = sim$W,
        method = "mult",
        symmetric = TRUE,
        symmetric_weight_method = "delta_ll",
        maxiter = 4,
        maxtime = 2,
        compute_ll = TRUE
    )

    expect_true(is.matrix(fit$prob_inv))
    expect_true(is.array(fit$cond_prob_inv))
    expect_equal(fit$symmetric_weight_method, "delta_ll")
    expect_true(is.numeric(fit$symmetric_weights))
    expect_equal(names(fit$symmetric_weights), c("original", "reverse"))
    expect_equal(sum(fit$symmetric_weights), 1, tolerance = 1e-8)

    expect_true(is.numeric(fit$LL_ind))
    expect_true(is.numeric(fit$LL_rev_ind))
    expect_true(is.numeric(fit$dLL))
    expect_true(is.numeric(fit$dLL_rev))
    expect_true(is.numeric(fit$nu))
    expect_true(is.numeric(fit$nu_rev))
})

test_that("run_em symmetric supports mae_inverse weighting", {
    sim <- simulate_election(
        num_ballots = 6,
        num_candidates = 3,
        num_groups = 2,
        ballot_voters = rep(40, 6),
        lambda = 0.25,
        seed = 143
    )

    fit <- run_em(
        X = sim$X,
        W = sim$W,
        method = "mult",
        symmetric = TRUE,
        symmetric_weight_method = "mae_inverse",
        maxiter = 4,
        maxtime = 2,
        compute_ll = TRUE
    )

    expect_equal(fit$symmetric_weight_method, "mae_inverse")
    expect_true(is.numeric(fit$symmetric_weights))
    expect_equal(names(fit$symmetric_weights), c("original", "reverse"))
    expect_equal(sum(fit$symmetric_weights), 1, tolerance = 1e-8)
    expect_true(is.numeric(fit$err_forward))
    expect_true(is.numeric(fit$err_inverse))
})

test_that("run_em symmetric supports joint", {
    sim <- simulate_election(
        num_ballots = 6,
        num_candidates = 3,
        num_groups = 2,
        ballot_voters = rep(40, 6),
        lambda = 0.25,
        seed = 144
    )

    fit <- run_em(
        X = sim$X,
        W = sim$W,
        method = "mult",
        symmetric = TRUE,
        symmetric_weight_method = "joint",
        maxiter = 6,
        maxtime = 2,
        compute_ll = TRUE
    )

    expect_equal(fit$symmetric_weight_method, "joint")
    expect_equal(fit$symmetric_weights, c(original = 0.5, reverse = 0.5))
    expect_null(fit$prob_inv)
    expect_null(fit$cond_prob_inv)
    expect_null(fit$expected_outcome_inv)
    expect_false(isTRUE(fit$adjust_prob_cond_every))
    expect_prob_matrix(fit$prob)
    expect_prob_array(fit$cond_prob)
})

test_that("run_em joint supports lp with adjust_prob_cond_every TRUE/FALSE", {
    sim <- simulate_election(
        num_ballots = 6,
        num_candidates = 3,
        num_groups = 2,
        ballot_voters = rep(40, 6),
        lambda = 0.25,
        seed = 145
    )

    fit_false <- run_em(
        X = sim$X,
        W = sim$W,
        method = "mult",
        symmetric = TRUE,
        symmetric_weight_method = "joint",
        adjust_prob_cond_method = "lp",
        adjust_prob_cond_every = FALSE,
        maxiter = 6,
        maxtime = 2,
        compute_ll = FALSE
    )

    fit_true <- run_em(
        X = sim$X,
        W = sim$W,
        method = "mult",
        symmetric = TRUE,
        symmetric_weight_method = "joint",
        adjust_prob_cond_method = "lp",
        adjust_prob_cond_every = TRUE,
        maxiter = 6,
        maxtime = 2,
        compute_ll = FALSE
    )

    expect_false(isTRUE(fit_false$adjust_prob_cond_every))
    expect_true(isTRUE(fit_true$adjust_prob_cond_every))
    expect_prob_matrix(fit_false$prob)
    expect_prob_matrix(fit_true$prob)
    expect_prob_array(fit_false$cond_prob)
    expect_prob_array(fit_true$cond_prob)
})

test_that("run_em returns consistent outputs", {
    sim <- simulate_election(
        num_ballots = 10,
        num_candidates = 3,
        num_groups = 2,
        ballot_voters = rep(50, 10),
        lambda = 0.3,
        seed = 110
    )

    colnames(sim$X) <- paste0("C", seq_len(ncol(sim$X)))
    colnames(sim$W) <- paste0("G", seq_len(ncol(sim$W)))
    rownames(sim$X) <- paste0("B", seq_len(nrow(sim$X)))
    rownames(sim$W) <- rownames(sim$X)

    fit <- run_em(
        X = sim$X,
        W = sim$W,
        method = "mult",
        maxiter = 5,
        miniter = 1,
        maxtime = 2,
        compute_ll = FALSE
    )

    expect_s3_class(fit, "eim")
    expect_equal(dim(fit$prob), c(ncol(sim$W), ncol(sim$X)))
    expect_equal(dim(fit$cond_prob), c(ncol(sim$W), ncol(sim$X), nrow(sim$X)))
    expect_equal(dim(fit$expected_outcome), c(ncol(sim$W), ncol(sim$X), nrow(sim$X)))

    expect_prob_matrix(fit$prob)
    expect_prob_array(fit$cond_prob)

    expect_dimnames_match(fit$cond_prob, colnames(sim$W), colnames(sim$X), rownames(sim$X))
    expect_equal(dimnames(fit$prob)[[1]], colnames(sim$W))
    expect_equal(dimnames(fit$prob)[[2]], colnames(sim$X))

    expected_by_group <- apply(fit$expected_outcome, c(1, 3), sum)
    expect_equal(expected_by_group, t(sim$W), tolerance = 1e-6)
})

test_that("run_em approximates simulated probabilities", {
    sim <- simulate_election(
        num_ballots = 20,
        num_candidates = 3,
        num_groups = 3,
        ballot_voters = rep(80, 20),
        lambda = 0.2,
        seed = 111
    )

    fit <- run_em(
        X = sim$X,
        W = sim$W,
        method = "mult",
        maxiter = 30,
        miniter = 5,
        maxtime = 4,
        compute_ll = FALSE
    )

    error_mae <- mae(sim$real_prob, fit$prob)
    expect_lt(error_mae, 0.4)
})

test_that("run_em aggregates groups when group_agg is provided", {
    sim <- simulate_election(
        num_ballots = 6,
        num_candidates = 3,
        num_groups = 4,
        ballot_voters = rep(30, 6),
        seed = 180
    )

    group_agg <- c(2, 4)
    fit <- run_em(
        X = sim$X,
        W = sim$W,
        method = "mult",
        group_agg = group_agg,
        maxiter = 4,
        maxtime = 2,
        compute_ll = FALSE
    )

    expect_equal(fit$group_agg, group_agg)
    expect_equal(dim(fit$W_agg), c(6, 2))
    expect_equal(dim(fit$prob), c(2, 3))
})

test_that("get_agg_proxy returns aggregated results", {
    sim <- simulate_election(
        num_ballots = 6,
        num_candidates = 3,
        num_groups = 3,
        ballot_voters = rep(30, 6),
        seed = 181
    )

    proxy <- get_agg_proxy(
        X = sim$X,
        W = sim$W,
        nboot = 2,
        sd_threshold = 0.2,
        feasible = FALSE,
        method = "mult",
        maxiter = 3,
        maxtime = 2,
        compute_ll = FALSE
    )

    expect_s3_class(proxy, "eim")
    expect_true(!is.null(proxy$W_agg))
    expect_true(!is.null(proxy$group_agg))
    expect_true(is.matrix(proxy$sd))
})

test_that("get_agg_opt handles aggregation outputs", {
    sim <- simulate_election(
        num_ballots = 5,
        num_candidates = 3,
        num_groups = 3,
        ballot_voters = rep(30, 5),
        seed = 182
    )

    opt <- get_agg_opt(
        X = sim$X,
        W = sim$W,
        nboot = 2,
        sd_threshold = 0.2,
        method = "mult",
        maxiter = 3,
        maxtime = 2,
        compute_ll = FALSE
    )

    expect_s3_class(opt, "eim")
    if (!is.null(opt$group_agg)) {
        expect_true(!is.null(opt$W_agg))
        expect_true(is.matrix(opt$sd))
        expect_equal(ncol(opt$W_agg), length(opt$group_agg))
    }
})

#' Simulate an Election
#'
#' @description
#' This function simulates an election by creating matrices representing candidate votes `(X)` and voters' demographic group `(W)` across a specified number of ballot-boxes. It either (i) receives as input or (ii) generates a probability matrix `(prob)`, indicating how likely each demographic group is to vote for each candidate.
#'
#' By default, the number of voters per ballot box `(ballot_voters)` is set to a vector of 100 with
#' length `num_ballots`. You can optionally override this by providing a custom vector.
#'
#' Optional parameters are available to control the distribution of votes:
#' \itemize{
#'   \item \strong{`group_proportions`}: A vector of length `num_groups` specifying
#'         the overall proportion of each demographic group. Entries must sum to one and be non-negative.
#'   \item \strong{`prob`}: A user-supplied probability matrix of dimension
#'         (`num_groups` \eqn{\times} `num_candidates`). If provided, this matrix is used directly. Otherwise, voting probabilities for each group are drawn from a Dirichlet distribution.
#' }
#'
#' @param num_ballots
#'   Number of ballot boxes (`b`).
#'
#' @param num_candidates
#'   Number of candidates (`c`).
#'
#' @param num_groups
#'   Number of demographic groups (`g`).
#'
#' @param ballot_voters
#'   A vector of length `num_ballots` representing the number of voters per ballot
#'   box. Defaults to `rep(100, num_ballots)`.
#'
#' @param lambda
#' A numeric value between 0 and 1 that represents the fraction of voters that are randomly assigned to ballot-boxes. The remaining voters are assigned sequentially according to their demographic group.
#'   - `lambda = 0`: The assignment of voters to ballot-boxes is fully sequential in terms of their demographic group. This leads to a **high heterogeneity** of the voters' groups across ballot-boxes.
#'   - `lambda = 1`: The assignment of voters to ballot-boxes is fully random. This leads to a **low heterogeneity** of the voters' group across ballot-boxes.
#'
#'  Default value is set to `0.5`. See **Shuffling Mechanish** for more details.
#'
#' @param seed
#'   If provided, overrides the current global seed. Defaults to `NULL`.
#'
#' @param group_proportions
#'   Optional. A vector of length `num_groups` that indicates the fraction of voters that belong to each group. Default is that all groups are of the same size.
#'
#' @param prob
#'   Optional. A user-supplied probability matrix of dimension `(g x c)`.
#'   If provided, this matrix is used as the underlying voting probability distribution. If not supplied, each row is sampled from a Dirichlet distribution with each parameter set to one.
#'
#' @section Shuffling Mechanism:
#' Without loss of generality, consider an order relation of groups and ballot-boxes. The shuffling step is controlled by the `lambda` parameter and operates as follows:
#'
#' 1. **Initial Assignment**: Voters are assigned to each ballot-box sequentially according to their demographic
#' group. More specifically, the first ballot-boxes receive voters from the first group. Then, the next ballot-boxes receive voters from the second group. This continues until all voters have been assigned. Note that most ballot-boxes will contain voters from a single group (as long as the number of ballot-boxes exceeds the number of groups).
#'
#' 2. **Shuffling**: A fraction `lambda` of voters who have already been assigned is selected at random. Then, the ballot-box assignment of this sample is shuffled. Hence, different `lambda` values are interpreted as follows:
#'
#' 	- `lambda = 0` means no one is shuffled (the initial assignment remains).
#' 	- `lambda = 1` means all individuals are shuffled.
#' 	- Intermediate values like `lambda = 0.5` shuffle half the voters.
#'
#' Using a high level of `lambda` (greater than 0.7) is not recommended, as this could make identification of the voting probabilities difficult. This is because higher values of lambda induce similar ballot-boxes in terms of voters' group.
#'
#' @return An eim object containing:
#' \describe{
#'   \item{\code{X}}{A \code{(b x c)} matrix with candidates' votes for each ballot box.}
#'   \item{\code{W}}{A \code{(b x g)} matrix with voters' groups for each ballot-box.}
#'   \item{\code{real_prob}}{A \code{(g x c)} matrix with the probability that a voter from each group votes for each candidate. If prob is provided, it would equal such probability.}
#' 	 \item{\code{outcome}}{A \code{(b x g x c)} array with the number of votes for each candidate in each ballot box, broken down by group.}
#' }
#'
#' @references
#' The algorithm is fully explained in ['Thraves, C. Ubilla, P. and Hermosilla, D.: *"A Fast Ecological Inference Algorithm for the R×C Case"*](https://papers.ssrn.com/sol3/papers.cfm?abstract_id=4832834).
#'
#' @examples
#' # Example 1: Default usage with 200 ballot boxes, each having 100 voters
#' result1 <- simulate_election(
#'     num_ballots = 200,
#'     num_candidates = 3,
#'     num_groups = 5
#' )
#'
#' # Example 2: Using a custom ballot_voters vector
#' result2 <- simulate_election(
#'     num_ballots = 340,
#'     num_candidates = 3,
#'     num_groups = 7,
#'     ballot_voters = rep(200, 340)
#' )
#'
#' # Example 3: Supplying group_proportions
#' result3 <- simulate_election(
#'     num_ballots = 93,
#'     num_candidates = 3,
#'     num_groups = 4,
#'     group_proportions = c(0.3, 0.5, 0.1, 0.1)
#' )
#'
#' # Example 4: Providing a user-defined prob matrix
#' custom_prob <- matrix(c(
#'     0.9,  0.1,
#'     0.4,  0.6,
#'     0.25, 0.75,
#'     0.32, 0.68,
#'     0.2,  0.8
#' ), nrow = 5, byrow = TRUE)
#'
#' result4 <- simulate_election(
#'     num_ballots = 200,
#'     num_candidates = 2,
#'     num_groups = 5,
#'     lambda = 0.3,
#'     prob = custom_prob
#' )
#'
#' result4$real_prob == custom_prob
#' # The attribute of the output real_prob matches the input custom_prob.
#' @export
simulate_election <- function(num_ballots,
                              num_candidates,
                              num_groups,
                              ballot_voters = rep(100, num_ballots),
                              lambda = 0.5,
                              seed = NULL,
                              group_proportions = rep(1 / num_groups, num_groups),
                              prob = NULL) {
    # If user provides a seed, override the current global seed
    if (!is.null(seed)) {
        set.seed(seed)
    }

    # Validate length of ballot_voters
    if (length(ballot_voters) != num_ballots) {
        stop("`ballot_voters` must be a vector of length `num_ballots`.")
    }

    # Validate group proportions
    if (length(group_proportions) != num_groups) {
        stop("group_proportions must have length equal to num_groups.")
    }
    tol <- .Machine$double.eps^0.5
    if (abs(sum(group_proportions) - 1) > tol) {
        stop("group_proportions must sum to 1.")
    }
    if (lambda < 0 || lambda > 1) {
        stop("Lambda must be a decimal between 0 and 1.")
    }

    # Build W (b x g), the distribution of groups in each ballot box
    W <- matrix(0, nrow = num_ballots, ncol = num_groups)

    # Get the total amount of voters
    n <- sum(ballot_voters)

    # Group assignment according group proportions
    counts <- floor(n * group_proportions)
    # Case when it's not an exact distribution
    remainder <- n - sum(counts)
    if (remainder > 0) {
        # Arbitrarily distribute the "fractional" person to the highest fractional parts
        fractional_parts <- n * group_proportions - counts
        indices <- order(fractional_parts, decreasing = TRUE)
        counts[indices[seq_len(remainder)]] <- counts[indices[seq_len(remainder)]] + 1
    }
    # Assign each of the n people to a group (uniformly) in a vector 'omega0'
    omega0 <- rep(seq_len(num_groups), times = counts)
    # omega0 <- ceiling(seq_len(n) * num_groups / n)
    # Copy for shuffling
    omega <- omega0

    # Shuffle lambda * n individuals among groups
    size_shuffle <- floor(lambda * n)
    if (size_shuffle > 0) {
        v <- sample.int(n, size_shuffle, replace = FALSE)
        v_sorted <- sort(v)
        for (k in seq_len(size_shuffle)) {
            # Shuffle individuals
            omega[v[k]] <- omega0[v_sorted[k]]
        }
    }

    # Get the cumulative sum for later creating the partition between ballot boxes
    # This will allow defining a range of voters between values of a[]. For
    # example: a[2] - a[1] = voters of ballot box 1.
    a <- c(0, cumsum(ballot_voters))

    # Fill in W by counting how many from each group are in each box
    for (b_idx in seq_len(num_ballots)) {
        start_idx <- a[b_idx] + 1
        end_idx <- a[b_idx + 1]
        sub_omega <- omega[start_idx:end_idx]
        counts_bg <- tabulate(sub_omega, nbins = num_groups)
        W[b_idx, ] <- counts_bg
    }

    # Build the probality matrix (g x c)
    # 1) If 'prob' is provided, use it directly
    # 2) Otherwise, sample each row from a Dirichlet with alpha=1
    if (!is.null(prob)) {
        # Validate dimension
        if (!all(dim(prob) == c(num_groups, num_candidates))) {
            stop("`prob` must be a matrix with dimensions (num_groups x num_candidates).")
        }
        p <- prob
    } else {
        # Sample from Dirichlet(alpha=1)
        rdirichlet_1c <- function() {
            x <- rgamma(num_candidates, shape = 1)
            x / sum(x)
        }
        p <- matrix(0, nrow = num_groups, ncol = num_candidates)
        for (g_idx in seq_len(num_groups)) {
            p[g_idx, ] <- rdirichlet_1c()
        }
    }

    # Build X (b x c) by aggregating multinomial draws per group
    z_bgc <- array(
        0L,
        dim = c(num_groups, num_candidates, num_ballots),
        dimnames = list(
            group = seq_len(num_groups),
            candidate = seq_len(num_candidates),
            `ballot box` = seq_len(num_ballots)
        )
    )
    X <- matrix(0, nrow = num_ballots, ncol = num_candidates)
    for (b_idx in seq_len(num_ballots)) {
        for (g_idx in seq_len(num_groups)) {
            w_bg <- W[b_idx, g_idx]
            if (w_bg > 0) {
                # Draw from Multinomial(w_bg, p[g_idx, ])
                z_draw <- rmultinom(1, size = w_bg, prob = p[g_idx, ])
                z_bgc[g_idx, , b_idx] <- as.integer(z_draw)
                # Acumulate in X
                X[b_idx, ] <- X[b_idx, ] + z_draw
            }
        }
    }

    toReturn <- list(
        W = W,
        X = X,
        real_prob = p,
        outcome = z_bgc
    )

    # Return it as an eim object
    class(toReturn) <- "eim"

    return(toReturn)
}

test_that("simulate_election creates consistent data", {
    num_ballots <- 12
    num_candidates <- 3
    num_groups <- 2
    voters <- rep(40, num_ballots)

    sim <- simulate_election(
        num_ballots = num_ballots,
        num_candidates = num_candidates,
        num_groups = num_groups,
        ballot_voters = voters,
        lambda = 0.2,
        seed = 101
    )

    expect_s3_class(sim, "eim")
    expect_equal(dim(sim$X), c(num_ballots, num_candidates))
    expect_equal(dim(sim$W), c(num_ballots, num_groups))
    expect_equal(dim(sim$real_prob), c(num_groups, num_candidates))
    expect_equal(dim(sim$outcome), c(num_groups, num_candidates, num_ballots))

    expect_equal(rowSums(sim$W), voters)
    expect_equal(rowSums(sim$X), voters)

    outcome_by_group <- apply(sim$outcome, c(1, 3), sum)
    outcome_by_candidate <- apply(sim$outcome, c(2, 3), sum)
    expect_equal(outcome_by_group, t(sim$W), ignore_attr = TRUE)
    expect_equal(outcome_by_candidate, t(sim$X), ignore_attr = TRUE)

    expect_prob_matrix(sim$real_prob)
})

#' Extracts voting and demographic data matrices for a given electoral district in Chile.
#'
#' This function retrieves the voting results and demographic group counts for a given electoral district from the 2021 Chilean election dataset included in this package. The function returns an [`eim`] object that can be directly used in [`run_em`] or other estimation functions.
#'
#' The function builds the `X` matrix using the number of votes per candidate, and the `W` matrix using the number of voters in each demographic group (e.g., age ranges). Optionally, blank and null votes can be merged into a single additional column (considered as another candidate).
#'
#' Additionally, ballot boxes where the number of votes does not match the number of registered voters (i.e., those where `MISMATCH == TRUE`) can be excluded from the dataset by setting `remove_mismatch = TRUE`.
#'
#' @param elect_district A string indicating the name of the electoral district to extract (e.g., "NIEBLA"). See **Note**.
#'
#' @param region A string indicating the name of the region to extract (e.g, "DE TARAPACA"). See **Note**.
#'
#' @param merge_blank_null Logical indicating whether blank and null votes should be merged into a single column. Defaults to `TRUE`.
#'
#' @param remove_mismatch Logical indicating whether to remove ballot boxes with mismatched vote totals (where `MISMATCH == TRUE`). Defaults to `FALSE`.
#'
#' @param use_sex Logical indicating whether to use the sex from the voters instead of the age ranges. Defaults to `FALSE`.
#'
#' @return
#' An [`eim`] object with the following attributes:
#' - **X**: A matrix `(b x c)` with the number of votes per candidate (including a column for blank + null votes if `merge_blank_null = TRUE`).
#' - **W**: A matrix `(b x g)` with the number of voters per group (e.g., age ranges) for each ballot box.
#'
#' This object can be passed to functions like [`run_em`] or [`get_agg_proxy`] for estimation and group aggregation. See **Example**.
#'
#' Only one parameter is accepted among `elect_district` and `region`. If either both parameters are given, it will return an error. If neither of these two inputs is supplied, it will return an `eim` object with an aggregation corresponding to the whole dataset. To see all electoral districts and regions names, see the function [chile_election_2021].
#'
#' @examples
#' # Load data and create an eim object for the electoral district of "NIEBLA"
#' eim_obj <- get_eim_chile(elect_district = "NIEBLA", remove_mismatch = FALSE)
#'
#' # Use it to run the EM algorithm
#' result <- run_em(eim_obj, allow_mismatch = TRUE)
#'
#' # Use it with group aggregation
#' agg_result <- get_agg_proxy(
#'     object = eim_obj,
#'     sd_threshold = 0.05,
#'     allow_mismatch = TRUE,
#'     seed = 123
#' )
#'
#' agg_result$group_agg
#'
#' @seealso [chile_election_2021]
#' @aliases get_eim_chile()
#' @export
get_eim_chile <- function(elect_district = NULL,
                          region = NULL,
                          merge_blank_null = TRUE,
                          remove_mismatch = FALSE,
                          use_sex = FALSE) {
    df <- get("chile_election_2021")

    # Apply filtering only if exactly one of region or elect_district is provided
    if (!is.null(region) && is.null(elect_district)) {
        df_ed <- df[df$REGION == toupper(region), ]
        rownames(df_ed) <- paste(df_ed$ELECTORAL.DISTRICT, df_ed$BALLOT.BOX, sep = " - ")
    } else if (!is.null(elect_district) && is.null(region)) {
        df_ed <- df[df$ELECTORAL.DISTRICT == toupper(elect_district), ]
        rownames(df_ed) <- df_ed$BALLOT.BOX
    } else if (is.null(elect_district) && is.null(region)) {
        # If both are provided or both are NULL, use full data
        # Generate unique composite key for rownames
        df_ed <- df
        df_ed$row_id <- paste(df_ed$REGION, df_ed$ELECTORAL.DISTRICT, df_ed$BALLOT.BOX, sep = " - ")
        # Remove duplicates based on this composite key
        df_ed <- df_ed[!duplicated(df_ed$row_id), ]
        # Assign rownames
        rownames(df_ed) <- df_ed$row_id
    } else {
        stop("You cannot provide an electoral district and a region simultaneously.")
    }

    # Remove mismatches if applicable
    if (remove_mismatch && "MISMATCH" %in% names(df_ed)) {
        df_ed <- df_ed[df_ed$MISMATCH == FALSE, ]
    }

    if (nrow(df_ed) == 0) {
        # Handle the empty case
        stop("No rows matched the filter.")
    }

    # Extract candidate votes
    X <- df_ed[, c("C1", "C2", "C3", "C4", "C5", "C6", "C7", "BLANK.VOTES", "NULL.VOTES")]

    # Merge blank and null votes if requested
    if (merge_blank_null) {
        X$C8 <- X$BLANK.VOTES + X$NULL.VOTES
    }

    # Keep only candidate columns including merged C8
    X <- X[, c("C1", "C2", "C3", "C4", "C5", "C6", "C7", "C8")]
    X <- as.matrix(X)

    # Extract demographic groups
    if (!use_sex) {
        W <- as.matrix(df_ed[, c("X18.19", "X20.29", "X30.39", "X40.49", "X50.59", "X60.69", "X70.79", "X80.")])
    } else {
        W <- as.matrix(df_ed[, c("M", "F")])
    }

    return(eim(X = X, W = W))
}

test_that("get_eim_chile loads district data", {
    eim_obj <- get_eim_chile(elect_district = "APOQUINDO", remove_mismatch = FALSE)

    expect_s3_class(eim_obj, "eim")
    expect_true(is.matrix(eim_obj$X))
    expect_true(is.matrix(eim_obj$W))
    expect_equal(nrow(eim_obj$X), nrow(eim_obj$W))
    expect_true(ncol(eim_obj$X) >= 2)
    expect_true(ncol(eim_obj$W) >= 2)
})

#' @description According to the state of the algorithm (either computed or not), it prints a message with its most relevant parameters
#'
#' @return Doesn't return anything. Yields messages on the console.
#'
#' @examples
#' simulations <- simulate_elections(
#'     num_ballots = 20,
#'     num_candidates = 5,
#'     num_groups = 3,
#'     ballot_voters = rep(100, 20)
#' )
#'
#' model <- eim(simulations$X, simulations$W)
#' print(model) # Will print the X and W matrix.
#' run_em(model)
#' print(model) # Will print the X and W matrix among the EM results.
#' @noRd
#' @export
print.eim <- function(x, ...) {
    object <- x
    cat("eim ecological inference model\n")

    # Determine if truncation is needed
    truncated_X <- (nrow(object$X) > 5)
    truncated_W <- (nrow(object$W) > 5)
    is_aggregated <- !is.null(object$W_agg)
    is_method <- !is.null(object$method)
    dim <- if (is_aggregated) "a" else "g"

    cat("Candidates' vote matrix (X) [b x c]:\n")
    print(object$X[1:min(5, nrow(object$X)), ], drop = FALSE) # nolint
    if (truncated_X) cat(".\n.\n.\n") else cat("\n")

    cat("Group-level voter matrix (W) [b x g]:\n")
    print(object$W[1:min(5, nrow(object$W)), , drop = FALSE]) # nolint
    if (truncated_W) cat(".\n.\n.\n") else cat("\n")

    if (is_aggregated) {
        cat("Macro group-level voter matrix (W_agg) [b x a]:\n")
        print(object$W_agg[1:min(5, nrow(object$W_agg)), , drop = FALSE]) # nolint
        truncated_W_agg <- (nrow(object$W_agg) > 5)
        if (truncated_W_agg) cat(".\n.\n.\n") else cat("\n")
    }

    if (is_method) {
        cat(sprintf("Estimated probability [%s x c]:\n", dim))
        truncated_P <- (nrow(object$prob) > 5)
        print(round(object$prob[1:min(5, nrow(object$prob)), ], 3)) # nolint
        if (truncated_P) cat(".\n.\n.\n") else cat("\n")
    }
    if (!is.null(object$sd)) {
        cat(sprintf("Standard deviation of the estimated probability matrix [%s x c]:\n", dim))
        truncated_boot <- (nrow(object$sd) > 5)
        print(round(object$sd[1:min(5, nrow(object$sd)), ], 3)) # nolint
        if (truncated_boot) cat(".\n.\n.\n") else cat("\n")
    }

    if (is_method) {
        cat("Method:\t", object$method, "\n")
        if (object$method == "mcmc") {
            cat("Step size (M):", object$mcmc_stepsize, "\n")
            cat("Samples (S):", object$mcmc_samples, "\n")
        } else if (object$method == "mvn_cdf") {
            cat("Montecarlo method:", object$mvncdf_method, "\n")
            cat("Montecarlo iterations:", object$mvncdf_samples, "\n")
            cat("Montecarlo error:", object$mvncdf_error, "\n")
        }
        cat("Total Iterations:", object$iterations, "\n")
        cat("Total Time (s):", object$time, "\n")
        if (!is.null(object$logLik)) {
            cat("Log-likelihood:", tail(object$logLik, 1), "\n")
        }
    }
}

#' @description Shows, in form of a list, a selection of the most important attributes. It'll retrieve the method, number of candidates, ballots, groups and total votes as well as the principal results of the EM algorithm.
#'
#' @param object An "eim" object.
#' @param ... Additional arguments that are ignored.
#' @return A list with the chosen attributes
#'
#' @examples
#'
#' simulations <- simulate_elections(
#'     num_ballots = 5,
#'     num_candidates = 3,
#'     num_groups = 2,
#'     ballot_voters = rep(100, 5)
#' )
#'
#' model <- eim(simulations$X, simulations$W)
#' summarised <- summary(model)
#' names(summarised)
#' # "candidates" "groups" "ballots" "votes"
#' @noRd
#' @export
summary.eim <- function(object, ...) {
    # Generates the list with the core attribute.
    object_core_attr <- list(
        candidates = ncol(object$X),
        groups = ncol(object$W),
        ballots = nrow(object$X)
    )

    # A list with attributes to display if the EM is computed.
    if (!is.null(object$method)) {
        object_run_em_attr <- list(
            method = object$method,
            prob = object$prob,
            logLik = object$logLik,
            status = object$status
        )
    } else {
        object_run_em_attr <- list(
            status = -1 # Not computed yet
        )
    }

    # Display sd if the bootstrapping method has been called.
    if (!is.null(object$sd)) {
        object_run_em_attr$sd <- object$sd
    }
    if (!is.null(object$avg_prob)) {
        object_run_em_attr$avg_prob <- object$avg_prob
    }
    final_list <- c(object_core_attr, object_run_em_attr)
    final_list
}

#' @title Plot estimated probabilities
#' @description
#'   Plots the estimated probabilities as pie charts using `ggplot2`, one per row of the probability matrix.
#'   Each slice displays its percentage label.
#'
#' @param x An "eim" object.
#' @param title Title for the plot.
#' @param legend_title Title for the legend.
#' @param color_scale A vector of colors or a palette for the candidates.
#' @param min_pct Minimum percentage required to display a label.
#' @param pies_per_row Number of pie charts to display per row. Defaults to `ceiling(sqrt(G))`,
#'   where `G` is the number of groups.
#' @param ... Additional arguments that are ignored.
#'
#' @return Returns a `ggplot2` object representing the pie charts.
#'
#' @examples
#' \donttest{
#' sim <- simulate_election(
#'     num_ballots = 100,
#'     num_candidates = 4,
#'     num_groups = 5,
#'     ballot_voters = rep(40, 100),
#'     seed = 42
#' )
#' fit <- run_em(sim, maxiter = 5)
#'
#' plot(fit, title = "Estimated probabilities", legend_title = "Candidates", min_pct = 7)
#' }
#' @export
plot.eim <- function(x,
                     title = "Estimated probabilities",
                     legend_title = "Candidates",
                     color_scale = NULL,
                     min_pct = 3,
                     pies_per_row = NULL,
                     ...) {
    if (!requireNamespace("ggplot2", quietly = TRUE)) {
        stop("plot.eim: package 'ggplot2' is required for plotting.")
    }
    object <- x
    prob <- object$prob

    W_use <- if (!is.null(object$W_agg)) object$W_agg else object$W
    G <- ncol(W_use)

    P <- as.matrix(prob)

    row_names <- rownames(P)
    col_names <- colnames(P)
    if (is.null(row_names)) row_names <- paste0("Row ", seq_len(nrow(P)))
    if (is.null(col_names)) col_names <- paste0("Col ", seq_len(ncol(P)))

    if (is.function(color_scale)) {
        colors <- color_scale(ncol(P))
    } else {
        colors <- color_scale
    }
    if (is.null(colors)) {
        colors <- grDevices::colorRampPalette(c("#4575B4", "#F7F7F7", "#D73027"))(ncol(P))
    } else if (length(colors) < ncol(P)) {
        colors <- grDevices::colorRampPalette(colors)(ncol(P))
    }

    df <- expand.grid(row = row_names, col = col_names, stringsAsFactors = FALSE)
    df$value <- as.vector(P)
    df$row <- factor(df$row, levels = row_names)
    df$col <- factor(df$col, levels = col_names)
    df$label <- ifelse(100 * df$value >= min_pct, sprintf("%.1f%%", 100 * df$value), "")

    if (is.null(pies_per_row)) {
        pies_per_row <- ceiling(sqrt(G))
    }
    pies_per_row <- max(1L, as.integer(pies_per_row))

    plot_obj <- ggplot2::ggplot(df, ggplot2::aes(x = 0.5, y = value, fill = col)) +
        ggplot2::geom_col(width = 1, color = NA) +
        ggplot2::coord_polar(theta = "y", clip = "off") +
        ggplot2::scale_x_continuous(limits = c(0, 1), expand = c(0, 0)) +
        ggplot2::facet_wrap(~row, strip.position = "top", ncol = pies_per_row) +
        ggplot2::geom_text(
            ggplot2::aes(label = label),
            position = ggplot2::position_stack(vjust = 0.5),
            size = 3
        ) +
        ggplot2::scale_fill_manual(values = colors, name = legend_title) +
        ggplot2::labs(title = title) +
        ggplot2::theme_void() +
        ggplot2::theme(
            plot.title = ggplot2::element_text(hjust = 0.5, face = "bold"),
            strip.background = ggplot2::element_blank(),
            strip.text = ggplot2::element_text(face = "bold"),
            legend.position = "right"
        )

    print(plot_obj)
    invisible(plot_obj)
}

#' Returns the object estimated probability.
#'
#' @param object An "eim" object.
#' @param ... Additional arguments that are ignored.
#' @return The global probability matrix
#' @noRd
#' @export
as.matrix.eim <- function(x, ...) {
    object <- x
    if (is.null(object$prob)) {
        stop(paste0(
            "Probability matrix not available. Run 'run_em()'."
        ))
    }
    prob <- object$prob

    P <- as.matrix(prob)
    return(P)
}

#' @title Extract log-likelihood
#' @description
#'   Return the log-likelihood of the last EM iteration, or compute it for a fixed probability matrix.
#'
#' @param object An `eim` object
#' @param prob Optional probability matrix `(g x c)` used to compute a fixed-parameter log-likelihood.
#' @param method Optional E-step method. Defaults to the fitted method in `object`, or `"mult"` if unavailable.
#' @param ... Additional optional parameters for fixed-probability evaluation:
#'   `mcmc_stepsize`, `mcmc_samples`, `mvncdf_method`, `mvncdf_error`, `mvncdf_samples`,
#'   `miniter`, `adjust_prob_cond_method`, and `adjust_prob_cond_every`.
#'
#' @return A numeric value with the log-likelihood.
#' @noRd
#' @export
logLik.eim <- function(object, prob = NULL, method = NULL, ...) {
    if (!inherits(object, "eim")) {
        stop("The object must be initialized with the `eim()` function.")
    }

    if (is.null(prob)) {
        if (is.null(object$logLik)) {
            stop("The `run_em()` method must be called for getting the log-likelihood.")
        }
        return(tail(object$logLik, 1))
    }

    method_alias <- function(m) {
        m <- trimws(tolower(as.character(m)))
        if (m == "multinomial") {
            return("mult")
        }
        m
    }

    method_use <- if (is.null(method)) {
        if (!is.null(object$method)) object$method else "mult"
    } else {
        method
    }
    method_use <- method_alias(method_use)
    valid_methods <- c("mult", "mcmc", "mvn_cdf", "mvn_pdf", "exact")
    if (!(method_use %in% valid_methods)) {
        stop("Invalid 'method'. Must be one of: ", paste(valid_methods, collapse = ", "))
    }

    if (is.null(object$X) || is.null(object$W)) {
        stop("The object must include 'X' and 'W' matrices.")
    }

    W_use <- if (is.null(object$W_agg)) object$W else object$W_agg
    prob_matrix <- as.matrix(prob)
    expected_dims <- c(ncol(W_use), ncol(object$X))
    if (!all(dim(prob_matrix) == expected_dims)) {
        stop(
            "Invalid 'prob' dimensions. Expected (", expected_dims[1], " x ", expected_dims[2], ")."
        )
    }

    dots <- list(...)
    step_size <- if (!is.null(dots$mcmc_stepsize)) dots$mcmc_stepsize else if (!is.null(object$mcmc_stepsize)) object$mcmc_stepsize else 3000
    samples <- if (!is.null(dots$mcmc_samples)) dots$mcmc_samples else if (!is.null(object$mcmc_samples)) object$mcmc_samples else 1000
    monte_method <- if (!is.null(dots$mvncdf_method)) dots$mvncdf_method else if (!is.null(object$mvncdf_method)) object$mvncdf_method else "genz"
    monte_error <- if (!is.null(dots$mvncdf_error)) dots$mvncdf_error else if (!is.null(object$mvncdf_error)) object$mvncdf_error else 1e-3
    monte_iter <- if (!is.null(dots$mvncdf_samples)) dots$mvncdf_samples else if (!is.null(object$mvncdf_samples)) object$mvncdf_samples else 5000
    miniter <- if (!is.null(dots$miniter)) dots$miniter else if (!is.null(object$miniter)) object$miniter else 0
    lp_method <- if (!is.null(dots$adjust_prob_cond_method)) dots$adjust_prob_cond_method else if (!is.null(object$adjust_prob_cond_method)) object$adjust_prob_cond_method else ""
    project_every <- if (!is.null(dots$adjust_prob_cond_every)) dots$adjust_prob_cond_every else if (!is.null(object$adjust_prob_cond_every)) object$adjust_prob_cond_every else FALSE

    ll <- EMLogLikFromProb(
        t(object$X),
        W_use,
        prob_matrix,
        method_use,
        as.integer(step_size),
        as.integer(samples),
        as.character(monte_method),
        as.numeric(monte_error),
        as.integer(monte_iter),
        as.integer(miniter),
        as.character(lp_method),
        as.logical(project_every)
    )
    as.numeric(ll)
}

#' Internal function!
#' Validates all of the 'run_em' arguments
#' @noRd
.validate_compute <- function(args) {
    # General checks: Vectors aren't accepted
    # if (any(sapply(args, function(x) length(x) > 1))) {
    #    stop("run_em:\tInvalid input: no vector inputs allowed")
    # }

    object_provided <- "object" %in% names(args) || "object1" %in% names(args)
    x_provided <- "X" %in% names(args) || "X1" %in% names(args)
    w_provided <- "W" %in% names(args) || "W1" %in% names(args)
    xw_provided <- x_provided || w_provided
    json_provided <- "json_path" %in% names(args)

    if (x_provided + w_provided == 1) {
        stop("If providing a matrix, 'X' and 'W' must be provided.")
    }

    if (sum(object_provided, xw_provided, json_provided) != 1) {
        stop(
            "You must provide exactly one of the following:\n",
            "(1)\tan `eim` object (initialized with `eim`)\n",
            "(2)\t`X` and `W`\n",
            "(3)\ta `json_path`"
        )
    }

    # Mismatch argument
    if ("allow_mismatch" %in% names(args)) {
        if (!is.logical(args$allow_mismatch)) {
            stop("run_em: Invalid 'allow_mismatch'. It has to be a boolean")
        }
    }

    # Method argument
    valid_methods <- c("mcmc", "exact", "mvn_cdf", "mvn_pdf", "mult", "metropolis")
    if ("method" %in% names(args) &&
        (!is.character(args$method) || length(args$method) != 1 || !(args$method %in% valid_methods))) {
        stop("Invalid 'method'. Must be one of: ", paste(valid_methods, collapse = ", "))
    }

    valid_symmetric_weight_methods <- c("average", "delta_ll", "mae_inverse", "joint")
    if ("symmetric_weight_method" %in% names(args) &&
        (!is.character(args$symmetric_weight_method) ||
            length(args$symmetric_weight_method) != 1 ||
            !(args$symmetric_weight_method %in% valid_symmetric_weight_methods))) {
        stop(
            "Invalid 'symmetric_weight_method'. Must be one of: ",
            paste(valid_symmetric_weight_methods, collapse = ", ")
        )
    }

    # Initial prob argument
    # valid_p_methods <- c("group_proportional", "proportional", "uniform", "random", "mult", "mcmc", "mvn_cdf", "mvn_pdf", "exact")
    # if ("initial_prob" %in% names(args) && (!is.matrix(args$initial_prob) ||
    #     (!is.character(args$initial_prob) || length(args$initial_prob) != 1 || !(args$initial_prob %in% valid_p_methods)))) {
    #     stop("Invalid 'initial_prob'. Must be one of: ", paste(valid_p_methods, collapse = ", "))
    # }

    if ("maxiter" %in% names(args)) {
        if (!is.numeric(args$maxiter) || as.integer(args$maxiter) != args$maxiter || args$maxiter < 1) { # Infinite are valid, skip this case
            stop("Invalid 'maxiter'. Must be a positive integer.")
        }
    }

    if ("ll_threshold" %in% names(args)) {
        if (!is.infinite(args$ll_threshold) && (!is.numeric(args$ll_threshold) || args$ll_threshold < 0)) { # Infinite are valid, skip this case
            stop("Invalid 'll_threshold'. Must be a positive numeric or infinite value.")
        }
    }

    # Maxtime argument
    if ("maxtime" %in% names(args) &&
        (!is.numeric(args$maxtime) || args$maxtime < 0)) {
        stop("Invalid 'maxtime'. Must be positive.")
    }

    # Stop threshold argument
    if ("param_threshold" %in% names(args)) {
        if (!is.infinite(args$param_threshold) && (!is.numeric(args$param_threshold) || args$param_threshold < 0)) {
            stop("run_em: Invalid 'param_threshold'. Must be a positive numeric or an infinite value.")
        }
        if (args$param_threshold >= 1) {
            warning("Warning: A 'param_threshold' greater or equal than one will always be true after the first iteration.")
        }
        if ("compute_ll" %in% names(args) && !args$compute_ll && is.infinite(args$param_threshold)) {
            stop("You must provide a parameter threshold if 'compute_ll' is FALSE.")
        }
    }

    # Verbose argument
    if ("verbose" %in% names(args) && !is.logical(args$verbose)) {
        stop("run_em: Invalid 'verbose'. It has to be a boolean.")
    }

    # mcmc: mcmc_stepsize argument
    if ("mcmc_stepsize" %in% names(args)) {
        if (!is.numeric(args$mcmc_stepsize) || as.integer(args$mcmc_stepsize) != args$mcmc_stepsize || args$mcmc_stepsize < 0) {
            stop("run_em: Invalid 'mcmc_stepsize'. Must be a positive integer.")
        }
        if (args$mcmc_stepsize < 15) {
            warning("Warning: A small 'mcmc_stepsize' could lead to highly correlated samples.")
        }
    }

    # mcmc: Samples argument
    if ("samples" %in% names(args) &&
        (!is.numeric(args$mcmc_samples) || as.integer(args$mcmc_samples) != args$mcmc_samples || args$mcmc_samples < 0)) {
        stop("run_em: Invalid 'mcmc_samples'. Must be a positive integer.")
    }

    # metropolis: Samples argument
    if ("metropolis_iter" %in% names(args) &&
        (!is.numeric(args$metropolis_iter) || as.integer(args$metropolis_iter) != args$metropolis_iter || args$metropolis_iter < 0)) {
        stop("run_em: Invalid 'metropolis_iter'. Must be a positive integer.")
    }

    # CDF: Mc_method argument
    valid_cdf_methods <- c("genz", "genz2")
    if ("mvncdf_method
" %in% names(args) &&
        (!is.character(args$mvncdf_method) || !args$mvncdf_method
            %in% valid_cdf_methods)) {
        stop("run_em: Invalid 'mvncdf_method
'. Must be one of: ", paste(valid_cdf_methods, collapse = ", "))
    }

    # CDF: Mc_error argument
    if ("mvncdf_error" %in% names(args) &&
        (!is.numeric(args$mvncdf_error) || args$mvncdf_error <= 0)) {
        stop("run_em: Invalid 'mvncdf_error'. Must be a positive number.")
    }

    # CDF: Mc_error argument
    if ("mvncdf_samples" %in% names(args) &&
        (!is.numeric(args$mvncdf_samples) || as.integer(args$mvncdf_samples) != args$mvncdf_samples || args$mvncdf_samples < 0)) {
        stop("run_em: Invalid 'mvncdf_samples'. Must be a positive integer.")
    }

    # Check mismatch
    if ("mismatch" %in% names(args)) {
        if (!is.logical(args$mismatch)) {
            stop("run_em: Invalid 'mismatch'. Must be a boolean value.")
        }
        # if ("method" %in% names(args) && "method" %in% c("exact")) {
        #    stop("run_em: Mismatched results are not supported when using 'exact'.")
        # }
    }

    # Include nboot aswell if bootstrapping is provided
    if ("nboot" %in% names(args) &&
        (!is.numeric(args$nboot) || as.integer(args$nboot) != args$nboot || args$nboot < 0)) {
        stop("Bootstrap: Invalid 'nboot'. Must be a positive integer.")
    }

    valid_sd_methods <- c("maximum", "average")
    if ("sd_statistic" %in% names(args) &&
        (!is.character(args$sd_statistic) || length(args$sd_statistic) != 1 || !(args$sd_statistic %in% valid_sd_methods))) {
        stop("Invalid 'sd_statistic'. Must be one of: ", paste(valid_sd_methods, collapse = ", "))
    }

    if ("sd_threshold" %in% names(args) &&
        (!is.numeric(args$sd_threshold) || args$sd_threshold <= 0)) {
        stop("Invalid 'sd_threshold'. Must be a positive number.")
    }

    if ("alternative" %in% names(args) && !args$alternative %in% c("two.sided", "greater", "less")) {
        stop("Invalid 'alternative'. Must be one of: two.sided, greater, less")
    }

    valid_lp_methods <- c("", "lp", "project_lp")
    if ("adjust_prob_cond_method" %in% names(args) &&
        (!is.character(args$adjust_prob_cond_method) || !(args$adjust_prob_cond_method %in% valid_lp_methods))) {
        stop("Invalid 'adjust_prob_cond_method'. Must be one of: ", paste(valid_lp_methods, collapse = ", "))
    }

    if ("adjust_prob_cond_every" %in% names(args)) {
        if (!is.logical(args$adjust_prob_cond_every)) {
            stop("Invalid 'adjust_prob_cond_every'. Must be a boolean value.")
        }
        if ("adjust_prob_cond_method" %in% names(args) && args$adjust_prob_cond_method == "") {
            warning("You provided 'adjust_prob_cond_every' but not 'adjust_prob_cond_method'. The former will be ignored.")
        }
    }
}


#' Internal function!
#'
#' Validate the 'eim' object inputs
#'
#' @param X A matrix representing candidate votes per ballot box.
#' @param W A matrix representing group votes per ballot box.
#' @return Stops execution if validation fails.
#' @noRd
.validate_eim <- function(X, W) {
    # Ensure X and W are provided
    if (is.null(X) || is.null(W)) {
        stop("Either provide X and W matrices, or a valid JSON path containing them.")
    }

    if (!is.matrix(X) || !is.matrix(W)) {
        stop("'X' and 'W' must be matrices.")
    }

    # Ensure they are matrices
    X <- as.matrix(X)
    W <- as.matrix(W)

    # Check matching dimensions
    if (nrow(X) != nrow(W)) {
        stop(
            "Mismatch in the number of ballot boxes: 'X' has ", nrow(X),
            " rows, but 'W' has ", nrow(W), " rows."
        )
    }

    # Check minimum column constraints
    if (ncol(X) < 2) {
        stop("Candidate matrix 'X' must have at least 2 columns.")
    }
    if (ncol(W) < 2) {
        # stop("Group matrix 'W' must have at least 2 columns.")
    }

    # Check for missing values
    if (any(is.na(X)) || any(is.na(W))) {
        stop("Matrices 'X' and 'W' cannot contain missing values (NA).")
    }

    TRUE
}

#' Internal function!
#'
#' Validate the 'eim' object JSON path
#'
#' @param json_path A path to a JSON file containing `"X"` and `"W"`.
#' @return A list with the `"X"` and `"W"` matrices. Stops execution if validation fails.
#' @noRd
.validate_json_eim <- function(json_path) {
    if (!file.exists(json_path)) {
        stop("The specified JSON file does not exist: ", json_path)
    }

    data <- tryCatch(
        jsonlite::fromJSON(json_path),
        error = function(e) stop("Failed to read JSON file: ", e$message)
    )

    # Validate JSON contents
    if (!all(c("X", "W") %in% names(data))) {
        stop("JSON file must contain the keys 'X' (candidate matrix) and 'W' (group matrix)")
    }

    if (is.null(data$X) || is.null(data$W)) {
        stop("'X' and 'W' cannot be NULL in the JSON file")
    }

    result <- list(
        X = as.matrix(data$X),
        W = as.matrix(data$W)
    )

    result
}

#' Internal function!
#'
#' Normalize each row of a probability matrix so every row sums to one.
#'
#' @param prob_matrix Numeric matrix with one probability vector per row.
#' @return A matrix with finite, non-negative rows normalized to one.
#' @noRd
.normalize_prob_rows <- function(prob_matrix) {
    prob_matrix[!is.finite(prob_matrix) | prob_matrix < 0] <- 0
    row_sums <- rowSums(prob_matrix)
    valid_rows <- is.finite(row_sums) & row_sums > 0

    if (any(valid_rows)) {
        prob_matrix[valid_rows, ] <- sweep(
            prob_matrix[valid_rows, , drop = FALSE],
            1,
            row_sums[valid_rows],
            "/"
        )
    }

    if (any(!valid_rows)) {
        prob_matrix[!valid_rows, ] <- rep(1 / ncol(prob_matrix), ncol(prob_matrix))
    }

    prob_matrix
}

#' Internal function!
#'
#' Normalize the last dimension of a 3d-array into valid probability vectors.
#'
#' @param arr3 Numeric 3d-array where the last dimension is normalized.
#' @return A 3d-array with finite, non-negative slices summing to one.
#' @noRd
.normalize_cube_last_dim <- function(arr3) {
    num_ballots <- dim(arr3)[1]
    num_rows <- dim(arr3)[2]
    num_cols <- dim(arr3)[3]

    for (ballot in seq_len(num_ballots)) {
        for (row in seq_len(num_rows)) {
            values <- arr3[ballot, row, ]
            values[!is.finite(values) | values < 0] <- 0
            values_sum <- sum(values)

            if (is.finite(values_sum) && values_sum > 0) {
                arr3[ballot, row, ] <- values / values_sum
            } else {
                arr3[ballot, row, ] <- rep(1 / num_cols, num_cols)
            }
        }
    }

    arr3
}

#' Internal function!
#'
#' Perform the M-step from ballot-box conditional probabilities.
#'
#' @param q_array A `(g x c x b)` array with conditional probabilities.
#' @param W_matrix A `(b x g)` matrix with group totals.
#' @return A `(g x c)` matrix with updated probabilities.
#' @noRd
.mstep_from_q <- function(q_array, W_matrix) {
    q_bgc <- aperm(q_array, c(3, 1, 2))
    weighted_q <- sweep(q_bgc, c(1, 2), W_matrix, "*")
    numerator <- apply(weighted_q, c(2, 3), sum)
    denominator <- colSums(W_matrix)
    probabilities <- sweep(numerator, 1, denominator, "/")

    .normalize_prob_rows(probabilities)
}

#' Internal function!
#'
#' Evaluate the log-likelihood of a fixed probability matrix under an EM method.
#'
#' @param object An `eim` object providing method-specific controls.
#' @param X_matrix A candidate-vote matrix.
#' @param W_matrix A group-vote matrix.
#' @param prob_matrix A probability matrix to evaluate.
#' @param method Character string with the EM method name.
#' @param miniter Minimum number of iterations stored in the temporary object.
#' @param adjust_prob_cond_method Character string controlling probability adjustment.
#' @param adjust_prob_cond_every Boolean indicating whether to project at every iteration.
#' @return A numeric scalar with the evaluated log-likelihood.
#' @noRd
.run_em_loglik_from_prob <- function(object,
                                     X_matrix,
                                     W_matrix,
                                     prob_matrix,
                                     method,
                                     miniter,
                                     adjust_prob_cond_method,
                                     adjust_prob_cond_every) {
    ll_object <- list(
        X = as.matrix(X_matrix),
        W = as.matrix(W_matrix),
        method = method,
        mcmc_stepsize = if (!is.null(object$mcmc_stepsize)) object$mcmc_stepsize else 3000,
        mcmc_samples = if (!is.null(object$mcmc_samples)) object$mcmc_samples else 1000,
        mvncdf_method = if (!is.null(object$mvncdf_method)) object$mvncdf_method else "genz",
        mvncdf_error = if (!is.null(object$mvncdf_error)) object$mvncdf_error else 1e-3,
        mvncdf_samples = if (!is.null(object$mvncdf_samples)) object$mvncdf_samples else 5000,
        miniter = miniter,
        adjust_prob_cond_method = adjust_prob_cond_method,
        adjust_prob_cond_every = adjust_prob_cond_every
    )
    class(ll_object) <- "eim"

    as.numeric(logLik(ll_object, prob = prob_matrix, method = method))
}

#' Internal function!
#'
#' Build the standard dimnames used by `run_em()` 3d outputs.
#'
#' @param object An `eim` object with candidate and ballot-box names.
#' @param W_matrix A group matrix providing group names.
#' @return A list of dimnames ordered as groups, candidates, ballot-boxes.
#' @noRd
.run_em_dimnames <- function(object, W_matrix) {
    list(
        colnames(W_matrix),
        colnames(object$X),
        rownames(object$X)
    )
}

#' Internal function!
#'
#' Retrieve the group matrix used internally by `run_em()`.
#'
#' @param object An `eim` object.
#' @return `object$W_agg` when available, otherwise `object$W`.
#' @noRd
.run_em_working_group_matrix <- function(object) {
    if (is.null(object$W_agg)) {
        object$W
    } else {
        object$W_agg
    }
}

#' Internal function!
#'
#' Apply a group aggregation specification to the group matrix of an `eim` object.
#'
#' @param object An `eim` object.
#' @param group_agg A vector describing how to aggregate `W` columns.
#' @return The updated `eim` object, including `W_agg` and `group_agg` when requested.
#' @noRd
.run_em_apply_group_agg <- function(object, group_agg) {
    if (is.null(group_agg)) {
        return(object)
    }

    sizes <- diff(c(0, group_agg))
    rep_labels <- rep(seq_along(sizes), sizes)
    groups <- split(seq_len(ncol(object$W)), rep_labels)
    W_agg <- do.call(
        cbind,
        lapply(groups, function(cols) rowSums(object$W[, cols, drop = FALSE]))
    )
    rownames(W_agg) <- rownames(object$W)

    object$W_agg <- W_agg
    object$group_agg <- group_agg
    object
}

#' Internal function!
#'
#' Prepare the input object used by `run_em()` before dispatching the EM routine.
#'
#' @param object Optional `eim` object provided by the user.
#' @param X Candidate-vote matrix.
#' @param W Group-vote matrix.
#' @param json_path Optional JSON path used to build the object.
#' @param scale_factor Numeric scaling factor applied to `X` and `W`.
#' @param method Character string with the selected EM method.
#' @param allow_mismatch Boolean indicating whether row mismatches are allowed.
#' @param group_agg Optional aggregation specification for `W`.
#' @return A prepared `eim` object ready to run EM.
#' @noRd
.run_em_prepare_object <- function(object,
                                   X,
                                   W,
                                   json_path,
                                   scale_factor,
                                   method,
                                   allow_mismatch,
                                   group_agg) {
    if (is.null(object)) {
        object <- eim(X = X, W = W, json_path = json_path)
    } else if (!inherits(object, "eim")) {
        stop("run_em: The object must be initialized with the eim() function.")
    }

    if (scale_factor != 1) {
        object$X <- round(object$X / scale_factor)
        object$W <- round(object$W / scale_factor)
    }

    mismatch_rows <- which(rowSums(object$X) != rowSums(object$W))
    if (!allow_mismatch && length(mismatch_rows) > 0) {
        stop(
            "run_em: Row-wise mismatch in vote totals detected.\n",
            "Rows with mismatches: ", paste(mismatch_rows, collapse = ", "), "\n",
            "To allow mismatches, set `allow_mismatch = TRUE`."
        )
    }

    if (method == "exact" && length(mismatch_rows) > 0) {
        .dhondt_correction(object$W, object$X)
        message("Applying a D'Hondt correction for correcting mismatches in W")
    }

    object <- .run_em_apply_group_agg(object, group_agg)
    object$method <- method
    object
}

#' Internal function!
#'
#' Populate method-specific defaults for EM runs.
#'
#' @param object An `eim` object.
#' @param method Character string with the selected EM method.
#' @param all_params List of evaluated `run_em()` arguments.
#' @return The updated `eim` object with method-specific defaults.
#' @noRd
.run_em_apply_method_defaults <- function(object, method, all_params) {
    if (method == "mcmc") {
        object$mcmc_stepsize <- as.integer(
            if ("mcmc_stepsize" %in% names(all_params)) all_params$mcmc_stepsize else 3000
        )
        object$mcmc_samples <- as.integer(
            if ("mcmc_samples" %in% names(all_params)) all_params$mcmc_samples else 1000
        )
        object$burn_in <- as.integer(
            if ("burn_in" %in% names(all_params)) all_params$burn_in else 10000
        )
    } else if (method == "mvn_cdf") {
        object$mvncdf_method <- if ("mvncdf_method" %in% names(all_params)) all_params$mvncdf_method else "genz"
        object$mvncdf_samples <- if ("mvncdf_samples" %in% names(all_params)) all_params$mvncdf_samples else 5000
        object$mvncdf_error <- if ("mvncdf_error" %in% names(all_params)) all_params$mvncdf_error else 1e-3
    }

    object
}

#' Internal function!
#'
#' Build the recursive call used for the reverse symmetric run.
#'
#' @param base_call Original `run_em()` call.
#' @param object The forward-run `eim` object.
#' @param all_params List of evaluated `run_em()` arguments.
#' @return A modified call object for the reverse run.
#' @noRd
.run_em_inverse_call <- function(base_call, object, all_params) {
    base_call_sym <- base_call
    base_call_sym$symmetric <- FALSE
    base_call_sym$X <- object$W
    base_call_sym$W <- object$X
    base_call_sym$json_path <- NULL
    base_call_sym$object <- NULL
    base_call_sym$scale_factor <- 1

    initial_prob <- all_params$initial_prob
    if (is.matrix(initial_prob)) {
        col_totals_x <- colSums(object$X)
        numerator <- sweep(initial_prob, 2, col_totals_x, "*")
        denominator <- rowSums(numerator)
        base_call_sym$initial_prob <- sweep(numerator, 1, denominator, "/")
        base_call_sym$initial_prob <- t(base_call_sym$initial_prob)
    }

    base_call_sym
}

#' Internal function!
#'
#' Copy EM results back into an `eim` object.
#'
#' @param object An `eim` object.
#' @param resulting_values Output list returned by `EMAlgorithmFull`.
#' @param W_matrix Group matrix used in the fit.
#' @param control List with the active `run_em()` controls.
#' @return The updated `eim` object.
#' @noRd
.run_em_assign_results <- function(object, resulting_values, W_matrix, control) {
    object$cond_prob <- resulting_values$q
    dimnames(object$cond_prob) <- .run_em_dimnames(object, W_matrix)
    object$expected_outcome <- resulting_values$expected_outcome
    dimnames(object$expected_outcome) <- .run_em_dimnames(object, W_matrix)
    object$prob <- as.matrix(resulting_values$result)
    dimnames(object$prob) <- list(colnames(W_matrix), colnames(object$X))
    object$iterations <- as.numeric(resulting_values$total_iterations)

    if (control$compute_ll) {
        object$logLik <- as.numeric(resulting_values$log_likelihood[length(resulting_values$log_likelihood)])
    }

    object$time <- resulting_values$total_time
    object$message <- resulting_values$stopping_reason
    object$status <- as.integer(resulting_values$finish_id)
    object$miniter <- control$miniter
    object$maxiter <- control$maxiter
    object$maxtime <- control$maxtime
    object$param_threshold <- control$param_threshold
    object$ll_threshold <- control$ll_threshold
    object$initial_prob <- control$initial_prob
    object$adjust_prob_cond_method <- control$adjust_prob_cond_method
    object$adjust_prob_cond_every <- control$adjust_prob_cond_every

    object
}

#' Internal function!
#'
#' Finalize the direct joint symmetric path without storing inverse outputs.
#'
#' @param object An `eim` object returned by the joint symmetric run.
#' @return The updated `eim` object with joint symmetric metadata.
#' @noRd
.run_em_finalize_joint <- function(object) {
    object$cond_prob_inv <- NULL
    object$prob_inv <- NULL
    object$expected_outcome_inv <- NULL
    object$symmetric_weight_method <- "joint"
    object$symmetric_weights <- c(original = 0.5, reverse = 0.5)

    object
}

#' Internal function!
#'
#' Compute the weights used to combine forward and reverse symmetric runs.
#'
#' @param object The forward-run `eim` object.
#' @param inverse The reverse-run `eim` object.
#' @param W_sym Group matrix used to rebuild symmetric probabilities.
#' @param control List with the active `run_em()` controls.
#' @return A list with the updated object and the two symmetric weights.
#' @noRd
.run_em_symmetric_weights <- function(object, inverse, W_sym, control) {
    weight_original <- 0.5
    weight_reverse <- 0.5

    if (identical(control$symmetric_weight_method, "delta_ll")) {
        forward_ll <- suppressWarnings(as.numeric(object$logLik))
        reverse_ll <- suppressWarnings(as.numeric(inverse$logLik))

        if (is.finite(forward_ll) && is.finite(reverse_ll)) {
            q_orig_bgc <- aperm(object$cond_prob, c(3, 1, 2))
            z_from_orig_bgc <- sweep(q_orig_bgc, c(1, 2), W_sym, "*")
            q_rev_ind_bcg <- sweep(aperm(z_from_orig_bgc, c(1, 3, 2)), c(1, 2), object$X, "/")
            q_rev_ind <- aperm(.normalize_cube_last_dim(q_rev_ind_bcg), c(2, 3, 1))
            p_rev_ind <- .mstep_from_q(q_rev_ind, object$X)

            q_rev_bcg <- aperm(inverse$cond_prob, c(3, 1, 2))
            z_from_rev_bgc <- aperm(sweep(q_rev_bcg, c(1, 2), object$X, "*"), c(1, 3, 2))
            q_ind_bgc <- sweep(z_from_rev_bgc, c(1, 2), W_sym, "/")
            q_ind <- aperm(.normalize_cube_last_dim(q_ind_bgc), c(2, 3, 1))
            p_ind <- .mstep_from_q(q_ind, W_sym)

            LL_ind <- .run_em_loglik_from_prob(
                object,
                object$X,
                W_sym,
                p_ind,
                control$method,
                control$miniter,
                control$adjust_prob_cond_method,
                control$adjust_prob_cond_every
            )
            LL_rev_ind <- .run_em_loglik_from_prob(
                object,
                object$W,
                object$X,
                p_rev_ind,
                control$method,
                control$miniter,
                control$adjust_prob_cond_method,
                control$adjust_prob_cond_every
            )
            dLL <- forward_ll - LL_ind
            dLL_rev <- reverse_ll - LL_rev_ind

            tau <- if (abs(forward_ll) > .Machine$double.eps) max(0, dLL / abs(forward_ll)) else 0
            tau_rev <- if (abs(reverse_ll) > .Machine$double.eps) max(0, dLL_rev / abs(reverse_ll)) else 0
            denominator <- tau + tau_rev

            if (is.finite(denominator) && denominator > 0) {
                weight_original <- tau_rev / denominator
                weight_reverse <- tau / denominator
            }

            object$LL_ind <- LL_ind
            object$LL_rev_ind <- LL_rev_ind
            object$dLL <- dLL
            object$dLL_rev <- dLL_rev
            object$nu <- tau
            object$nu_rev <- tau_rev
            object$symmetric_weight_method <- "delta_ll"
        } else {
            object$symmetric_weight_method <- "average"
        }
    } else if (identical(control$symmetric_weight_method, "mae_inverse")) {
        prob_forward <- .mstep_from_q(object$cond_prob, W_sym)
        prob_reverse <- as.matrix(inverse$prob)

        if (!all(dim(prob_reverse) == c(ncol(object$X), ncol(W_sym)))) {
            prob_reverse <- .mstep_from_q(inverse$cond_prob, object$X)
        }

        if (all(dim(prob_reverse) == c(ncol(object$X), ncol(W_sym)))) {
            x_hat_forward <- W_sym %*% prob_forward
            w_hat_reverse <- object$X %*% prob_reverse
            tau <- sum(abs(object$X - x_hat_forward))
            tau_rev <- sum(abs(W_sym - w_hat_reverse))
            denominator <- tau + tau_rev

            if (is.finite(denominator) && denominator > 0) {
                weight_original <- tau_rev / denominator
                weight_reverse <- 1 - weight_original
            }

            object$err_forward <- tau
            object$err_inverse <- tau_rev
            object$symmetric_weight_method <- "mae_inverse"
        } else {
            object$symmetric_weight_method <- "average"
        }
    } else {
        object$symmetric_weight_method <- "average"
    }

    list(
        object = object,
        weight_original = weight_original,
        weight_reverse = weight_reverse
    )
}

#' Internal function!
#'
#' Combine the forward and reverse runs into a symmetric result.
#'
#' @param object The forward-run `eim` object.
#' @param inverse The reverse-run `eim` object.
#' @param control List with the active `run_em()` controls.
#' @return The symmetric `eim` object.
#' @noRd
.run_em_apply_symmetry <- function(object, inverse, control) {
    object$cond_prob_inv <- inverse$cond_prob
    object$prob_inv <- inverse$prob
    object$expected_outcome_inv <- inverse$expected_outcome
    object$time <- object$time + inverse$time
    object$iterations <- object$iterations + inverse$iterations

    W_sym <- .run_em_working_group_matrix(object)
    reverse_expected_outcome <- aperm(inverse$expected_outcome, c(2, 1, 3))
    weights <- .run_em_symmetric_weights(object, inverse, W_sym, control)
    object <- weights$object
    object$symmetric_weights <- c(
        original = weights$weight_original,
        reverse = weights$weight_reverse
    )
    object$expected_outcome <- weights$weight_original * object$expected_outcome +
        weights$weight_reverse * reverse_expected_outcome

    expected_bgc <- aperm(object$expected_outcome, c(3, 1, 2))
    cond_prob_bgc <- sweep(expected_bgc, c(1, 2), W_sym, "/")
    object$cond_prob <- aperm(.normalize_cube_last_dim(cond_prob_bgc), c(2, 3, 1))

    numerator <- apply(object$expected_outcome, c(1, 2), sum)
    denominator <- colSums(W_sym)
    object$prob <- sweep(numerator, 1, denominator, "/")
    object$prob <- .normalize_prob_rows(object$prob)

    object
}

#' Internal function!
#'
#' Execute `run_em()`.
#'
#' @param object A prepared `eim` object.
#' @param control List with the active `run_em()` controls.
#' @return An updated `eim` object with EM results.
#' @noRd
.run_em_core <- function(object, control) {
    object <- .run_em_apply_method_defaults(object, control$method, control$all_params)
    W_matrix <- .run_em_working_group_matrix(object)

    resulting_values <- EMAlgorithmFull(
        t(object$X),
        W_matrix,
        control$method,
        if (is.character(control$initial_prob)) control$initial_prob else "custom",
        control$maxiter,
        control$maxtime,
        control$param_threshold,
        control$ll_threshold,
        control$compute_ll,
        control$verbose,
        as.integer(if (!is.null(object$mcmc_stepsize)) object$mcmc_stepsize else 3000),
        as.integer(if (!is.null(object$mcmc_samples)) object$mcmc_samples else 1000),
        if (!is.null(object$mvncdf_method)) object$mvncdf_method else "genz",
        as.numeric(if (!is.null(object$mvncdf_error)) object$mvncdf_error else 1e-3),
        as.numeric(if (!is.null(object$mvncdf_samples)) object$mvncdf_samples else 5000),
        control$miniter,
        control$adjust_prob_cond_method,
        control$adjust_prob_cond_every,
        if (is.matrix(control$initial_prob)) control$initial_prob else matrix(-1, nrow = 1, ncol = 1),
        control$symmetric,
        control$symmetric_weight_method
    )

    object <- .run_em_assign_results(object, resulting_values, W_matrix, control)
    if (control$symmetric && identical(control$symmetric_weight_method, "joint")) {
        return(.run_em_finalize_joint(object))
    }
    if (!control$symmetric) {
        return(object)
    }

    inverse_call <- .run_em_inverse_call(control$base_call, object, control$all_params)
    inverse <- eval(inverse_call, control$caller_env)

    .run_em_apply_symmetry(object, inverse, control)
}

#' Internal function!
#'
#' Randomly create a voting instance by defining an interval
#'
#' @description
#' Given a range of possible \strong{observed} outcomes (such as ballot boxes, number of candidates, etc.),
#' it creates a completely random voting instance, simulating the unobserved results as well.
#'
#' @param ballots_range (integer) A vector of size 2 with the lower and upper bound of ballot boxes.
#'
#' @param candidates_range (integer) A vector of size 2 with the lower and upper bound of candidates to draw.
#'
#' @param demographic_range (integer) A vector of size 2 with the lower and upper bound of demographic groups
#' to draw.
#'
#' @param voting_range (integer) A vector of size 2 with the lower and upper bound of votes per ballot box.
#'
#' @param seed \emph{(numeric(1)} Optional. If provided, it overrides the current global seed. (default: \code{NULL})
#'
#' @return A list with components:
#' \item{X}{A matrix (b x c) with candidate votes per ballot box.}
#' \item{W}{A matrix (b x g) with demographic votes per ballot box.}
#' \item{real_p}{A matrix (g x c) with the estimated \strong{(unobserved)} probabilities that a demographic group votes for a given candidate.}
#' \item{ballots}{The number of ballot boxes that were drawn.}
#' \item{candidates}{The number of candidates that were drawn.}
#' \item{groups}{The number of demographic groups that were drawn.}
#' \item{total_votes}{A vector with the number of total votes per ballot box.}
#'
#' @seealso [simulate_election()]
#' @examples
#'
#' bal_range <- c(30, 50)
#' can_range <- c(2, 4)
#' group_range <- c(2, 6)
#' voting_range <- c(50, 100)
#' results <- random_samples(bal_range, can_range, group_range, voting_range)
#'
#' # X matrix
#' results$X # A randomly generated matrix of dimension (b x c)
#' ncol(results$X <= can_range[2]) # Always TRUE
#' ncol(results$X >= can_range[1]) # Always TRUE
#' nrow(results$X <= bal_range[2]) # Always TRUE
#' nrow(results$X >= bal_range[1]) # Always TRUE
#'
#' # W matrix
#' results$W # A randomly generated matrix of dimension (b x g)
#' ncol(results$W <= group_range[2]) # Always TRUE
#' ncol(results$W >= group_range[1]) # Always TRUE
#' nrow(results$W <= bal_range[2]) # Always TRUE
#' nrow(results$W >= bal_range[1]) # Always TRUE
#'
#' # Probability matrix
#' results$real_p # A matrix (g x c) that summarizes the unobserved outcomes
#' ncol(results$real_p) == ncol(results$X) # Always TRUE
#' nrow(results$real_p) == ncol(results$W) # Always TRUE
#'
#' @noRd
.random_samples <- function(ballots_range, # Arguments must be vectors of size 2
                            candidates_range,
                            demographic_range,
                            voting_range,
                            seed = NULL) {
    param_list <- list(ballots_range, candidates_range, demographic_range, voting_range)
    if (!(all(sapply(param_list, length) == 2))) {
        stop("The vectors must be of size 2.")
    }
    if (!is.null(seed)) {
        set.seed(seed)
    }
    # Randomly choose a ballot box
    num_ballots <- sample(ballots_range[1]:ballots_range[2], 1)
    # Randomly choose demographic groups
    num_groups <- sample(demographic_range[1]:demographic_range[2], 1)
    # Randomly choose candidates
    num_candidates <- sample(candidates_range[1]:candidates_range[2], 1)
    # Randomly choose the total amount of votes per ballot box
    total_votes <- sample(
        seq.int(voting_range[1], voting_range[2]),
        size = num_ballots,
        replace = TRUE
    )
    # Randomly choose the group proportions
    group_prop <- rgamma(num_groups, shape = 1, rate = 1)
    group_prop <- group_prop / sum(group_prop)

    choosen_values <- list(
        ballots = num_ballots,
        candidates = num_candidates,
        groups = num_groups,
        total_votes = total_votes
    )

    result <- simulate_election(
        num_ballots = num_ballots,
        num_candidates = num_candidates,
        num_groups = num_groups,
        ballot_voters = total_votes,
        seed = seed,
        group_proportions = group_prop
    )

    appended_list <- c(result, choosen_values)
    appended_list
}

#' Internal function!
#' Applies dhond't correction to W and X matrices
#' @noRd
.dhondt_correction <- function(W, X) {
    if (any(W < 0) || any(X < 0)) stop("W and X must be non-negative.")
    adjust_row <- function(w, target_sum) {
        w <- as.numeric(w)
        cur <- sum(w)

        # trivial cases
        if (target_sum == cur) {
            return(w)
        }

        # if row is all zeros and target > 0, start from ones via uniform weights
        if (cur == 0 && target_sum > 0) {
            seats <- integer(length(w))
            for (k in seq_len(target_sum)) {
                # uniform weights -> all quotients equal; break ties by first
                j <- which.max(rep(1, length(w)) / (seats + 1))
                seats[j] <- seats[j] + 1L
            }
            return(seats)
        }

        if (target_sum > cur) {
            # add (target_sum - cur) units by D’Hondt on base weights w
            add <- target_sum - cur
            seats <- integer(length(w))
            for (k in seq_len(add)) {
                quot <- w / (seats + 1)
                j <- which.max(quot)
                seats[j] <- seats[j] + 1L
            }
            return(w + seats)
        } else {
            # remove (cur - target_sum) units greedily from the largest current entries
            rem <- cur - target_sum
            out <- w
            for (k in seq_len(rem)) {
                j <- which.max(out)
                if (out[j] > 0) out[j] <- out[j] - 1L else break
            }
            return(out)
        }
    }

    targets <- rowSums(X)
    W_adj <- W
    for (i in seq_len(nrow(W))) {
        W_adj[i, ] <- adjust_row(W[i, ], targets[i])
    }

    W_adj
}

#' Chilean 2021 First Round Presidential Election
#'
#' This dataset contains the results of the first round of the 2021 Chilean presidential elections. It provides
#' 9 possible voting options (7 candidates, blank, and null). Each ballot-box is identified by its id (`BALLOT.BOX`) and an electoral circumscription
#' (`ELECTORAL.DISTRICT`). Additionally, it provides demographic information on voters' age range for each ballot box.
#'
#' @docType data
#' @usage data("chile_election_2021")
#'
#' @format A data frame with 46,639 rows and 14 variables:
#' \describe{
#'   \item{\code{REGION}}{The region of the \code{ELECTORAL.DISTRICT}}
#'   \item{\code{ELECTORAL.DISTRICT}}{The electoral circumscription of the ballot box.}
#'   \item{\code{BALLOT.BOX}}{An identifier for the ballot box within the \code{ELECTORAL.DISTRICT}.}
#'   \item{\code{C1}}{The number of votes cast for the candidate *Gabriel Boric*.}
#'   \item{\code{C2}}{The number of votes cast for the candidate *José Antonio Kast*.}
#'   \item{\code{C3}}{The number of votes cast for the candidate *Yasna Provoste*.}
#'   \item{\code{C4}}{The number of votes cast for the candidate *Sebastián Sichel*.}
#'   \item{\code{C5}}{The number of votes cast for the candidate *Eduardo Artés*.}
#'   \item{\code{C6}}{The number of votes cast for the candidate *Marco Enríquez-Ominami*.}
#'   \item{\code{C7}}{The number of votes cast for the candidate *Franco Parisi*.}
#'   \item{\code{BLANK.VOTES}}{The number of blank votes.}
#'   \item{\code{NULL.VOTES}}{The number of null votes.}
#'   \item{\code{X18.19}}{Number of voters aged 18--19.}
#'   \item{\code{X20.29}}{Number of voters aged 20--29.}
#'   \item{\code{X30.39}}{Number of voters aged 30--39.}
#'   \item{\code{X40.49}}{Number of voters aged 40--49.}
#'   \item{\code{X50.59}}{Number of voters aged 50--59.}
#'   \item{\code{X60.69}}{Number of voters aged 60--69.}
#'   \item{\code{X70.79}}{Number of voters aged 70--79.}
#'   \item{\code{X80.}}{Number of voters aged 80 and older.}
#'   \item{\code{MISMATCH}}{Boolean that takes value `TRUE` if the ballot-box has a mismatch between the total number of votes and the total number of voters. If this is not the case, its value is `FALSE`.}
#' 	 \item{\code{F}}{Number of female voters in the ballot box.}
#' 	 \item{\code{M}}{Number of male voters in the ballot box.}
#' }
#'
#' @keywords datasets
#' @source [Chilean Electoral Service (SERVEL)](https://www.servel.cl/)
#' @examples
#' data("chile_election_2021")
#' head(chile_election_2021)
#'
"chile_election_2021"

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

EMAlgorithmFull <- function(candidate_matrix, group_matrix, em_method, probability_method, maximum_iterations, maximum_seconds, stopping_threshold, log_stopping_threshold, compute_ll, verbose, step_size, samples, monte_method, monte_error, monte_iter, miniterations, LP_method, project_every, initial_probabilities, symmetric, symmetric_weight_method) {
    .Call(`_fastei_EMAlgorithmFull`, candidate_matrix, group_matrix, em_method, probability_method, maximum_iterations, maximum_seconds, stopping_threshold, log_stopping_threshold, compute_ll, verbose, step_size, samples, monte_method, monte_error, monte_iter, miniterations, LP_method, project_every, initial_probabilities, symmetric, symmetric_weight_method)
}

EMLogLikFromProb <- function(candidate_matrix, group_matrix, probability_matrix, em_method, step_size, samples, monte_method, monte_error, monte_iter, miniterations, LP_method, project_every) {
    .Call(`_fastei_EMLogLikFromProb`, candidate_matrix, group_matrix, probability_matrix, em_method, step_size, samples, monte_method, monte_error, monte_iter, miniterations, LP_method, project_every)
}

bootstrapAlg <- function(candidate_matrix, group_matrix, nboot, em_method, probability_method, maximum_iterations, maximum_seconds, stopping_threshold, log_stopping_threshold, compute_ll, verbose, step_size, samples, monte_method, monte_error, monte_iter, miniterations, LP_method, project_every, initial_probabilities) {
    .Call(`_fastei_bootstrapAlg`, candidate_matrix, group_matrix, nboot, em_method, probability_method, maximum_iterations, maximum_seconds, stopping_threshold, log_stopping_threshold, compute_ll, verbose, step_size, samples, monte_method, monte_error, monte_iter, miniterations, LP_method, project_every, initial_probabilities)
}

groupAgg <- function(sd_statistic, sd_threshold, feasible, candidate_matrix, group_matrix, nboot, em_method, probability_method, maximum_iterations, maximum_seconds, stopping_threshold, log_stopping_threshold, compute_ll, verbose, step_size, samples, monte_method, monte_error, monte_iter, miniterations, LP_method, project_every, initial_probabilities) {
    .Call(`_fastei_groupAgg`, sd_statistic, sd_threshold, feasible, candidate_matrix, group_matrix, nboot, em_method, probability_method, maximum_iterations, maximum_seconds, stopping_threshold, log_stopping_threshold, compute_ll, verbose, step_size, samples, monte_method, monte_error, monte_iter, miniterations, LP_method, project_every, initial_probabilities)
}

groupAggGreedy <- function(sd_statistic, sd_threshold, candidate_matrix, group_matrix, nboot, em_method, probability_method, maximum_iterations, maximum_seconds, stopping_threshold, log_stopping_threshold, compute_ll, verbose, step_size, samples, monte_method, monte_error, monte_iter, miniterations, LP_method, project_every) {
    .Call(`_fastei_groupAggGreedy`, sd_statistic, sd_threshold, candidate_matrix, group_matrix, nboot, em_method, probability_method, maximum_iterations, maximum_seconds, stopping_threshold, log_stopping_threshold, compute_ll, verbose, step_size, samples, monte_method, monte_error, monte_iter, miniterations, LP_method, project_every)
}

#' fastei: Methods for "A Fast Ecological Inference Algorithm for the R\eqn{\times}C case"
#'
#' Package that implements the methods of [Thraves, C.,Ubilla, P. and Hermosilla, D. (2024): *"A Fast Ecological Inference Algorithm for the R×C Case"*.](https://papers.ssrn.com/sol3/papers.cfm?abstract_id=4832834)
#'
#' Includes a method ([run_em]) to solve the R\eqn{\times}C ecological inference problem using the EM algorithm with different approximation methods for the E-step. Covariates are supported but not required. Standard errors of the estimated probabilities can be computed via bootstrapping ([bootstrap]).
#'
#' It also provides a function that generates synthetic election data ([simulate_election]) and a function that imports real election data ([chile_election_2021]) from the Chilean first-round presidential election of 2021.
#'
#' The setting in which the documentation presents the Ecological Inference problem is an election context where for a set of ballot-boxes we observe (i) the votes obtained by each candidate and (ii) the number of voters of each demographic group (for example, these can be defined by age ranges or sex). See [Thraves, C.,Ubilla, P. and Hermosilla, D. (2024): *"A Fast Ecological Inference Algorithm for the R×C Case"*.](https://papers.ssrn.com/sol3/papers.cfm?abstract_id=4832834)
#'
#' The methods to compute the conditional probabilities of the E-Step included in this package are the following:
#' - **Markov Chain Monte Carlo** `(mcmc)`: Performs MCMC to sample vote outcomes for each ballot-box consistent with the observed data. This sample is used to estimate the conditional probability of the *E*-Step.
#' - **Multivariate Normal PDF** `(mvn_pdf)`: Uses the PDF of a Multivariate Normal to approximate the conditional probability.
#' - **Multivariate Normal CDF** `(mvn_cdf)`: Uses the CDF of a Multivariate Normal to approximate the conditional probability.
#' - **Multinomial** `(mult)`: A single Multinomial is used to approximate the sum of Multinomial distributions.
#' - **Exact** `(exact)`: Solves the E-Step exactly using the Total Probability Law, which requires enumerating an exponential number of terms.
#'
#' On average, the **Multinomial** method is the most efficient and precise. Its precision matches the **Exact** method.
#'
#' The documentation uses the following notation:
#' - `b`: number of ballot-boxes.
#' - `g`: number of demographic groups.
#' - `c`: number of candidates.
#' - `a`: number of aggregated macro-groups.
#'
#' To learn more about `fastei`, please consult the available vignettes:
#'
#' \preformatted{
#' browseVignettes("fastei")
#' }
#'
#' @references
#' [Thraves, C., Ubilla, P and Hermosilla D. (2024): *"A Fast Ecological Inference Algorithm for the R×C Case"*.](https://papers.ssrn.com/sol3/papers.cfm?abstract_id=4832834)
#' @aliases NULL
#' @concept package
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
#' @useDynLib fastei, .registration = TRUE
#' @import Rcpp
#' @importFrom Rcpp evalCpp
#' @importFrom stats logLik pt rgamma rmultinom pnorm rnorm runif
#' @importFrom utils modifyList tail write.csv
NULL

utils::globalVariables(c("label", "value"))

expect_prob_matrix <- function(mat, tol = 1e-6) {
    testthat::expect_true(is.matrix(mat))
    testthat::expect_true(all(is.finite(mat)))
    testthat::expect_true(all(mat >= -tol))
    testthat::expect_true(all(mat <= 1 + tol))
    rs <- rowSums(mat)
    testthat::expect_true(all(abs(rs - 1) < tol))
}

expect_prob_array <- function(arr, tol = 1e-6) {
    testthat::expect_true(is.array(arr))
    testthat::expect_true(length(dim(arr)) == 3)
    testthat::expect_true(all(is.finite(arr)))
    testthat::expect_true(all(arr >= -tol))
    testthat::expect_true(all(arr <= 1 + tol))
    sums <- apply(arr, c(1, 3), sum)
    testthat::expect_true(all(abs(sums - 1) < tol))
}

expect_dimnames_match <- function(arr, group_names, cand_names, ballot_names) {
    testthat::expect_equal(dimnames(arr)[[1]], group_names)
    testthat::expect_equal(dimnames(arr)[[2]], cand_names)
    testthat::expect_equal(dimnames(arr)[[3]], ballot_names)
}

mae <- function(x, y) {
    mean(abs(x - y))
}

expected_votes_from_q <- function(W, q) {
    B <- nrow(W)
    C <- dim(q)[2]
    out <- matrix(0, nrow = B, ncol = C)
    for (b in seq_len(B)) {
        out[b, ] <- as.numeric(W[b, ] %*% q[, , b])
    }
    out
}

---
title: "Demonstration of the package usage"
author: "Charles Thraves, Pablo Ubilla, Daniel Hermosilla"
date: "`r Sys.Date()`"

vignette: >
  %\VignetteIndexEntry{Demonstration of the package usage}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}

output:
  rmarkdown::html_vignette:
    toc: false
    number_sections: false
---

The `fastei` library implements the Expectation-Maximization algorithm to estimate probabilities in Ecological Inference for the RxC case. It offers both an exact method and four approximation methods suitable for large datasets. The application demonstrated here is based on an electoral context where, for each ballot box, we know the number of voters in each demographic group and the number of votes received by each candidate.

# Estimate Voting Probabilities
We use data from the First Round of the Chilean Presidential Election of 2021, where at each ballot box we have the voters of eight age ranges (groups), and candidate votes obtained. The function `get_eim_chile()` loads this data at either the country, region, or electoral district level.

```{r}
library(fastei)

eim_apo <- get_eim_chile(elect_district = "APOQUINDO")

eim_apo
```

As shown in the output, it returns an eim object that contains two matrices: the number of votes per candidate and the number of votes per group. The rows of both matrices correspond to the specific ballot-box, and the columns are candidates and groups, respectively. This eim object is used as input to run the EM algorithm that estimates the voting probabilities. In this example, it uses the default method `mult`, which is the most efficient in terms of runtime.
Running the algorithm is done by calling `run_em()`.

```{r, digits=4}
eim_apo <- run_em(eim_apo)
round(eim_apo$prob, 4)
```

Note that each row corresponds to the probability that a demographic group (`g`) voted for a candidate (`c`). It is worth noting how the estimated probabilities differ substantially across groups.

# Standard deviation estimates

We can compute the standard deviation of the estimated probabilities using bootstrapping. This can be done with the function `bootstrap()`.

```{r}
eim_apo <- bootstrap(eim_apo, seed = 42, nboot = 30)
round(eim_apo$sd, 4)
```

The standard deviations obtained in the district "Apoquindo" are low in general. One reason for this is the high number of ballot boxes in this district. In contrast, standard deviations of estimated probabilities in districts with fewer ballot boxes, such as "Navidad", are larger.

```{r}
eim_nav <- get_eim_chile(elect_district = "NAVIDAD")
eim_nav <- bootstrap(eim_nav, seed = 42, nboot = 30)
round(eim_nav$sd, 4)
```
It is possible to see the difference in a visual way by plotting the standard deviations of the estimated probabilities across groups. 

```{r, message=FALSE, warning=FALSE, echo=TRUE}
library(ggplot2)
library(reshape2)
library(viridis)

plot_district <- function(matrix1, district1, matrix2, district2, sd = FALSE) {
    value <- ifelse(sd == FALSE, "prob", "sd")
    df1 <- melt(matrix1)
    df2 <- melt(matrix2)
    df1$Matrix <- district1
    df2$Matrix <- district2
    combined_df <- rbind(df1, df2)
    color <- ifelse(value == "prob", "plasma", "viridis")

    # Add text to each cell of the matrix
    combined_df$label <- sprintf("%.3f", combined_df$value)
    combined_df$text_color <- ifelse(combined_df$value > round(max(combined_df$value) * 0.75 + min(combined_df$value) * 0.25, 2), "black", "white")
    districts <- sort(c(district1, district2))

    # Call the plot
    ggplot(combined_df, aes(x = Var2, y = Var1, fill = value)) +
        geom_tile() +
        geom_text(aes(label = label, color = text_color), size = 2.5) +
        scale_fill_viridis(
            name = value,
            option = color,
        ) +
        scale_color_identity() +
        facet_wrap(~Matrix) +
        coord_fixed() +
        theme_bw() +
        labs(
            title = ifelse(value == "prob",
                paste("Estimated probabilities in districts:", districts[1], "and", districts[2]),
                paste("Standard deviation of estimated probabilities in districts:", districts[1], "and", districts[2])
            ),
            x = "Candidates' votes", y = "Voters' age range", fill = value
        )
}
```


```{r, navidad_apoq_sd_comparison, fig.width = 8, fig.height = 6.5, fig.cap = "Navidad and Apoquindo standard deviation comparison", fig.align = "center", message=FALSE, warning=FALSE, results="hide"}
plot_district(
    matrix1 = eim_nav$sd, district1 = "Navidad",
    matrix2 = eim_apo$sd, district2 = "Apoquindo", sd = TRUE
)
```


# Reduce Estimation Error using Group Aggregation

Demographic groups can be merged to have probability estimates with lower error. A greedy strategy that maximizes the variability of the distribution of groups across ballot boxes is used, which ensures standard deviations are below a specific threshold. The package provides the following function for the latter:

```{r}
eim_nav_proxy <- get_agg_proxy(eim_nav, seed = 6, sd_threshold = 0.03, sd_statistic = "maximum")
eim_nav_proxy$group_agg
```

 As shown in the output, the heuristic finds a group aggregation that merges the first and last four age ranges, namely, macro-groups with voters aged 18-49, and older than 50. We can evaluate the effectiveness of this grouping by comparing the mean standard deviation to the original formulation:

```{r}
mean(eim_nav$sd) - mean(eim_nav_proxy$sd)
```
It is possible to see the aggregated standard deviations visually:

```{r}
plot_matrix <- function(mat, sd = FALSE, y_labels = NULL) {
    # Initial configurations
    if (!sd) mat <- t(mat)
    df <- reshape2::melt(mat)
    colnames(df) <- c("Row", "Column", "Value")
    df$Row <- factor(df$Row, levels = rev(sort(unique(df$Row))))
    df$Column <- factor(df$Column, levels = sort(unique(df$Column)))
    if (!sd) {
        df$Label <- sprintf("%d", df$Value)
        title_text <- "Voters distribution"
        x_lab <- "Ballot Box"
        y_lab <- "Dem. Group"
        fill_lab <- "Voters"
        df$text_color <- ifelse(df$Value > 30, "black", "white")
        option <- "inferno"
        start <- 0.5
        limits <- NULL
    } else {
        df$Label <- sprintf("%.3f", df$Value)
        title_text <- "Standard deviation of estimated probabilities on district: Navidad"
        x_lab <- "Candidates' votes"
        y_lab <- "Voters' age range"
        fill_lab <- "sd"
        df$text_color <- ifelse(df$Value > 0.13, "black", "white")
        option <- "viridis"
        start <- 0
        limits <- c(0, 0.1)
    }

    # Plot
    p <- ggplot(df, aes(x = Column, y = Row, fill = Value)) +
        geom_tile() +
        geom_text(aes(label = Label, color = text_color), size = 3) +
        scale_color_identity() +
        scale_fill_viridis_c(option = option, begin = start, limits = limits) +
        coord_fixed() +
        theme_bw() +
        theme(axis.text.y = element_text(size = 7), axis.text.x = element_text(size = 7)) +
        labs(
            title = title_text,
            x = x_lab,
            y = y_lab,
            fill = fill_lab
        )
    # Add custom y-axis labels if provided
    if (!is.null(y_labels)) {
        p <- p + scale_y_discrete(labels = y_labels)
    }
    p
}
```

We can visualize the standard deviations of the estimated probabilities.

```{r, standard_deviation_proxy, fig.width = 8, fig.height = 3.5, fig.cap = "Navidad aggregated standard deviation with proxy method", fig.align = "center", message=FALSE, warning=FALSE, echo=TRUE}
plot_matrix(eim_nav_proxy$sd, sd = TRUE, y_labels = c("X18.49", "X50."))
```

An exhaustive algorithm is also included in the package. It explores all combinations of adjacent groups in order to maximize the log-likelihood subject to having standard deviations below a given threshold. It might require substantial computation time; therefore it is recommended to use the default method, "mult".

```{r}
eim_nav_opt <- get_agg_opt(eim_nav, seed = 0, sd_threshold = 0.03, sd_statistic = "maximum")
eim_nav_opt$group_agg
```

The optimal group aggregation differs slightly from one obtained before. In this case, the group aggregation consists in the three age range: 18-29, 30-49, and older than 70.

We can visualize the standard deviations of the estimated probabilities.

```{r, standard_deviation_opt, fig.width = 8, fig.height = 3.5, fig.cap = "Navidad aggregated standard deviation with opt method", fig.align = "center", message=FALSE, warning=FALSE}
plot_matrix(eim_nav_opt$sd, sd = TRUE, y_labels = c("X18.29", "X30.49", "X50."))
```

# Test difference between estimates

A relevant question is how significantly different are the probability estimates of two sets of data, such as two different districts. For instance, we can compare the estimated probabilities of the district "LO BARNECHEA" and "LA GRANJA", whose voting tendencies are expected to differ. 

```{r, granja_lobarnechea_comparison, fig.width = 8, fig.height=6.5, fig.cap = "Lo Barnechea and La Granja comparison", fig.align = "center", message=FALSE}
eim_gra <- get_eim_chile("LA GRANJA")
eim_gra <- run_em(eim_gra)
eim_bar <- get_eim_chile("LO BARNECHEA")
eim_bar <- run_em(eim_bar)

plot_district(eim_gra$prob, "La Granja", eim_bar$prob, "Lo Barnechea")
```

A Wald test can be applied to each component of the two probability matrix estimates:

```{r}
comparison2 <- waldtest(
    object1 = eim_gra,
    object2 = eim_bar,
    method = "mult",
    nboot = 30,
    seed = 42,
)

round(comparison2$pvals, 3)
```

Conversely, one may examine the alternative scenario in which ballot boxes within a given region are partitioned into equally sized groups. In this configuration, both groups are expected to exhibit identical probability distributions.

```{r, half_comparison, fig.width = 8, fig.height=6.5, fig.cap = "Equally sorted districts comparison", fig.align = "center", message=FALSE, warning=FALSE}
eimRM <- get_eim_chile(region = "METROPOLITANA DE SANTIAGO")

n <- nrow(eimRM$X)
n_train <- floor(n * 0.5)
train_indices <- sample(seq_len(n), n_train)

eimRMhalf1 <- eim(X = eimRM$X[train_indices, ], W = eimRM$W[train_indices, ])
eimRMhalf2 <- eim(X = eimRM$X[-train_indices, ], W = eimRM$W[-train_indices, ])

eimRMhalf1 <- run_em(eimRMhalf1)
eimRMhalf2 <- run_em(eimRMhalf2)

plot_district(eimRMhalf1$prob, "Half 1", eimRMhalf2$prob, "Half 2")
```

```{r}
comparison2 <- waldtest(
    object1 = eimRMhalf1,
    object2 = eimRMhalf2,
    nboot = 10,
    seed = 42
)

round(comparison2$pvals, 4)
```

# Simulating Election Results

It is possible to simulate an artificial election and get its values as an eim object with the simulated probability. This is useful for testing the package's performance and comparing the different methods available.

```{r}
eim_sim <- simulate_election(num_ballots = 15, num_groups = 2, num_candidates = 3, seed = 42)
eim_sim
```

The real voting probabilities of each group for each candidate can be obtained as follows:

```{r}
eim_sim$real_prob
```

In this case, we can access these values since this is a simulation; however, this is not possible when working with real data.

The estimated voting probabilities are:

```{r}
eim_sim <- run_em(eim_sim)
eim_sim$prob
```

It is possible to provide a specific voting probability for each candidate for each group.

```{r}
input_probability <- matrix(c(0.9, 0.05, 0.05, 0.2, 0.3, 0.5), nrow = 2, byrow = TRUE)
input_probability
```

```{r}
eim_sim2 <- simulate_election(
    num_ballots = 30, num_groups = 2, num_candidates = 3, seed = 42,
    prob = input_probability
)
eim_sim2
eim_sim2$real_prob
```

There is a parameter, lambda, that controls for the heterogeneity of voters' groups across ballot boxes. A value of lambda = 0 indicates that voters are all randomly assigned in ballot boxes; in contrast to lambda = 1, in which case voters are assigned in ballot boxes according to their demographic group. This is explained in detail in the documentation in `simulate_election()`.

```{r}
eim_sim3 <- simulate_election(
    num_ballots = 20, num_groups = 4, num_candidates = 2, seed = 42,
    lambda = 0.1
)
```

```{r, eim_sim3_heatmap, fig.width = 8, fig.height = 3.5, fig.cap = "Voters' heatmap for a low lambda value", fig.align = "center", message=FALSE, warning=FALSE, results="hide"}
plot_matrix(eim_sim3$W)
```

Having the following estimated probabilities:

```{r}
run_em(eim_sim3)$prob
```

On the other side, it is possible to simulate an heterogeneous sample
```{r}
eim_sim4 <- simulate_election(
    num_ballots = 20, num_groups = 4, num_candidates = 2, seed = 42,
    lambda = 0.9
)
```

```{r, eim_sim4_heatmap, fig.width = 8, fig.height = 3.5, fig.cap = "Voters' heatmap for a high lambda value", fig.align = "center", message=FALSE, warning=FALSE, results="hide"}
plot_matrix(eim_sim4$W)
```

With the underlying probabilities

```{r}
run_em(eim_sim4)$prob
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eim-class.R
\name{get_agg_proxy}
\alias{get_agg_proxy}
\title{Runs the EM algorithm aggregating adjacent groups, maximizing the variability of macro-group allocation in ballot boxes.}
\usage{
get_agg_proxy(
  object = NULL,
  X = NULL,
  W = NULL,
  json_path = NULL,
  sd_statistic = "maximum",
  sd_threshold = 0.05,
  method = "mult",
  feasible = TRUE,
  nboot = 100,
  allow_mismatch = TRUE,
  seed = NULL,
  ...
)
}
\arguments{
\item{object}{An object of class \code{eim}, which can be created using the \link{eim} function. This parameter should not be used if either (i) \code{X} and \code{W} matrices or (ii) \code{json_path} is supplied. See \strong{Note} in \link{run_em}.}

\item{X}{A \verb{(b x c)} matrix representing candidate votes per ballot box.}

\item{W}{A \verb{(b x g)} matrix representing group votes per ballot box.}

\item{json_path}{A path to a JSON file containing \code{X} and \code{W} fields, stored as nested arrays. It may contain additional fields with other attributes, which will be added to the returned object.}

\item{sd_statistic}{String indicates the statistic for the standard deviation \verb{(g x c)} matrix for the stopping condition, i.e., the algorithm stops when the statistic is below the threshold. It can take the value \code{maximum}, in which case computes the maximum over the standard deviation matrix, or \code{average}, in which case computes the average.}

\item{sd_threshold}{Numeric with the value to use as a threshold for the statistic (\code{sc_statistic}) of the standard deviation of the estimated probabilities. Defaults to 0.05.}

\item{method}{An optional string specifying the method used for estimating the E-step. Valid
options are:
\itemize{
\item \code{mult}: The default method, using a single sum of Multinomial distributions.
\item \code{mvn_cdf}: Uses a Multivariate Normal CDF distribution to approximate the conditional probability.
\item \code{mvn_pdf}: Uses a Multivariate Normal PDF distribution to approximate the conditional probability.
\item \code{mcmc}: Uses MCMC to sample vote outcomes. This is used to estimate the conditional probability of the E-step.
\item \code{exact}: Solves the E-step using the Total Probability Law.
}}

\item{feasible}{Logical indicating whether the returned matrix must strictly satisfy the \code{sd_threshold}.
If \code{TRUE}, no output is returned if the method does not find a group aggregation whose standard deviation statistic is below the threshold. If \code{FALSE} and the latter holds, it returns the group aggregation obtained from the DP with the the lowest standard deviation statistic. See \strong{Details} for more information. Default is \code{TRUE}.}

\item{nboot}{Integer specifying how many times to run the
EM algorithm.}

\item{allow_mismatch}{Boolean, if \code{TRUE}, allows a mismatch between the voters and votes for each ballot-box. If \code{FALSE}, throws an error if there is a mismatch. By default it is \code{TRUE}. See \strong{Notes} in \link{run_em} for more details.}

\item{seed}{An optional integer indicating the random seed for the randomized algorithms. This argument is only applicable if \code{initial_prob = "random"} or \code{method} is either \code{"mcmc"} or \code{"mvn_cdf"}. Aditionally, it sets the random draws of the ballot boxes.}

\item{...}{Additional arguments passed to the \link{run_em} function that will execute the EM algorithm.}
}
\value{
It returns an eim object with the same attributes as the output of \link{run_em}, plus the attributes:
\itemize{
\item \strong{sd}: A \verb{(a x c)} matrix with the standard deviation of the estimated probabilities computed with bootstrapping. Note that \code{a} denotes the number of macro-groups of the resulting group aggregation, it should be between \code{1} and \code{g}.
\item \strong{nboot}: Number of samples used for the \link{bootstrap} method.
\item \strong{seed}: Random seed used (if specified).
\item \strong{sd_statistic}: The statistic used as input.
\item \strong{sd_threshold}: The threshold used as input.
\item \strong{is_feasible}:  Boolean indicating whether the statistic of the standard deviation matrix is below the threshold.
\item \strong{group_agg}: Vector with the resulting group aggregation. See \strong{Examples} for more details.
}

Additionally, it will create the \code{W_agg} attribute with the aggregated groups, along with the attributes corresponding to running \link{run_em} with the aggregated groups.
}
\description{
This function estimates the voting probabilities (computed using \link{run_em}) aggregating adjacent groups so that the estimated probabilities' standard deviation (computed using \link{bootstrap}) is below a given threshold. See \strong{Details} for more information.
}
\note{
Groups need to have an order relation so that adjacent groups can be merged. Groups of consecutive column indices in the matrix W are considered adjacent. For example, consider the following seven groups defined by voters' age ranges: 20-29, 30-39, 40-49, 50-59, 60-69, 70-79, and 80+. A possible group aggregation can be a macro-group composed of the three following age ranges: 20-39, 40-59, and 60+. Since there are multiple group aggregations, even for a fixed number of macro-groups, a Dynamic Program (DP) mechanism is used to find the group aggregation that maximizes the sum of the standard deviation of the macro-groups proportions among ballot boxes for a specific number of macro-groups. If no group aggregation standard deviation statistic meets the threshold condition, \code{NULL} is returned.

To find the best group aggregation, the function runs the DP iteratively, starting with all groups (this case is trivial since the group aggregation is such that all macro-groups match exactly the original groups). If the standard deviation statistic (\code{sd_statistic}) is below the threshold (\code{sd_threshold}), it stops. Otherwise, it runs the DP such that the number of macro-groups is one unit less than the original number of macro-groups. If the standard deviation statistic is below the threshold, it stops. This continues until either the algorithm stops, or until no group aggregation obtained by the DP satisfies the threshold condition. If the former holds, then the last group aggregation obtained (before stopping) is returned; while if the latter holds, then no output is returned unless the user sets the input parameter \code{feasible=FALSE}, in which case it returns the group aggregation that has the least standard deviation statistic, among the group-aggregations obtained from the DP.
}
\examples{
# Example 1: Using a simulated instance
simulations <- simulate_election(
    num_ballots = 400,
    num_candidates = 3,
    num_groups = 6,
    group_proportions = c(0.4, 0.1, 0.1, 0.1, 0.2, 0.1),
    lambda = 0.7,
    seed = 42
)

result <- get_agg_proxy(
    X = simulations$X,
    W = simulations$W,
    sd_threshold = 0.015,
    seed = 42
)

result$group_agg # c(2 6)
# This means that the resulting group aggregation is conformed by
# two macro-groups: one that has the original groups 1 and 2; and
# a second that has the original groups 3, 4, 5, and 6.

# Example 2: Using the chilean election results
data(chile_election_2021)

niebla_df <- chile_election_2021[chile_election_2021$ELECTORAL.DISTRICT == "NIEBLA", ]

# Create the X matrix with selected columns
X <- as.matrix(niebla_df[, c("C1", "C2", "C3", "C4", "C5", "C6", "C7")])

# Create the W matrix with selected columns
W <- as.matrix(niebla_df[, c(
    "X18.19", "X20.29",
    "X30.39", "X40.49",
    "X50.59", "X60.69",
    "X70.79", "X80."
)])

solution <- get_agg_proxy(
    X = X, W = W,
    allow_mismatch = TRUE, sd_threshold = 0.03,
    sd_statistic = "average", seed = 42
)

solution$group_agg # c(3, 4, 5, 6, 8)
# This means that the resulting group aggregation consists of
# five macro-groups: one that includes the original groups 1, 2, and 3;
# three singleton groups (4, 5, and 6); and one macro-group that includes groups 7 and 8.

}
\seealso{
The \link{eim} object and \link{run_em} implementation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eim-class.R
\name{get_agg_opt}
\alias{get_agg_opt}
\title{Runs the EM algorithm \strong{over all possible group aggregating}, returning the one with higher likelihood while constraining the standard deviation of the probabilities.}
\usage{
get_agg_opt(
  object = NULL,
  X = NULL,
  W = NULL,
  json_path = NULL,
  sd_statistic = "maximum",
  sd_threshold = 0.05,
  method = "mult",
  nboot = 100,
  allow_mismatch = TRUE,
  seed = NULL,
  ...
)
}
\arguments{
\item{object}{An object of class \code{eim}, which can be created using the \link{eim} function. This parameter should not be used if either (i) \code{X} and \code{W} matrices or (ii) \code{json_path} is supplied. See \strong{Note} in \link{run_em}.}

\item{X}{A \verb{(b x c)} matrix representing candidate votes per ballot box.}

\item{W}{A \verb{(b x g)} matrix representing group votes per ballot box.}

\item{json_path}{A path to a JSON file containing \code{X} and \code{W} fields, stored as nested arrays. It may contain additional fields with other attributes, which will be added to the returned object.}

\item{sd_statistic}{String indicates the statistic for the standard deviation \verb{(g x c)} matrix for the stopping condition, i.e., the algorithm stops when the statistic is below the threshold. It can take the value \code{maximum}, in which case computes the maximum over the standard deviation matrix, or \code{average}, in which case computes the average.}

\item{sd_threshold}{Numeric with the value to use as a threshold for the statistic (\code{sc_statistic}) of the standard deviation of the estimated probabilities. Defaults to 0.05.}

\item{method}{An optional string specifying the method used for estimating the E-step. Valid
options are:
\itemize{
\item \code{mult}: The default method, using a single sum of Multinomial distributions.
\item \code{mvn_cdf}: Uses a Multivariate Normal CDF distribution to approximate the conditional probability.
\item \code{mvn_pdf}: Uses a Multivariate Normal PDF distribution to approximate the conditional probability.
\item \code{mcmc}: Uses MCMC to sample vote outcomes. This is used to estimate the conditional probability of the E-step.
\item \code{exact}: Solves the E-step using the Total Probability Law.
}}

\item{nboot}{Integer specifying how many times to run the
EM algorithm.}

\item{allow_mismatch}{Boolean, if \code{TRUE}, allows a mismatch between the voters and votes for each ballot-box. If \code{FALSE}, throws an error if there is a mismatch. By default it is \code{TRUE}. See \strong{Notes} in \link{run_em} for more details.}

\item{seed}{An optional integer indicating the random seed for the randomized algorithms. This argument is only applicable if \code{initial_prob = "random"} or \code{method} is either \code{"mcmc"} or \code{"mvn_cdf"}. Aditionally, it sets the random draws of the ballot boxes.}

\item{...}{Additional arguments passed to the \link{run_em} function that will execute the EM algorithm.}
}
\value{
It returns an eim object with the same attributes as the output of \link{run_em}, plus the attributes:
\itemize{
\item \strong{sd}: A \verb{(a x c)} matrix with the standard deviation of the estimated probabilities computed with bootstrapping. Note that \code{a} denotes the number of macro-groups of the resulting group aggregation, it should be between \code{1} and \code{g}.
\item \strong{nboot}: Number of samples used for the \link{bootstrap} method.
\item \strong{seed}: Random seed used (if specified).
\item \strong{sd_statistic}: The statistic used as input.
\item \strong{sd_threshold}: The threshold used as input.
\item \strong{group_agg}: Vector with the resulting group aggregation. See \strong{Examples} for more details.
}

Additionally, it will create the \code{W_agg} attribute with the aggregated groups, along with the attributes corresponding to running \link{run_em} with the aggregated groups.
}
\description{
Runs the EM algorithm \strong{over all possible group aggregating}, returning the one with higher likelihood while constraining the standard deviation of the probabilities.
}
\note{
This function estimates the voting probabilities (computed using \link{run_em}) by trying all group aggregations (of adjacent groups), choosing
the one that achieves the higher likelihood as long as the standard deviation (computed using \link{bootstrap}) of the estimated probabilities
is below a given threshold. See \strong{Details} for more informacion on adjacent groups.

Groups of consecutive column indices in the matrix \code{W} are considered adjacent. For example, consider the following seven groups defined by voters' age
ranges: 20-29, 30-39, 40-49, 50-59, 60-69, 70-79, and 80+. A possible group aggregation can be a macro-group composed of the three following age
ranges: 20-39, 40-59, and 60+. Since there are multiple group aggregations, the method evaluates all possible group aggregations (merging only adjacent groups).
}
\examples{
# Example 1: Using a simulated instance
simulations <- simulate_election(
    num_ballots = 20,
    num_candidates = 3,
    num_groups = 8,
    seed = 42
)

result <- get_agg_opt(
    X = simulations$X,
    W = simulations$W,
    sd_threshold = 0.05,
    seed = 42
)

result$group_agg # c(3,8)
# This means that the resulting group aggregation consists of
# two macro-groups: one that includes the original groups 1, 2, and 3;
# the remaining one with groups 4, 5, 6, 7 and 8.

\donttest{
# Example 2: Getting an unfeasible result
result2 <- get_agg_opt(
    X = simulations$X,
    W = simulations$W,
    sd_threshold = 0.001
)

result2$group_agg # Error
result2$X # Input candidates' vote matrix
result2$W # Input group-level voter matrix
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eim-class.R
\name{bootstrap}
\alias{bootstrap}
\alias{bootstrap()}
\title{Runs a Bootstrap to Estimate the \strong{Standard Deviation} of Predicted Probabilities}
\usage{
bootstrap(
  object = NULL,
  X = NULL,
  W = NULL,
  json_path = NULL,
  nboot = 100,
  allow_mismatch = TRUE,
  seed = NULL,
  ...
)
}
\arguments{
\item{object}{An object of class \code{eim}, which can be created using the \link{eim} function. This parameter should not be used if either (i) \code{X} and \code{W} matrices or (ii) \code{json_path} is supplied. See \strong{Note}.}

\item{X}{A \verb{(b x c)} matrix representing candidate votes per ballot box.}

\item{W}{A \verb{(b x g)} matrix representing group votes per ballot box.}

\item{json_path}{A path to a JSON file containing \code{X} and \code{W} fields, stored as nested arrays. It may contain additional fields with other attributes, which will be added to the returned object.}

\item{nboot}{Integer specifying how many times to run the
EM algorithm.}

\item{allow_mismatch}{Boolean, if \code{TRUE}, allows a mismatch between the voters and votes for each ballot-box. If \code{FALSE}, throws an error if there is a mismatch. By default it is \code{TRUE}. See \strong{Notes} for more details.}

\item{seed}{An optional integer indicating the random seed for the randomized algorithms. This argument is only applicable if \code{initial_prob = "random"} or \code{method} is either \code{"mcmc"} or \code{"mvn_cdf"}. Aditionally, it sets the random draws of the ballot boxes.}

\item{...}{Additional arguments passed to the \link{run_em} function that will execute the EM algorithm.}
}
\value{
Returns an \code{eim} object with the \code{sd} field containing the estimated standard deviations of the probabilities and the \code{avg_prob} field with the average bootstrapped probability matrix. If an \code{eim} object is provided, its attributes (see \link{run_em}) are retained in the returned object.
}
\description{
This function computes the Expected-Maximization (EM) algorithm "\code{nboot}" times. It then computes the standard deviation from the \code{nboot} estimated probability matrices on each component.
}
\note{
This function can be executed using one of three mutually exclusive approaches:
\enumerate{
\item By providing an existing \code{eim} object.
\item By supplying both input matrices (\code{X} and \code{W}) directly.
\item By specifying a JSON file (\code{json_path}) containing the matrices.
}

These input methods are \strong{mutually exclusive}, meaning that you must provide \strong{exactly one} of
these options. Attempting to provide more than one or none of these inputs will result in an error.

When called with an \code{eim} object, the function updates the object with the computed results.
If an \code{eim} object is not provided, the function will create one internally using either the
supplied matrices or the data from the JSON file before executing the algorithm.

If there are ballot-boxes with mismatch between \code{W} and \code{X}, and \code{allow_mismatch = TRUE}, then: if \code{method = "exact"}, at each ballot-box with mismatch D'Hont is applied to add or remove the necessary voters from (\code{W}) so that its total match the total number of votes (\code{X}); if method is "\code{mvn_pdf}", "\code{mvn_cdf}" or "\code{mcmc}", the number of voters (\code{W}) of the ballot-box with mismatch is scaled to match its total number of votes (\code{X}).
}
\examples{
\donttest{
# Example 1: Using an 'eim' object directly
simulations <- simulate_election(
    num_ballots = 200,
    num_candidates = 5,
    num_groups = 3,
)

model <- eim(X = simulations$X, W = simulations$W)

model <- bootstrap(
    object = model,
    nboot = 30,
    method = "mult",
    maxiter = 500,
    verbose = FALSE,
)

# Access standard deviation throughout 'model'
print(model$sd)


# Example 2: Providing 'X' and 'W' matrices directly
model <- bootstrap(
    X = simulations$X,
    W = simulations$W,
    nboot = 15,
    method = "mvn_pdf",
    maxiter = 100,
    maxtime = 5,
    param_threshold = 0.01,
    allow_mismatch = FALSE
)

print(model$sd)
}

# Example 3: Using a JSON file as input

\dontrun{
model <- bootstrap(
    json_path = "path/to/election_data.json",
    nboot = 70,
    method = "mult",
)

print(model$sd)
}

}
\seealso{
The \link{eim} object and \link{run_em} implementation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{chile_election_2021}
\alias{chile_election_2021}
\title{Chilean 2021 First Round Presidential Election}
\format{
A data frame with 46,639 rows and 14 variables:
\describe{
\item{\code{REGION}}{The region of the \code{ELECTORAL.DISTRICT}}
\item{\code{ELECTORAL.DISTRICT}}{The electoral circumscription of the ballot box.}
\item{\code{BALLOT.BOX}}{An identifier for the ballot box within the \code{ELECTORAL.DISTRICT}.}
\item{\code{C1}}{The number of votes cast for the candidate \emph{Gabriel Boric}.}
\item{\code{C2}}{The number of votes cast for the candidate \emph{José Antonio Kast}.}
\item{\code{C3}}{The number of votes cast for the candidate \emph{Yasna Provoste}.}
\item{\code{C4}}{The number of votes cast for the candidate \emph{Sebastián Sichel}.}
\item{\code{C5}}{The number of votes cast for the candidate \emph{Eduardo Artés}.}
\item{\code{C6}}{The number of votes cast for the candidate \emph{Marco Enríquez-Ominami}.}
\item{\code{C7}}{The number of votes cast for the candidate \emph{Franco Parisi}.}
\item{\code{BLANK.VOTES}}{The number of blank votes.}
\item{\code{NULL.VOTES}}{The number of null votes.}
\item{\code{X18.19}}{Number of voters aged 18--19.}
\item{\code{X20.29}}{Number of voters aged 20--29.}
\item{\code{X30.39}}{Number of voters aged 30--39.}
\item{\code{X40.49}}{Number of voters aged 40--49.}
\item{\code{X50.59}}{Number of voters aged 50--59.}
\item{\code{X60.69}}{Number of voters aged 60--69.}
\item{\code{X70.79}}{Number of voters aged 70--79.}
\item{\code{X80.}}{Number of voters aged 80 and older.}
\item{\code{MISMATCH}}{Boolean that takes value \code{TRUE} if the ballot-box has a mismatch between the total number of votes and the total number of voters. If this is not the case, its value is \code{FALSE}.}
\item{\code{F}}{Number of female voters in the ballot box.}
\item{\code{M}}{Number of male voters in the ballot box.}
}
}
\source{
\href{https://www.servel.cl/}{Chilean Electoral Service (SERVEL)}
}
\usage{
data("chile_election_2021")
}
\description{
This dataset contains the results of the first round of the 2021 Chilean presidential elections. It provides
9 possible voting options (7 candidates, blank, and null). Each ballot-box is identified by its id (\code{BALLOT.BOX}) and an electoral circumscription
(\code{ELECTORAL.DISTRICT}). Additionally, it provides demographic information on voters' age range for each ballot box.
}
\examples{
data("chile_election_2021")
head(chile_election_2021)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eim-class.R
\name{eim}
\alias{eim}
\alias{eim()}
\title{S3 Object for the Expectation-Maximization Algorithm}
\usage{
eim(X = NULL, W = NULL, json_path = NULL)
}
\arguments{
\item{X}{A \verb{(b x c)} matrix representing candidate votes per ballot box.}

\item{W}{A \verb{(b x g)} matrix representing group votes per ballot box.}

\item{json_path}{A path to a JSON file containing \code{X} and \code{W} fields, stored as nested arrays. It may contain additional fields with other attributes, which will be added to the returned object.}
}
\value{
A list of class \code{eim} containing:
\describe{
\item{\code{X}}{The candidate votes matrix \code{(b x c)}.}
\item{\code{W}}{The group votes matrix \code{(b x g)}.}
}
}
\description{
This constructor creates an \code{eim} S3 object, either by using matrices
\code{X} and \code{W} directly or by reading them from a JSON file. Each
\code{eim} object encapsulates the data (votes for candidates and demographic groups) required by the underlying Expectation-Maximization algorithm.
}
\details{
If \code{X} and \code{W} are directly supplied, they must match the
dimensions of ballot boxes \code{(b)}. Alternatively, if \code{json_path} is provided, the function expects
the JSON file to contain elements named \code{"X"} and \code{"W"} under the
top-level object. This two approaches are \strong{mutually exclusable}, yielding an error otherwise.

Internally, this function also initializes the corresponding instance within
the low-level (C-based) API, ensuring the data is correctly registered for
further processing by the EM algorithm.
}
\note{
A way to generate synthetic data for \code{X} and \code{W} is by using the \link{simulate_election} function. See Example 2 below.
}
\section{Methods}{

In addition to this constructor, the "eim" class provides several
S3 methods for common operations. Some of these methods are fully documented,
while others are ommited due to its straightfoward implementantion. The available methods are:

\itemize{
\item \code{\link{run_em}} - Runs the EM algorithm.
\item \code{\link{bootstrap}} - Estimates the standard deviation.
\item \code{\link{save_eim}} - Saves the object to a file.
\item \code{\link{get_agg_proxy}} - Estimates an ideal group aggregation given their standard deviations.
\item \code{\link{get_agg_opt}} - Estimates an ideal group aggregation among all combinations, given the log-likelihood.
\item \code{\link{plot.eim}} - Plots the probability matrix.
\item \code{print.eim} - Print info about the object.
\item \code{summary.eim} - Summarize the object.
\item \code{as.matrix.eim} - Returns the probability matrix.
\item \code{logLik.eim} - Returns the final log-likelihood, or evaluates log-likelihood at a fixed probability matrix.
}
}

\examples{

# Example 1: Create an eim object from a JSON file
\dontrun{
model1 <- eim(json_path = "path/to/file.json")
}

# Example 2: Use simulate_election with optional parameters, then create an eim object
# from matrices

# Simulate data for 500 ballot boxes, 4 candidates and 5 groups
sim_result <- simulate_election(
    num_ballots = 500,
    num_candidates = 3,
    num_groups = 5,
    group_proportions = c(0.2, 0.2, 0.4, 0.1, 0.1)
)

model2 <- eim(X = sim_result$X, W = sim_result$W)

# Example 3: Create an object from a user defined matrix with 8 ballot boxes,
# 2 candidates and 7 groups.

x_mat <- matrix(c(
    57, 90,
    60, 84,
    43, 102,
    72, 71,
    63, 94,
    52, 80,
    60, 72,
    54, 77
), nrow = 8, ncol = 2, byrow = TRUE)

w_mat <- matrix(c(
    10, 15, 25, 21, 10, 40, 26,
    11, 21, 37, 32, 8, 23, 12,
    17, 12, 43, 27, 12, 19, 15,
    20, 18, 25, 15, 22, 17, 26,
    21, 19, 27, 16, 23, 22, 29,
    18, 16, 20, 14, 19, 22, 23,
    10, 15, 21, 18, 20, 16, 32,
    12, 17, 19, 22, 15, 18, 28
), nrow = 8, ncol = 7, byrow = TRUE)

model3 <- eim(X = x_mat, W = w_mat)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eim-class.R
\name{waldtest}
\alias{waldtest}
\title{Performs a matrix-wise Wald test for two eim objects}
\usage{
waldtest(
  object1 = NULL,
  object2 = NULL,
  X1 = NULL,
  W1 = NULL,
  X2 = NULL,
  W2 = NULL,
  nboot = 100,
  seed = NULL,
  alternative = "two.sided",
  ...
)
}
\arguments{
\item{object1}{An \code{eim} object, as returned by \link{eim}.}

\item{object2}{A second \code{eim} object to compare with \code{object}.}

\item{X1}{A \verb{(b x c)} matrix representing candidate votes per ballot box.}

\item{W1}{A \verb{(b x g)} matrix representing group votes per ballot box.}

\item{X2}{A second \verb{(b x c)} matrix to compare with \code{X}.}

\item{W2}{A second \verb{(b x g)} matrix to compare with \code{W}.}

\item{nboot}{Integer specifying how many times to run the EM algorithm per object.}

\item{seed}{An optional integer indicating the random seed for the randomized algorithms. This argument is only applicable if \code{initial_prob = "random"} or \code{method} is either \code{"mcmc"} or \code{"mvn_cdf"}. Aditionally, it sets the random draws of the ballot boxes.}

\item{alternative}{Character string specifying the type of alternative hypothesis to test. Must be one of \code{"two.sided"} (default), \code{"greater"}, or \code{"less"}. If \code{"two.sided"}, the test checks for any difference in estimated probabilities. If \code{"greater"}, it tests whether the first object has systematically higher probabilities than the second. If \code{"less"}, it tests whether the first has systematically lower probabilities.}

\item{...}{Additional arguments passed to \link{bootstrap} and \link{run_em}.}
}
\value{
A list with components:
\itemize{
\item \code{pvals}: a numeric matrix of p-values with the same dimensions as the estimated probability matrices (\code{pvals}) from the input objects.
\item \code{statistic}: a numeric matrix of z-statistics with the same dimensions as the estimated probability matrices (\code{pvals}).
\item \code{eim1} and \code{eim2}: the original \code{eim} objects used for comparison.
}

Each entry in the pvals matrix is the p-value from Wald test between the corresponding
entries of the two estimated probability matrices.
}
\description{
This function compares two \code{eim} objects (or sets of matrices that can be converted to such objects) by computing a Wald test on each component
of their estimated probability matrices. The Wald test is applied using bootstrap-derived standard deviations, and the result is a matrix
of p-values corresponding to each group-candidate combination.
}
\details{
It uses Wald test to analyze if there is a significant difference between the estimated probabilities between a treatment and a control set. The test is performed independently for each component of the probability matrix.

The user must provide either of the following (but not both):
\itemize{
\item Two \code{eim} objects via \code{object1} and \code{object2}, or
\item Four matrices: \code{X1}, \code{W1}, \code{X2}, and \code{W2}, which will be converted into \code{eim} objects internally.
}

The Wald test is computed using the formula:

\deqn{
z_{ij} = \frac{p_{1,ij}-p_{2,ij}}{\sqrt{s_{1,ij}^2+s_{2,ij}^2}}
}
In this expression, \eqn{s_{1,ij}^2} and \eqn{s_{2,ij}^2} represent the bootstrap sample variances for the treatment and control sets, respectively, while \eqn{p_{1,ij}} and \eqn{p_{2,ij}} are the corresponding estimated probability matrices obtained via the EM algorithm.
}
\examples{
sim1 <- simulate_election(num_ballots = 100, num_candidates = 3, num_groups = 5, seed = 123)
sim2 <- simulate_election(num_ballots = 100, num_candidates = 3, num_groups = 5, seed = 124)

result <- waldtest(sim1, sim2, nboot = 100)

# Check which entries are significantly different
which(result$pvals < 0.05, arr.ind = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eim-class.R
\name{save_eim}
\alias{save_eim}
\alias{save_eim()}
\title{Save an \code{eim} object to a file}
\usage{
save_eim(object, filename, ...)
}
\arguments{
\item{object}{An \code{eim} object.}

\item{filename}{A character string specifying the file path, including the desired file extension (\code{.rds}, \code{.json}, or \code{.csv}).}

\item{...}{Additional arguments (currently unused but included for compatibility).}
}
\value{
The function does not return anything explicitly but saves the object to the specified file.
}
\description{
This function saves an \code{eim} object to a specified file format. Supported formats are
\strong{RDS}, \strong{JSON}, and \strong{CSV}. The function dynamically extracts and saves all available
attributes when exporting to JSON. If the \code{prob} field exists, it is saved when using CSV;
otherwise, it yields an error.
}
\details{
\itemize{
\item If the file extension is \strong{RDS}, the entire object is saved using \code{saveRDS()}.
\item If the file extension is \strong{JSON}, all available attributes of the object are stored in JSON format.
\item If the file extension is \strong{CSV}:
\itemize{
\item If the object contains a \code{prob} field, only that field is saved as a CSV.
\item Otherwise, returns an error.
}
}
}
\examples{
\donttest{
model <- eim(X = matrix(1:9, 3, 3), W = matrix(1:9, 3, 3))

model <- run_em(model)

td <- tempdir()
out_rds <- file.path(td, "model_results.rds")
out_json <- file.path(td, "model_results.json")
out_csv <- file.path(td, "model_results.csv")

# Save as RDS
save_eim(model, filename = out_rds)

# Save as JSON
save_eim(model, filename = out_json)

# Save as CSV
save_eim(model, filename = out_csv)

# Remove the files
files <- c(out_rds, out_json, out_csv)
file.remove(files)
}

}
\seealso{
The \link{eim} object implementation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fastei-package.R
\docType{package}
\name{fastei-package}
\alias{fastei}
\alias{fastei-package}
\title{fastei: Methods for "A Fast Ecological Inference Algorithm for the R\eqn{\times}C case"}
\description{
Package that implements the methods of \href{https://papers.ssrn.com/sol3/papers.cfm?abstract_id=4832834}{Thraves, C.,Ubilla, P. and Hermosilla, D. (2024): \emph{"A Fast Ecological Inference Algorithm for the R×C Case"}.}
}
\details{
Includes a method (\link{run_em}) to solve the R\eqn{\times}C ecological inference problem using the EM algorithm with different approximation methods for the E-step. Covariates are supported but not required. Standard errors of the estimated probabilities can be computed via bootstrapping (\link{bootstrap}).

It also provides a function that generates synthetic election data (\link{simulate_election}) and a function that imports real election data (\link{chile_election_2021}) from the Chilean first-round presidential election of 2021.

The setting in which the documentation presents the Ecological Inference problem is an election context where for a set of ballot-boxes we observe (i) the votes obtained by each candidate and (ii) the number of voters of each demographic group (for example, these can be defined by age ranges or sex). See \href{https://papers.ssrn.com/sol3/papers.cfm?abstract_id=4832834}{Thraves, C.,Ubilla, P. and Hermosilla, D. (2024): \emph{"A Fast Ecological Inference Algorithm for the R×C Case"}.}

The methods to compute the conditional probabilities of the E-Step included in this package are the following:
\itemize{
\item \strong{Markov Chain Monte Carlo} \code{(mcmc)}: Performs MCMC to sample vote outcomes for each ballot-box consistent with the observed data. This sample is used to estimate the conditional probability of the \emph{E}-Step.
\item \strong{Multivariate Normal PDF} \code{(mvn_pdf)}: Uses the PDF of a Multivariate Normal to approximate the conditional probability.
\item \strong{Multivariate Normal CDF} \code{(mvn_cdf)}: Uses the CDF of a Multivariate Normal to approximate the conditional probability.
\item \strong{Multinomial} \code{(mult)}: A single Multinomial is used to approximate the sum of Multinomial distributions.
\item \strong{Exact} \code{(exact)}: Solves the E-Step exactly using the Total Probability Law, which requires enumerating an exponential number of terms.
}

On average, the \strong{Multinomial} method is the most efficient and precise. Its precision matches the \strong{Exact} method.

The documentation uses the following notation:
\itemize{
\item \code{b}: number of ballot-boxes.
\item \code{g}: number of demographic groups.
\item \code{c}: number of candidates.
\item \code{a}: number of aggregated macro-groups.
}

To learn more about \code{fastei}, please consult the available vignettes:

\preformatted{
browseVignettes("fastei")
}
}
\references{
\href{https://papers.ssrn.com/sol3/papers.cfm?abstract_id=4832834}{Thraves, C., Ubilla, P and Hermosilla D. (2024): \emph{"A Fast Ecological Inference Algorithm for the R×C Case"}.}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://danielhermosilla.github.io/ecological-inference-elections/reference/fastei-package.html}
  \item Report bugs at \url{https://github.com/DanielHermosilla/ecological-inference-elections/issues}
}

}
\author{
\strong{Maintainer}: Daniel Hermosilla \email{daniel.hermosilla.r@ug.uchile.cl}

Authors:
\itemize{
  \item Charles Thraves \email{cthraves@dii.uchile.cl} (\href{https://orcid.org/0000-0001-7491-7477}{ORCID})
  \item Pablo Ubilla (\href{https://orcid.org/0009-0003-9048-1767}{ORCID})
}

}
\concept{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eim-S3.R
\name{plot.eim}
\alias{plot.eim}
\title{Plot estimated probabilities}
\usage{
\method{plot}{eim}(
  x,
  title = "Estimated probabilities",
  legend_title = "Candidates",
  color_scale = NULL,
  min_pct = 3,
  pies_per_row = NULL,
  ...
)
}
\arguments{
\item{x}{An "eim" object.}

\item{title}{Title for the plot.}

\item{legend_title}{Title for the legend.}

\item{color_scale}{A vector of colors or a palette for the candidates.}

\item{min_pct}{Minimum percentage required to display a label.}

\item{pies_per_row}{Number of pie charts to display per row. Defaults to \code{ceiling(sqrt(G))},
where \code{G} is the number of groups.}

\item{...}{Additional arguments that are ignored.}
}
\value{
Returns a \code{ggplot2} object representing the pie charts.
}
\description{
Plots the estimated probabilities as pie charts using \code{ggplot2}, one per row of the probability matrix.
Each slice displays its percentage label.
}
\examples{
\donttest{
sim <- simulate_election(
    num_ballots = 100,
    num_candidates = 4,
    num_groups = 5,
    ballot_voters = rep(40, 100),
    seed = 42
)
fit <- run_em(sim, maxiter = 5)

plot(fit, title = "Estimated probabilities", legend_title = "Candidates", min_pct = 7)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instancer.R
\name{simulate_election}
\alias{simulate_election}
\title{Simulate an Election}
\usage{
simulate_election(
  num_ballots,
  num_candidates,
  num_groups,
  ballot_voters = rep(100, num_ballots),
  lambda = 0.5,
  seed = NULL,
  group_proportions = rep(1/num_groups, num_groups),
  prob = NULL
)
}
\arguments{
\item{num_ballots}{Number of ballot boxes (\code{b}).}

\item{num_candidates}{Number of candidates (\code{c}).}

\item{num_groups}{Number of demographic groups (\code{g}).}

\item{ballot_voters}{A vector of length \code{num_ballots} representing the number of voters per ballot
box. Defaults to \code{rep(100, num_ballots)}.}

\item{lambda}{A numeric value between 0 and 1 that represents the fraction of voters that are randomly assigned to ballot-boxes. The remaining voters are assigned sequentially according to their demographic group.
\itemize{
\item \code{lambda = 0}: The assignment of voters to ballot-boxes is fully sequential in terms of their demographic group. This leads to a \strong{high heterogeneity} of the voters' groups across ballot-boxes.
\item \code{lambda = 1}: The assignment of voters to ballot-boxes is fully random. This leads to a \strong{low heterogeneity} of the voters' group across ballot-boxes.
}

Default value is set to \code{0.5}. See \strong{Shuffling Mechanish} for more details.}

\item{seed}{If provided, overrides the current global seed. Defaults to \code{NULL}.}

\item{group_proportions}{Optional. A vector of length \code{num_groups} that indicates the fraction of voters that belong to each group. Default is that all groups are of the same size.}

\item{prob}{Optional. A user-supplied probability matrix of dimension \verb{(g x c)}.
If provided, this matrix is used as the underlying voting probability distribution. If not supplied, each row is sampled from a Dirichlet distribution with each parameter set to one.}
}

\value{
An eim object containing:
\describe{
\item{\code{X}}{A \code{(b x c)} matrix with candidates' votes for each ballot box.}
\item{\code{W}}{A \code{(b x g)} matrix with voters' groups for each ballot-box.}
\item{\code{real_prob}}{A \code{(g x c)} matrix with the probability that a voter from each group votes for each candidate. If prob is provided, it would equal such probability.}
\item{\code{outcome}}{A \code{(b x g x c)} array with the number of votes for each candidate in each ballot box, broken down by group.}
}
}
\description{
This function simulates an election by creating matrices representing candidate votes \code{(X)} and voters' demographic group \code{(W)} across a specified number of ballot-boxes. It either (i) receives as input or (ii) generates a probability matrix \code{(prob)}, indicating how likely each demographic group is to vote for each candidate.

By default, the number of voters per ballot box \code{(ballot_voters)} is set to a vector of 100 with
length \code{num_ballots}. You can optionally override this by providing a custom vector.

Optional parameters are available to control the distribution of votes:
\itemize{
\item \strong{\code{group_proportions}}: A vector of length \code{num_groups} specifying
the overall proportion of each demographic group. Entries must sum to one and be non-negative.
\item \strong{\code{prob}}: A user-supplied probability matrix of dimension
(\code{num_groups} \eqn{\times} \code{num_candidates}). If provided, this matrix is used directly. Otherwise, voting probabilities for each group are drawn from a Dirichlet distribution.
}
}
\section{Shuffling Mechanism}{

Without loss of generality, consider an order relation of groups and ballot-boxes. The shuffling step is controlled by the \code{lambda} parameter and operates as follows:
\enumerate{
\item \strong{Initial Assignment}: Voters are assigned to each ballot-box sequentially according to their demographic
group. More specifically, the first ballot-boxes receive voters from the first group. Then, the next ballot-boxes receive voters from the second group. This continues until all voters have been assigned. Note that most ballot-boxes will contain voters from a single group (as long as the number of ballot-boxes exceeds the number of groups).
\item \strong{Shuffling}: A fraction \code{lambda} of voters who have already been assigned is selected at random. Then, the ballot-box assignment of this sample is shuffled. Hence, different \code{lambda} values are interpreted as follows:
\itemize{
\item \code{lambda = 0} means no one is shuffled (the initial assignment remains).
\item \code{lambda = 1} means all individuals are shuffled.
\item Intermediate values like \code{lambda = 0.5} shuffle half the voters.
}
}

Using a high level of \code{lambda} (greater than 0.7) is not recommended, as this could make identification of the voting probabilities difficult. This is because higher values of lambda induce similar ballot-boxes in terms of voters' group.
}

\examples{
# Example 1: Default usage with 200 ballot boxes, each having 100 voters
result1 <- simulate_election(
    num_ballots = 200,
    num_candidates = 3,
    num_groups = 5
)

# Example 2: Using a custom ballot_voters vector
result2 <- simulate_election(
    num_ballots = 340,
    num_candidates = 3,
    num_groups = 7,
    ballot_voters = rep(200, 340)
)

# Example 3: Supplying group_proportions
result3 <- simulate_election(
    num_ballots = 93,
    num_candidates = 3,
    num_groups = 4,
    group_proportions = c(0.3, 0.5, 0.1, 0.1)
)

# Example 4: Providing a user-defined prob matrix
custom_prob <- matrix(c(
    0.9,  0.1,
    0.4,  0.6,
    0.25, 0.75,
    0.32, 0.68,
    0.2,  0.8
), nrow = 5, byrow = TRUE)

result4 <- simulate_election(
    num_ballots = 200,
    num_candidates = 2,
    num_groups = 5,
    lambda = 0.3,
    prob = custom_prob
)

result4$real_prob == custom_prob
# The attribute of the output real_prob matches the input custom_prob.
}
\references{
The algorithm is fully explained in \href{https://papers.ssrn.com/sol3/papers.cfm?abstract_id=4832834}{'Thraves, C. Ubilla, P. and Hermosilla, D.: \emph{"A Fast Ecological Inference Algorithm for the R×C Case"}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eim-class.R
\name{run_em}
\alias{run_em}
\alias{run_em()}
\title{Compute the Expected-Maximization Algorithm}
\usage{
run_em(
  object = NULL,
  X = NULL,
  W = NULL,
  json_path = NULL,
  method = "mult",
  initial_prob = "group_proportional",
  allow_mismatch = TRUE,
  maxiter = 1000,
  miniter = 0,
  maxtime = 3600,
  param_threshold = 0.001,
  ll_threshold = as.double(-Inf),
  compute_ll = TRUE,
  seed = NULL,
  verbose = FALSE,
  group_agg = NULL,
  mcmc_samples = 1000,
  mcmc_stepsize = 3000,
  mvncdf_method = "genz",
  mvncdf_error = 0.001,
  mvncdf_samples = 5000,
  adjust_prob_cond_method = "project_lp",
  adjust_prob_cond_every = FALSE,
  scale_factor = 1,
  symmetric = FALSE,
  symmetric_weight_method = "joint",
  ...
)
}
\arguments{
\item{object}{An object of class \code{eim}, which can be created using the \link{eim} function. This parameter should not be used if either (i) \code{X} and \code{W} matrices or (ii) \code{json_path} is supplied. See \strong{Note}.}

\item{X}{A \verb{(b x c)} matrix representing candidate votes per ballot box.}

\item{W}{A \verb{(b x g)} matrix representing group votes per ballot box.}

\item{json_path}{A path to a JSON file containing \code{X} and \code{W} fields, stored as nested arrays. It may contain additional fields with other attributes, which will be added to the returned object.}

\item{method}{An optional string specifying the method used for estimating the E-step. Valid
options are:
\itemize{
\item \code{mult}: The default method, using a single sum of Multinomial distributions.
\item \code{mvn_cdf}: Uses a Multivariate Normal CDF distribution to approximate the conditional probability.
\item \code{mvn_pdf}: Uses a Multivariate Normal PDF distribution to approximate the conditional probability.
\item \code{mcmc}: Uses MCMC to sample vote outcomes. This is used to estimate the conditional probability of the E-step.
\item \code{exact}: Solves the E-step using the Total Probability Law.
}

For a detailed description of each method, see \link{fastei-package} and \strong{References}.}

\item{initial_prob}{An optional string specifying the method used to obtain the initial
probability. Accepted values are:
\itemize{
\item \code{uniform}: Assigns equal probability to every candidate within each group.
\item \code{proportional}: Assigns probabilities to each group based on the proportion of candidates votes.
\item \code{group_proportional}: Computes the probability matrix by taking into account both group and candidate proportions. This is the default method.
\item \code{random}: Use randomized values to fill the probability matrix.
}}

\item{allow_mismatch}{Boolean, if \code{TRUE}, allows a mismatch between the voters and votes for each ballot-box. If \code{FALSE}, throws an error if there is a mismatch. By default it is \code{TRUE}. See \strong{Notes} for more details.}

\item{maxiter}{An optional integer indicating the maximum number of EM iterations.
The default value is \code{1000}.}

\item{miniter}{An optional integer indicating the minimum number of EM iterations. The default value is \code{0}.}

\item{maxtime}{An optional numeric specifying the maximum running time (in seconds) for the
algorithm. This is checked at every iteration of the EM algorithm. The default value is \code{3600}, which corresponds to an hour.}

\item{param_threshold}{An optional numeric value indicating the maximum Frobenius norm of the difference between consecutive probability matrices required to stop iterating. The default value is \code{0.001}. Note that the algorithm will stop if either \code{ll_threshold} \strong{or} \code{param_threshold} is accomplished.}

\item{ll_threshold}{An optional numeric value indicating the minimum difference between consecutive log-likelihood values to stop iterating. The default value is \code{inf}, essentially deactivating
the threshold. Note that the algorithm will stop if either \code{ll_threshold} \strong{or} \code{param_threshold} is accomplished.}

\item{compute_ll}{An optional boolean indicating whether to compute the log-likelihood at each iteration. The default value is \code{TRUE}.}

\item{seed}{An optional integer indicating the random seed for the randomized algorithms. This argument is only applicable if \code{initial_prob = "random"} or \code{method} is either \code{"mcmc"} or \code{"mvn_cdf"}.}

\item{verbose}{An optional boolean indicating whether to print informational messages during the EM
iterations. The default value is \code{FALSE}.}

\item{group_agg}{An optional vector of increasing integers from 1 to the number of columns in \code{W}, specifying how to aggregate groups in \code{W} before running the EM algorithm. Each value represents the highest column index included in each aggregated group. For example, if \code{W} has four columns, \code{group_agg = c(2, 4)} indicates that columns 1 and 2 should be combined into one group, and columns 3 and 4 into another. Defaults to \code{NULL}, in which case no group aggregation is performed.}

\item{mcmc_samples}{An optional integer indicating the number of samples to generate for the
\strong{MCMC} method. This parameter is only relevant when \code{method = "mcmc"}.
The default value is \code{1000}.}

\item{mcmc_stepsize}{An optional integer specifying the step size for the \code{mcmc}
algorithm. This parameter is only applicable when \code{method = "mcmc"} and will
be ignored otherwise. The default value is \code{3000}.}

\item{mvncdf_method}{An optional string specifying the method used to estimate the \code{mvn_cdf} method
via a Monte Carlo simulation. Accepted values are \code{genz} and \code{genz2}, with \code{genz}
set as the default. This parameter is only applicable when \code{method = "mvn_cdf"}. See \strong{References} for more details.}

\item{mvncdf_error}{An optional numeric value defining the error threshold for the Monte Carlo
simulation when estimating the \code{mvn_cdf} method. The default value is \code{1e-6}. This parameter is only relevant
when \code{method = "mvn_cdf"}.}

\item{mvncdf_samples}{An optional integer specifying the number of Monte Carlo
samples for the \code{mvn_cdf} method. The default value is \code{5000}. This argument is only applicable when \code{method = "mvn_cdf"}.}

\item{adjust_prob_cond_method}{An optional string indicating the method to adjust the conditional probability so that for each candidate, the sum product of voters and conditional probabilities across groups equals the votes obtained by the candidate. It can take values: \code{""} if no adjusting is made, \code{"lp"} if the adjustment is based on a linear programming that penalizes with L1-norm, \code{"project_lp"} if the adjustment is performed using projection and linear programming (this is the default)}

\item{adjust_prob_cond_every}{An optional boolean indicating whether to adjust the conditional probability on every iteration (if \code{TRUE}), or only at the conditional probabilities obtained at the end of the EM algorithm (if \code{FALSE}, this is the default). This parameter applies only if \code{adjust_prob_conditional_method} is \code{lp} or \code{project_lp}.}

\item{scale_factor}{An optional numeric value used to scale down the \code{X} and \code{W} matrices before executing the EM algorithm. This scaling can help improve performance when dealing with large vote counts. For example if \code{scale_factor = 2} all elements of \code{X} and \code{W} are divided by two and rounded. The default value is \code{1}, which means no scaling is applied. In case the scaling results in mismatch between \code{W} and \code{X}, ensure that \code{allow_mismatch = TRUE}.}

\item{symmetric}{A boolean indicating whether to perform a symmetric estimation. If \code{TRUE}, the algorithm runs twice: first estimating the probabilities of candidates given groups, and then estimating the probabilities of groups given candidates. The final probabilities are obtained by combining both directions using \code{symmetric_weight_method} (\code{"joint"} by default). The default value is \code{FALSE}.}

\item{symmetric_weight_method}{Character string indicating how to combine both directions when \code{symmetric = TRUE}. Valid values are \code{"joint"} (default behavior: joint EM where forward and reverse E-steps are averaged each iteration using equal weights), \code{"average"} (preserves the historical two-run average), \code{"delta_ll"} (weights based on the relative likelihood gaps \code{tau = (LL - LL_ind) / abs(LL)} and \code{tau_rev = (LL_rev - LL_rev_ind) / abs(LL_rev)}), and \code{"mae_inverse"} (weights based on reconstruction errors \code{tau = sum(abs(X - W \%*\% p))} and \code{tau_rev = sum(abs(W - X \%*\% p_rev))}).}

\item{...}{Added for compability}
}
\value{
The function returns an \code{eim} object with the function arguments and the following attributes:
\describe{
\item{prob}{
The estimated global probability matrix \verb{(g x c)}.
}
\item{cond_prob}{
A \verb{(g x c x b)} 3d-array with the probability that at each ballot-box a voter of each group voted for each candidate,
given the observed outcome at the particular ballot-box.
}
\item{expected_outcome}{
A \verb{(g x c x b)} 3d-array with the expected votes cast for each ballot-box.
}
\item{logLik}{The log-likelihood value from the last iteration.}
\item{iterations}{The total number of iterations performed by the EM algorithm.}
\item{time}{The total execution time of the algorithm in seconds.}
\item{status}{
The final status ID of the algorithm upon completion:
\itemize{
\item \code{0}: Converged
\item \code{1}: Maximum time reached.
\item \code{2}: Maximum iterations reached.
}
}
\item{message}{The finishing status displayed as a message, matching the status ID value.}
\item{method}{The method for estimating the conditional probability in the E-step.}
}
Aditionally, it will create \code{mcmc_samples} and \code{mcmc_stepsize} parameters if the specified \code{method = "mcmc"}, or \code{mvncdf_method}, \code{mvncdf_error} and \code{mvncdf_samples} if \code{method = "mvn_cdf"}.

Also, if the eim object supplied is created with the function \link{simulate_election}, it also returns the real probability and unobserved votes with the name \code{real_prob} and \code{outcome} respectively. See \link{simulate_election}.

If \code{group_agg} is different than \code{NULL}, two values are returned: \code{W_agg} a \verb{(b x a)} matrix with the number of voters of each aggregated group o each ballot-box, and \code{group_agg} the same input vector.

Furthermore, if \code{symmetric = TRUE}, the following additional attributes are included:
\describe{
\item{prob_inv}{The estimated probability matrix \verb{(c x g)}, obtained by swapping \code{X} and \code{W}, when the symmetric scheme performs an explicit reverse run (for example \code{"average"}, \code{"delta_ll"}, and \code{"mae_inverse"}).}
\item{cond_prob_inv}{A \verb{(c x g x b)} 3d-array with the probability that at each ballot-box a voter of each candidate voted for each group, given the observed outcome at the particular ballot-box, when the symmetric scheme performs an explicit reverse run.}
}
If \code{symmetric_weight_method = "delta_ll"} and both \code{logLik} values are available, the object also includes \code{LL_ind}, \code{LL_rev_ind}, \code{dLL}, \code{dLL_rev}, \code{nu}, \code{nu_rev}, and \code{symmetric_weights}.
If \code{symmetric_weight_method = "mae_inverse"}, the object also includes \code{err_forward}, \code{err_inverse}, and \code{symmetric_weights}.
Under this argument, the conditional probabilities will be obtained by dividing new expected outcomes by \code{W}. The probabilities will be calculated by performing an M-step.
}
\description{
Executes the Expectation-Maximization (EM) algorithm indicating the approximation method to use in the E-step.
Certain methods may require additional arguments, which can be passed through \code{...} (see \link{fastei-package} for more details).
}
\note{
This function can be executed using one of three mutually exclusive approaches:
\enumerate{
\item By providing an existing \code{eim} object.
\item By supplying both input matrices (\code{X} and \code{W}) directly.
\item By specifying a JSON file (\code{json_path}) containing the matrices.
}

These input methods are \strong{mutually exclusive}, meaning that you must provide \strong{exactly one} of
these options. Attempting to provide more than one or none of these inputs will result in an error.

When called with an \code{eim} object, the function updates the object with the computed results.
If an \code{eim} object is not provided, the function will create one internally using either the
supplied matrices or the data from the JSON file before executing the algorithm.

If there are ballot-boxes with mismatch between \code{W} and \code{X}, and \code{allow_mismatch = TRUE}, then: if \code{method = "exact"}, at each ballot-box with mismatch D'Hont is applied to add or remove the necessary voters from (\code{W}) so that its total match the total number of votes (\code{X}); if method is "\code{mvn_pdf}", "\code{mvn_cdf}" or "\code{mcmc}", the number of voters (\code{W}) of the ballot-box with mismatch is scaled to match its total number of votes (\code{X}).
}
\examples{
\donttest{
# Example 1: Compute the Expected-Maximization with default settings
simulations <- simulate_election(
    num_ballots = 300,
    num_candidates = 5,
    num_groups = 3,
)
model <- eim(simulations$X, simulations$W)
model <- run_em(model) # Returns the object with updated attributes

# Example 2: Compute the Expected-Maximization using the mvn_pdf method
model <- run_em(
    object = model,
    method = "mvn_pdf",
)

# Example 3: Run the mvn_cdf method with default settings
model <- run_em(object = model, method = "mvn_cdf")
}
\dontrun{
# Example 4: Perform an Exact estimation using user-defined parameters

run_em(
    json_path = "a/json/file.json",
    method = "exact",
    initial_prob = "uniform",
    maxiter = 10,
    maxtime = 600,
    param_threshold = 1e-3,
    ll_threshold = 1e-5,
    verbose = TRUE
)
}

}
\references{
\href{https://papers.ssrn.com/sol3/papers.cfm?abstract_id=4832834}{Thraves, C., Ubilla, P. and Hermosilla, D.: \emph{"Fast Ecological Inference Algorithm for the RxC Case"}}. Aditionally, the MVN CDF is computed by the methods introduced in \href{https://www.researchgate.net/publication/2463953_Numerical_Computation_Of_Multivariate_Normal_Probabilities}{Genz, A. (2000). Numerical computation of multivariate normal probabilities. \emph{Journal of Computational and Graphical Statistics}}
}
\seealso{
The \link{eim} object implementation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-utils.R
\name{get_eim_chile}
\alias{get_eim_chile}
\alias{get_eim_chile()}
\title{Extracts voting and demographic data matrices for a given electoral district in Chile.}
\usage{
get_eim_chile(
  elect_district = NULL,
  region = NULL,
  merge_blank_null = TRUE,
  remove_mismatch = FALSE,
  use_sex = FALSE
)
}
\arguments{
\item{elect_district}{A string indicating the name of the electoral district to extract (e.g., "NIEBLA"). See \strong{Note}.}

\item{region}{A string indicating the name of the region to extract (e.g, "DE TARAPACA"). See \strong{Note}.}

\item{merge_blank_null}{Logical indicating whether blank and null votes should be merged into a single column. Defaults to \code{TRUE}.}

\item{remove_mismatch}{Logical indicating whether to remove ballot boxes with mismatched vote totals (where \code{MISMATCH == TRUE}). Defaults to \code{FALSE}.}

\item{use_sex}{Logical indicating whether to use the sex from the voters instead of the age ranges. Defaults to \code{FALSE}.}
}
\value{
An \code{\link{eim}} object with the following attributes:
\itemize{
\item \strong{X}: A matrix \verb{(b x c)} with the number of votes per candidate (including a column for blank + null votes if \code{merge_blank_null = TRUE}).
\item \strong{W}: A matrix \verb{(b x g)} with the number of voters per group (e.g., age ranges) for each ballot box.
}

This object can be passed to functions like \code{\link{run_em}} or \code{\link{get_agg_proxy}} for estimation and group aggregation. See \strong{Example}.
}
\description{
This function retrieves the voting results and demographic group counts for a given electoral district from the 2021 Chilean election dataset included in this package. The function returns an \code{\link{eim}} object that can be directly used in \code{\link{run_em}} or other estimation functions.
}
\details{
The function builds the \code{X} matrix using the number of votes per candidate, and the \code{W} matrix using the number of voters in each demographic group (e.g., age ranges). Optionally, blank and null votes can be merged into a single additional column (considered as another candidate).

Additionally, ballot boxes where the number of votes does not match the number of registered voters (i.e., those where \code{MISMATCH == TRUE}) can be excluded from the dataset by setting \code{remove_mismatch = TRUE}.
}
\note{
Only one parameter is accepted among \code{elect_district} and \code{region}. If either both parameters are given, it will return an error. If neither of these two inputs is supplied, it will return an \code{eim} object with an aggregation corresponding to the whole dataset. To see all electoral districts and regions names, see the function \link{chile_election_2021}.
}
\examples{
# Load data and create an eim object for the electoral district of "NIEBLA"
eim_obj <- get_eim_chile(elect_district = "NIEBLA", remove_mismatch = FALSE)

# Use it to run the EM algorithm
result <- run_em(eim_obj, allow_mismatch = TRUE)

# Use it with group aggregation
agg_result <- get_agg_proxy(
    object = eim_obj,
    sd_threshold = 0.05,
    allow_mismatch = TRUE,
    seed = 123
)

agg_result$group_agg

}
\seealso{
\link{chile_election_2021}
}
